@@ -18,7 +18,8 @@
 // This header file is intended to be the future home of xchg, cmpxchg, xadd, and
 // other threading-related exchange instructions.
 
-namespace x86Emitter {
+namespace x86Emitter
+{
 
 
-}	// End namespace x86Emitter
+} // End namespace x86Emitter
