@@ -31,7 +31,7 @@ EOF
 
 # Default value
 GIT_SHA1=0;
-BRANCH="1.4.x"
+BRANCH="1.6.x"
 while [ -n "$1" ]; do
 case $1 in
     -help|-h   ) help;shift 1;;
@@ -83,7 +83,6 @@ remove_3rdparty()
 {
     echo "Remove 3rdparty code"
     rm -fr $LOCAL_REPO/3rdparty
-    rm -fr $LOCAL_REPO/fps2bios
     rm -fr $LOCAL_REPO/tools
 }
 
@@ -91,7 +90,7 @@ remove_not_yet_free_plugin()
 {
     echo "Remove non free plugins"
     # remove also deprecated plugins
-    for plugin in CDVDiso CDVDisoEFP CDVDlinuz CDVDolio CDVDpeops dev9ghzdrk PeopsSPU2 SSSPSXPAD USBqemu xpad zerogs zerospu2
+    for plugin in USBqemu zerogs zerospu2
     do
         rm -fr $LOCAL_REPO/plugins/$plugin
     done
@@ -100,7 +99,6 @@ remove_not_yet_free_plugin()
 remove_remaining_non_free_file()
 {
     echo "Remove remaining non free file. TODO UPSTREAM"
-    rm -fr $LOCAL_REPO/unfree
     rm -fr $LOCAL_REPO/plugins/GSdx/baseclasses
     rm -f  $LOCAL_REPO/plugins/zzogl-pg/opengl/Win32/aviUtil.h
     rm -f  $LOCAL_REPO/common/src/Utilities/x86/MemcpyFast.cpp
