@@ -82,15 +82,51 @@ u8 Test20[] = { 0x98, 0x06, 0x10, 0xC3 };
 u8 Test22[] = { 0x66, 0x6F, 0x72, 0x20, 0x45, 0x75, 0x72, 0x6F };
 u8 Test23[] = { 0x43, 0x58, 0x44, 0x32, 0x39 ,0x34, 0x30, 0x51 };
 
+//backported from PCSXR 
+// cdr.Stat:
+#define NoIntr		0
+#define DataReady	1
+#define Complete	2
+#define Acknowledge	3
+#define DataEnd		4
+#define DiskError	5
+
+/* Modes flags */
+#define MODE_SPEED       (1<<7) // 0x80
+#define MODE_STRSND      (1<<6) // 0x40 ADPCM on/off
+#define MODE_SIZE_2340   (1<<5) // 0x20
+#define MODE_SIZE_2328   (1<<4) // 0x10
+#define MODE_SIZE_2048   (0<<4) // 0x00
+#define MODE_SF          (1<<3) // 0x08 channel on/off
+#define MODE_REPORT      (1<<2) // 0x04
+#define MODE_AUTOPAUSE   (1<<1) // 0x02
+#define MODE_CDDA        (1<<0) // 0x01
+
+/* Status flags, to go on cdr.StatP */
+#define STATUS_PLAY      (1<<7) // 0x80
+#define STATUS_SEEK      (1<<6) // 0x40
+#define STATUS_READ      (1<<5) // 0x20
+#define STATUS_SHELLOPEN (1<<4) // 0x10
+#define STATUS_UNKNOWN3  (1<<3) // 0x08
+#define STATUS_UNKNOWN2  (1<<2) // 0x04
+#define STATUS_ROTATING  (1<<1) // 0x02
+#define STATUS_ERROR     (1<<0) // 0x01
+
+/* Errors */
+#define ERROR_NOTREADY   (1<<7) // 0x80
+#define ERROR_INVALIDCMD (1<<6) // 0x40
+#define ERROR_INVALIDARG (1<<5) // 0x20
+
 // 1x = 75 sectors per second
 // PSXCLK = 1 sec in the ps
 // so (PSXCLK / 75) / BIAS = cdr read time (linuzappz)
-//#define cdReadTime ((PSXCLK / 75) / BIAS)
-u32 cdReadTime;// = ((PSXCLK / 75) / BIAS);
+u32 cdReadTime; // = ((PSXCLK / 75) / BIAS);
 
 #define CDR_INT(eCycle)    PSX_INT(IopEvt_Cdrom, eCycle)
 #define CDREAD_INT(eCycle) PSX_INT(IopEvt_CdromRead, eCycle)
 
+const uint shortSectorSeekReadDelay = 1000; // delay for reads/seeks that may or may not have a seek action preceeding it
+uint sectorSeekReadDelay = 0x800; // for calculated seek delays
 
 static void AddIrqQueue(u8 irq, u32 ecycle);
 
@@ -98,7 +134,9 @@ static __fi void StartReading(u32 type) {
    	cdr.Reading = type;
   	cdr.FirstSector = 1;
   	cdr.Readed = 0xff;
-	AddIrqQueue(READ_ACK, 0x800);
+	//DevCon.Warning("ReadN/ReadS delay: %d", sectorSeekReadDelay);
+	AddIrqQueue(READ_ACK, sectorSeekReadDelay);
+	sectorSeekReadDelay = shortSectorSeekReadDelay;
 }
 
 static __fi void StopReading() {
@@ -110,7 +148,7 @@ static __fi void StopReading() {
 
 static __fi void StopCdda() {
 	if (cdr.Play) {
-		cdr.StatP&=~0x80;
+		cdr.StatP &= ~STATUS_PLAY;
 		cdr.Play = 0;
 	}
 }
@@ -127,20 +165,11 @@ static void ReadTrack() {
 	cdr.Prev[2] = itob(cdr.SetSector[2]);
 
 	CDVD_LOG("KEY *** %x:%x:%x", cdr.Prev[0], cdr.Prev[1], cdr.Prev[2]);
+	if (EmuConfig.CdvdVerboseReads)
+		DevCon.WriteLn("CD Read Sector %x", msf_to_lsn(cdr.SetSector));
 	cdr.RErr = DoCDVDreadTrack(msf_to_lsn(cdr.SetSector), CDVD_MODE_2340);
 }
 
-// cdr.Stat:
-enum cdr_stat_values
-{
-    NoIntr = 0,
-    DataReady,
-    Complete,
-    Acknowledge,
-    DataEnd,
-    DiskError
-};
-
 static void AddIrqQueue(u8 irq, u32 ecycle) {
 	cdr.Irq = irq;
 	if (cdr.Stat) {
@@ -166,7 +195,7 @@ void  cdrInterrupt() {
 	switch (Irq) {
 		case CdlSync:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			break;
@@ -180,7 +209,7 @@ void  cdrInterrupt() {
 		case CdlSetloc:
 			cdr.CmdProcess = 0;
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			break;
@@ -190,13 +219,13 @@ void  cdrInterrupt() {
 			SetResultSize(1);
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
-			cdr.StatP|= 0x82;
+			cdr.StatP |= STATUS_ROTATING | STATUS_PLAY;
 			break;
 
 		case CdlForward:
 			cdr.CmdProcess = 0;
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
 			break;
@@ -204,7 +233,7 @@ void  cdrInterrupt() {
 		case CdlBackward:
 			cdr.CmdProcess = 0;
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
 			break;
@@ -212,7 +241,7 @@ void  cdrInterrupt() {
 		case CdlStandby:
 			cdr.CmdProcess = 0;
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
 			break;
@@ -220,7 +249,7 @@ void  cdrInterrupt() {
 		case CdlStop:
 			cdr.CmdProcess = 0;
 			SetResultSize(1);
-			cdr.StatP&=~0x2;
+			cdr.StatP &= ~STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
         		// cdr.Stat = Acknowledge;
@@ -235,15 +264,15 @@ void  cdrInterrupt() {
 
 		case CdlPause + 0x20:
 			SetResultSize(1);
-			cdr.StatP&=~0x20;
-			cdr.StatP|= 0x2;
+			cdr.StatP &= ~STATUS_READ;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
 			break;
 
 		case CdlInit:
 			SetResultSize(1);
-			cdr.StatP = 0x2;
+			cdr.StatP = STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			AddIrqQueue(CdlInit + 0x20, 0x800);
@@ -258,35 +287,35 @@ void  cdrInterrupt() {
 
 		case CdlMute:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			break;
 
 		case CdlDemute:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			break;
 
 		case CdlSetfilter:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			break;
 
 		case CdlSetmode:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			break;
 
 		case CdlGetmode:
 			SetResultSize(6);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Result[1] = cdr.Mode;
 			cdr.Result[2] = cdr.File;
@@ -319,7 +348,7 @@ void  cdrInterrupt() {
 		case CdlGetTN:
 			cdr.CmdProcess = 0;
 			SetResultSize(3);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			if (CDVD->getTN(&cdr.ResultTN) == -1) {
 				cdr.Stat = DiskError;
@@ -335,7 +364,7 @@ void  cdrInterrupt() {
 			cdr.CmdProcess = 0;
 			cdr.Track = btoi(cdr.Param[0]);
 			SetResultSize(4);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			if (CDVD->getTD(cdr.Track, &trackInfo) == -1) {
 				cdr.Stat = DiskError;
 				cdr.Result[0]|= 0x01;
@@ -351,7 +380,7 @@ void  cdrInterrupt() {
 
 		case CdlSeekL:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			AddIrqQueue(CdlSeekL + 0x20, 0x800);
@@ -359,14 +388,14 @@ void  cdrInterrupt() {
 
 		case CdlSeekL + 0x20:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
 			break;
 
 		case CdlSeekP:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			AddIrqQueue(CdlSeekP + 0x20, 0x800);
@@ -374,7 +403,7 @@ void  cdrInterrupt() {
 
 		case CdlSeekP + 0x20:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
 			break;
@@ -402,7 +431,7 @@ void  cdrInterrupt() {
 
 		case CdlID:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			AddIrqQueue(CdlID + 0x20, 0x800);
@@ -422,14 +451,14 @@ void  cdrInterrupt() {
 
 		case CdlReset:
 			SetResultSize(1);
-			cdr.StatP = 0x2;
+			cdr.StatP = STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			break;
 
 		case CdlReadToc:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 			AddIrqQueue(CdlReadToc + 0x20, 0x800);
@@ -437,7 +466,7 @@ void  cdrInterrupt() {
 
 		case CdlReadToc + 0x20:
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Complete;
 			break;
@@ -451,7 +480,7 @@ void  cdrInterrupt() {
 			if (!cdr.Reading) return;
 
 			SetResultSize(1);
-			cdr.StatP|= 0x2;
+			cdr.StatP |= STATUS_ROTATING;
 			cdr.Result[0] = cdr.StatP;
 			cdr.Stat = Acknowledge;
 
@@ -491,8 +520,8 @@ void  cdrReadInterrupt() {
 	if (!cdr.Reading)
 		return;
 
-	if (cdr.Stat) {
-		CDREAD_INT(0x800);
+	if (cdr.Stat) { // CDR_LOG_I("cdrom: read stat hack %02x %x\n", cdr.Irq, cdr.Stat);
+		CDREAD_INT(0x800 * 4); // * 4 reduces dma3 errors lots here
 		return;
 	}
 
@@ -500,7 +529,7 @@ void  cdrReadInterrupt() {
 
 	cdr.OCUP = 1;
 	SetResultSize(1);
-	cdr.StatP|= 0x22;
+	cdr.StatP |= STATUS_READ|STATUS_ROTATING;
 	cdr.Result[0] = cdr.StatP;
 
 	if( cdr.RErr == 0 )
@@ -515,10 +544,10 @@ void  cdrReadInterrupt() {
 
 	if (cdr.RErr == -1)
 	{
-		CDVD_LOG(" err\n");
+		DevCon.Warning("CD err");
 		memzero(cdr.Transfer);
 		cdr.Stat = DiskError;
-		cdr.Result[0] |= 0x01;
+		cdr.Result[0] |= STATUS_ERROR;
 		ReadTrack();
 		CDREAD_INT((cdr.Mode & 0x80) ? (cdReadTime / 2) : cdReadTime);
 		return;
@@ -542,11 +571,12 @@ void  cdrReadInterrupt() {
 	cdr.Readed = 0;
 
 	if ((cdr.Transfer[4+2] & 0x80) && (cdr.Mode & 0x2)) { // EOF
-		CDVD_LOG("AutoPausing Read");
+		DevCon.Warning("CD AutoPausing Read");
 		AddIrqQueue(CdlPause, 0x800);
 	}
 	else {
 		ReadTrack();
+		//DevCon.Warning("normal: %d",cdReadTime);
 		CDREAD_INT((cdr.Mode & 0x80) ? (cdReadTime / 2) : cdReadTime);
 	}
 
@@ -601,6 +631,15 @@ void cdrWrite0(u8 rt) {
 	}
 }
 
+void setPsxSpeed()
+{
+	// psxmode: trying to get delays right (fix "dma3 not ready" and mdec glitches)
+	// odd.. tests suggest this should be exactly * 2. could it be that the old psx 1x / 2x speeds aren't handled the same on PS2?
+	// used Chrono Cross intro music and see that it doesn't stutter, then use any other FMV game (with sound) and see that it doesn't stall.
+	// result: cdReadTime = ((PSXCLK / 75) / BIAS) * 2; is exactly right
+	cdReadTime = ((PSXCLK / 75) / BIAS) * 2;
+}
+
 u8 cdrRead1(void) {
 	if (cdr.ResultReady && cdr.Ctrl & 0x1) {
 		psxHu8(0x1801) = cdr.Result[cdr.ResultP++];
@@ -620,13 +659,14 @@ void cdrWrite1(u8 rt) {
 	cdr.Cmd = rt;
 	cdr.OCUP = 0;
 
+//#define CDRCMD_DEBUG
 #ifdef CDRCMD_DEBUG
-	SysPrintf("CD1 write: %x (%s)", rt, CmdName[rt]);
+	DevCon.Warning("CD1 write: %x (%s)", rt, CmdName[rt]);
 	if (cdr.ParamC) {
-		SysPrintf(" Param[%d] = {", cdr.ParamC);
-		for (i=0;i<cdr.ParamC;i++) SysPrintf(" %x,", cdr.Param[i]);
-		SysPrintf("}\n");
-	} else SysPrintf("\n");
+		DevCon.Warning(" Param[%d] = {", cdr.ParamC);
+		for (i=0;i<cdr.ParamC;i++) DevCon.Warning(" %x,", cdr.Param[i]);
+		DevCon.Warning("}\n");
+	} else DevCon.Warning("\n");
 #endif
 
 	if (cdr.Ctrl & 0x1) return;
@@ -645,16 +685,28 @@ void cdrWrite1(u8 rt) {
 			break;
 
 		case CdlSetloc:
-			StopReading();
-			for (i=0; i<3; i++) cdr.SetSector[i] = btoi(cdr.Param[i]);
+		{
+			//StopReading();
+			// Setloc is memorizing the wanted target, and marks it as unprocessed, and has no other effect 
+			// (it doesn't start reading or seeking, and doesn't interrupt or redirect any active reads).
+			int oldSector = msf_to_lsn(cdr.SetSector);
+			for (i = 0; i < 3; i++) cdr.SetSector[i] = btoi(cdr.Param[i]);
 			cdr.SetSector[3] = 0;
 			if ((cdr.SetSector[0] | cdr.SetSector[1] | cdr.SetSector[2]) == 0) {
 				*(u32 *)cdr.SetSector = *(u32 *)cdr.SetSectorSeek;
 			}
-			cdr.Ctrl|= 0x80;
+			int newSector = msf_to_lsn(cdr.SetSector);
+			
+			// sectorSeekReadDelay should lead to sensible random seek results in QA (Aging Disk) test
+			sectorSeekReadDelay = abs(newSector - oldSector) * 100;
+			if (sectorSeekReadDelay < shortSectorSeekReadDelay) sectorSeekReadDelay = shortSectorSeekReadDelay;
+			//DevCon.Warning("CdlSetloc sectorSeekReadDelay: %d", sectorSeekReadDelay);
+
+			cdr.Ctrl |= 0x80;
 			cdr.Stat = NoIntr;
-			AddIrqQueue(cdr.Cmd, 0x800);
-			break;
+			AddIrqQueue(cdr.Cmd, 0x800); // the seek delay occurs on the next read / seek command (CdlReadS, CdlSeekL, etc)
+		}
+		break;
 
 		case CdlPlay:
 			cdr.Play = 1;
@@ -783,14 +835,20 @@ void cdrWrite1(u8 rt) {
 			((u32 *)cdr.SetSectorSeek)[0] = ((u32 *)cdr.SetSector)[0];
 			cdr.Ctrl|= 0x80;
 			cdr.Stat = NoIntr;
-			AddIrqQueue(cdr.Cmd, 0x800);
+
+			//DevCon.Warning("CdlSeekL delay: %d", sectorSeekReadDelay);
+			AddIrqQueue(cdr.Cmd, sectorSeekReadDelay);
+			sectorSeekReadDelay = shortSectorSeekReadDelay;
 			break;
 
 		case CdlSeekP:
 			((u32 *)cdr.SetSectorSeek)[0] = ((u32 *)cdr.SetSector)[0];
 			cdr.Ctrl|= 0x80;
 			cdr.Stat = NoIntr;
-			AddIrqQueue(cdr.Cmd, 0x800);
+
+			//DevCon.Warning("CdlSeekP delay: %d", sectorSeekReadDelay);
+			AddIrqQueue(cdr.Cmd, sectorSeekReadDelay);
+			sectorSeekReadDelay = shortSectorSeekReadDelay;
 			break;
 
 		case CdlTest:
@@ -820,7 +878,7 @@ void cdrWrite1(u8 rt) {
 			break;
 
 		default:
-			CDVD_LOG("Unknown Cmd: %x\n", cdr.Cmd);
+			DevCon.Warning("Unknown CD Cmd: %x\n", cdr.Cmd);
 			return;
     }
 	if (cdr.Stat != NoIntr)
@@ -912,7 +970,9 @@ void psxDma3(u32 madr, u32 bcr, u32 chcr) {
 		case 0x11000000:
 		case 0x11400100:
 			if (cdr.Readed == 0) {
-				CDVD_LOG("*** DMA 3 *** NOT READY");
+				DevCon.Warning("*** DMA 3 *** NOT READY");
+				HW_DMA3_CHCR &= ~0x01000000; //hack
+				psxDmaInterrupt(3); //hack
 				return;
 			}
 
@@ -934,38 +994,6 @@ void psxDma3(u32 madr, u32 bcr, u32 chcr) {
 	psxDmaInterrupt(3);
 }
 
-#ifdef ENABLE_NEW_IOPDMA
-s32 CALLBACK cdvdDmaRead(s32 channel, u32* data, u32 bytesLeft, u32* bytesProcessed)
-{
-#ifdef ENABLE_NEW_IOPDMA_CDVD
-	// hacked up from the code above
-
-	if (cdr.Readed == 0)
-	{
-		//CDVD_LOG("*** DMA 3 *** NOT READY");
-		wordsProcessed = 0;
-		return 10000;
-	}
-
-	memcpy(data, cdr.pTransfer, wordsLeft);
-	//psxCpu->Clear(madr, cdsize/4);
-	cdr.pTransfer+=wordsLeft;
-	*wordsProcessed = wordsLeft;
-
-	Console.WriteLn(Color_Black,"New IOP DMA handled CDVD DMA: channel %d, data %p, remaining %08x, processed %08x.", channel,data,wordsLeft, *wordsProcessed);
-#endif
-	return 0;
-}
-
-void CALLBACK cdvdDmaInterrupt(s32 channel)
-{
-#ifdef ENABLE_NEW_IOPDMA_CDVD
-	cdrInterrupt();
-#endif
-}
-
-#endif
-
 void cdrReset() {
 	memzero(cdr);
 	cdr.CurTrack=1;
