@@ -18,6 +18,7 @@
 
 #include "IsoFS.h"
 #include "IsoFile.h"
+#include <memory>
 
 //////////////////////////////////////////////////////////////////////////
 // IsoDirectory
@@ -109,10 +110,6 @@ IsoDirectory::IsoDirectory(SectorSource& r, IsoFileDescriptor directoryEntry)
 	Init(directoryEntry);
 }
 
-IsoDirectory::~IsoDirectory() throw()
-{
-}
-
 void IsoDirectory::Init(const IsoFileDescriptor& directoryEntry)
 {
 	// parse directory sector
@@ -172,7 +169,7 @@ IsoFileDescriptor IsoDirectory::FindFile(const wxString& filePath) const
 	wxFileName parts( filePath, wxPATH_DOS );
 	IsoFileDescriptor info;
 	const IsoDirectory* dir = this;
-	ScopedPtr<IsoDirectory> deleteme;
+	std::unique_ptr<IsoDirectory> deleteme;
 
 	// walk through path ("." and ".." entries are in the directories themselves, so even if the
 	// path included . and/or .., it still works)
@@ -182,7 +179,8 @@ IsoFileDescriptor IsoDirectory::FindFile(const wxString& filePath) const
 		info = dir->GetEntry(parts.GetDirs()[i]);
 		if(info.IsFile()) throw Exception::FileNotFound( filePath );
 
-		dir = deleteme = new IsoDirectory(internalReader, info);
+		deleteme.reset(new IsoDirectory(internalReader, info));
+		dir = deleteme.get();
 	}
 
 	if( !parts.GetFullName().IsEmpty() )
