@@ -17,6 +17,10 @@
 
 #include "x86emitter/tools.h"
 
+// Disable TAS on release builds, threading issue maybe? It hangs the emu.
+// Linux always hangs, Windows far less but it's still an issue.
+#define DISABLE_RECORDING
+
 class IniInterface;
 
 enum PluginsEnum_t
@@ -55,13 +59,29 @@ enum GamefixId
 	Fix_DMABusy,
 	Fix_VIFFIFO,
 	Fix_VIF1Stall,
-	Fix_GIFReverse,
+	Fix_GIFFIFO,
 	Fix_FMVinSoftware,
 	Fix_GoemonTlbMiss,
+	Fix_ScarfaceIbit,
+	Fix_CrashTagTeamIbit,
 
 	GamefixId_COUNT
 };
 
+enum class VsyncMode
+{
+	Off,
+	On,
+	Adaptive,
+};
+
+// Template function for casting enumerations to their underlying type
+template <typename Enumeration>
+typename std::underlying_type<Enumeration>::type enum_cast(Enumeration E)
+{
+	return static_cast<typename std::underlying_type<Enumeration>::type>(E);
+}
+
 ImplementEnumOperators( GamefixId );
 
 //------------ DEFAULT sseMXCSR VALUES ---------------
@@ -280,17 +300,12 @@ struct Pcsx2Config
 		// forces the MTGS to execute tags/tasks in fully blocking/synchronous
 		// style.  Useful for debugging potential bugs in the MTGS pipeline.
 		bool	SynchronousMTGS;
-		bool	DisableOutput;
+
 		int		VsyncQueueSize;
 
-		bool	FrameLimitEnable;
-		bool	FrameSkipEnable;
-		bool	VsyncEnable;
-
-		// The region mode controls the default Maximum/Minimum FPS settings and also
-		// regulates the vsync rates (which in turn control the IOP's SPU2 tick sync and ensure
-		// proper audio playback speed).
-		int		DefaultRegionMode;	// 0=NTSC and 1=PAL
+		bool		FrameLimitEnable;
+		bool		FrameSkipEnable;
+		VsyncMode	VsyncEnable;
 
 		int		FramesToDraw;	// number of consecutive frames (fields) to render
 		int		FramesToSkip;	// number of consecutive frames (fields) to skip
@@ -302,11 +317,12 @@ struct Pcsx2Config
 		GSOptions();
 		void LoadSave( IniInterface& conf );
 
+		int GetVsync() const;
+
 		bool operator ==( const GSOptions& right ) const
 		{
 			return
 				OpEqu( SynchronousMTGS )		&&
-				OpEqu( DisableOutput )			&&
 				OpEqu( VsyncQueueSize )			&&
 				
 				OpEqu( FrameSkipEnable )		&&
@@ -317,7 +333,6 @@ struct Pcsx2Config
 				OpEqu( FramerateNTSC )			&&
 				OpEqu( FrameratePAL )			&&
 
-				OpEqu( DefaultRegionMode )		&&
 				OpEqu( FramesToDraw )			&&
 				OpEqu( FramesToSkip );
 		}
@@ -333,24 +348,26 @@ struct Pcsx2Config
 	// NOTE: The GUI's GameFixes panel is dependent on the order of bits in this structure.
 	struct GamefixOptions
 	{
-		BITFIELD32()
-			bool
-				VuAddSubHack	:1,		// Tri-ace games, they use an encryption algorithm that requires VU ADDI opcode to be bit-accurate.
-				VuClipFlagHack	:1,		// Persona games, maybe others. It's to do with the VU clip flag (again).
-				FpuCompareHack	:1,		// Digimon Rumble Arena 2, fixes spinning/hanging on intro-menu.
-				FpuMulHack		:1,		// Tales of Destiny hangs.
-				FpuNegDivHack	:1,		// Gundam games messed up camera-view.
-				XgKickHack		:1,		// Erementar Gerad, adds more delay to VU XGkick instructions. Corrects the color of some graphics, but breaks Tri-ace games and others.
-				IPUWaitHack     :1,		// FFX FMV, makes GIF flush before doing IPU work. Fixes bad graphics overlay.
-				EETimingHack	:1,		// General purpose timing hack.
-				SkipMPEGHack	:1,		// Skips MPEG videos (Katamari and other games need this)
-				OPHFlagHack		:1,		// Bleach Blade Battlers
-				DMABusyHack		:1,		// Denies writes to the DMAC when it's busy. This is correct behaviour but bad timing can cause problems.
-				VIFFIFOHack		:1,     // Pretends to fill the non-existant VIF FIFO Buffer.
-				VIF1StallHack   :1,     // Like above, processes FIFO data before the stall is allowed (to make sure data goes over).
-				GIFReverseHack  :1,		// Allows PATH3 to continue even if the FIFO is reversed.
-				FMVinSoftwareHack:1,	// Toggle in and out of software rendering when an FMV runs.
-				GoemonTlbHack:1;		// Gomeon tlb miss hack. The game need to access unmapped virtual address. Instead to handle it as exception, tlb are preloaded at startup
+        BITFIELD32()
+        bool
+            VuAddSubHack : 1,           // Tri-ace games, they use an encryption algorithm that requires VU ADDI opcode to be bit-accurate.
+            VuClipFlagHack : 1,         // Persona games, maybe others. It's to do with the VU clip flag (again).
+            FpuCompareHack : 1,         // Digimon Rumble Arena 2, fixes spinning/hanging on intro-menu.
+            FpuMulHack : 1,             // Tales of Destiny hangs.
+            FpuNegDivHack : 1,          // Gundam games messed up camera-view.
+            XgKickHack : 1,             // Erementar Gerad, adds more delay to VU XGkick instructions. Corrects the color of some graphics, but breaks Tri-ace games and others.
+            IPUWaitHack : 1,            // FFX FMV, makes GIF flush before doing IPU work. Fixes bad graphics overlay.
+            EETimingHack : 1,           // General purpose timing hack.
+            SkipMPEGHack : 1,           // Skips MPEG videos (Katamari and other games need this)
+            OPHFlagHack : 1,            // Bleach Blade Battlers
+            DMABusyHack : 1,            // Denies writes to the DMAC when it's busy. This is correct behaviour but bad timing can cause problems.
+            VIFFIFOHack : 1,            // Pretends to fill the non-existant VIF FIFO Buffer.
+            VIF1StallHack : 1,          // Like above, processes FIFO data before the stall is allowed (to make sure data goes over).
+            GIFFIFOHack : 1,            // Enabled the GIF FIFO (more correct but slower)
+            FMVinSoftwareHack : 1,      // Toggle in and out of software rendering when an FMV runs.
+            GoemonTlbHack : 1,          // Gomeon tlb miss hack. The game need to access unmapped virtual address. Instead to handle it as exception, tlb are preloaded at startup
+            ScarfaceIbit : 1,           // Scarface I bit hack. Needed to stop constant VU recompilation
+            CrashTagTeamRacingIbit : 1; // Crash Tag Team Racing I bit hack. Needed to stop constant VU recompilation
 		BITFIELD_END
 
 		GamefixOptions();
@@ -388,7 +405,7 @@ struct Pcsx2Config
 		BITFIELD_END
 
 		s8	EECycleRate;		// EE cycle rate selector (1.0, 1.5, 2.0)
-		u8	VUCycleSteal;		// VU Cycle Stealer factor (0, 1, 2, or 3)
+		u8	EECycleSkip;		// EE Cycle skip factor (0, 1, 2, or 3)
 
 		SpeedhackOptions();
 		void LoadSave( IniInterface& conf );
@@ -396,7 +413,7 @@ struct Pcsx2Config
 
 		bool operator ==( const SpeedhackOptions& right ) const
 		{
-			return OpEqu( bitset ) && OpEqu( EECycleRate ) && OpEqu( VUCycleSteal );
+			return OpEqu( bitset ) && OpEqu( EECycleRate ) && OpEqu( EECycleSkip );
 		}
 
 		bool operator !=( const SpeedhackOptions& right ) const
@@ -410,12 +427,15 @@ struct Pcsx2Config
 		BITFIELD32()
 			bool
 				ShowDebuggerOnStart	:1;
+			bool
+				AlignMemoryWindowStart :1;
 		BITFIELD_END
 
 		u8 FontWidth;
 		u8 FontHeight;
 		u32 WindowWidth;
 		u32 WindowHeight;
+		u32 MemoryViewBytesPerRow;
 
 		DebugOptions();
 		void LoadSave( IniInterface& conf );
@@ -423,7 +443,7 @@ struct Pcsx2Config
 		bool operator ==( const DebugOptions& right ) const
 		{
 			return OpEqu( bitset ) && OpEqu( FontWidth ) && OpEqu( FontHeight )
-				&& OpEqu( WindowWidth ) && OpEqu( WindowHeight );
+				&& OpEqu( WindowWidth ) && OpEqu( WindowHeight ) && OpEqu( MemoryViewBytesPerRow );
 		}
 
 		bool operator !=( const DebugOptions& right ) const
@@ -440,6 +460,9 @@ struct Pcsx2Config
 			EnablePatches		:1,		// enables patch detection and application
 			EnableCheats		:1,		// enables cheat detection and application
 			EnableWideScreenPatches		:1,
+#ifndef DISABLE_RECORDING
+			EnableRecordingTools :1,
+#endif
 		// when enabled uses BOOT2 injection, skipping sony bios splashes
 			UseBOOT2Injection	:1,
 			BackupSavestate		:1,
@@ -529,7 +552,7 @@ TraceLogFilters&				SetTraceConfig();
 #define CHECK_DMABUSYHACK			(EmuConfig.Gamefixes.DMABusyHack)    // Denies writes to the DMAC when it's busy. This is correct behaviour but bad timing can cause problems.
 #define CHECK_VIFFIFOHACK			(EmuConfig.Gamefixes.VIFFIFOHack)    // Pretends to fill the non-existant VIF FIFO Buffer.
 #define CHECK_VIF1STALLHACK			(EmuConfig.Gamefixes.VIF1StallHack)  // Like above, processes FIFO data before the stall is allowed (to make sure data goes over).
-#define CHECK_GIFREVERSEHACK		(EmuConfig.Gamefixes.GIFReverseHack) // Allows PATH3 to continue even if the FIFO is reversed.
+#define CHECK_GIFFIFOHACK			(EmuConfig.Gamefixes.GIFFIFOHack)	 // Enabled the GIF FIFO (more correct but slower)
 #define CHECK_FMVINSOFTWAREHACK	 	(EmuConfig.Gamefixes.FMVinSoftwareHack) // Toggle in and out of software rendering when an FMV runs.
 //------------ Advanced Options!!! ---------------
 #define CHECK_VU_OVERFLOW			(EmuConfig.Cpu.Recompiler.vuOverflow)
@@ -572,12 +595,6 @@ TraceLogFilters&				SetTraceConfig();
 
 #define EE_CONST_PROP 1 // rec2 - enables constant propagation (faster)
 
-// Change to 1 if working on getting PS1 emulation working.
-// This disables the exception normally caused by trying to load PS1
-// games. Note: currently PS1 games will error out even without this
-// commented, so this is for development purposes only.
-#define ENABLE_LOADING_PS1_GAMES 0
-
 // Change to 1 for console logs of SIF, GPU (PS1 mode) and MDEC (PS1 mode).
 // These do spam a lot though!
 #define PSX_EXTRALOGS 0
