@@ -17,6 +17,12 @@
 #include <deque>
 #include "System/SysThreads.h"
 #include "Gif.h"
+#include "Vif.h"
+#include "GS.h"
+
+// FIXME common path ?
+#include "Utilities/boost_spsc_queue.hpp"
+
 struct GS_Packet;
 extern void Gif_MTGS_Wait(bool isMTVU);
 extern void Gif_FinishIRQ();
@@ -74,6 +80,7 @@ struct Gif_Tag {
 		hasAD = false;
 		nRegIdx = 0;
 		isValid = 1;
+		len = 0; // avoid uninitialized compiler warning
 		switch(tag.FLG) {
 			case GIF_FLG_PACKED:
 				nRegs  = ((tag.NREG-1)&0xf) + 1;
@@ -118,11 +125,13 @@ struct Gif_Tag {
 };
 
 struct GS_Packet {
+	// PERF note: this struct is copied various time in hot path. Don't add
+	// new field
+
 	u32  offset;     // Path buffer offset for start of packet
 	u32  size;	     // Full size of GS-Packet
 	s32  cycles;     // EE Cycles taken to process this GS packet
 	s32  readAmount; // Dummy read-amount data needed for proper buffer calculations
-	bool done;	     // 0 = Incomplete, 1 = Complete
 	GS_Packet()  { Reset(); }
 	void Reset() { memzero(*this); }
 };
@@ -146,14 +155,21 @@ static __fi void incTag(u32& offset, u32& size, u32 incAmount) {
 
 struct Gif_Path_MTVU {
 	u32   fakePackets; // Fake packets pending to be sent to MTGS
-	Mutex gsPackMutex; // Used for atomic access to gsPackQueue
-	std::deque<GS_Packet> gsPackQueue; // VU1 programs' XGkick(s)
+	GS_Packet fakePacket;
+	// Set a size based on MTGS but keep a factor 2 to avoid too waste to much
+	// memory overhead. Note the struct is instantied 3 times (for each gif
+	// path)
+	ringbuffer_base<GS_Packet, RingBufferSize / 2> gsPackQueue;
 	Gif_Path_MTVU() { Reset(); }
-	void Reset()    { fakePackets = 0; gsPackQueue.clear(); }
+	void Reset()    { fakePackets = 0;
+		gsPackQueue.reset();
+		fakePacket.Reset();
+		fakePacket.size =~0u; // Used to indicate that its a fake packet
+	}
 };
 
 struct Gif_Path {
-	__aligned(4) volatile s32 readAmount; // Amount of data MTGS still needs to read
+	std::atomic<int> readAmount; // Amount of data MTGS still needs to read
 	u8* buffer;		  // Path packet buffer
 	u32 buffSize;	  // Full size of buffer
 	u32 buffLimit;	  // Cut off limit to wrap around
@@ -166,7 +182,7 @@ struct Gif_Path {
 	GIF_PATH_STATE state; // Path State
 	Gif_Path_MTVU  mtvu;  // Must be last for saved states
 
-	Gif_Path()  {}
+	Gif_Path()  { Reset(); }
 	~Gif_Path() { _aligned_free(buffer); }
 
 	void Init(GIF_PATH _idx, u32 _buffSize, u32 _buffSafeZone) {
@@ -195,7 +211,7 @@ struct Gif_Path {
 	}
 
 	bool isMTVU() const           { return !idx && THREAD_VU1; }
-	s32 getReadAmount()           { return AtomicRead(readAmount) + gsPack.readAmount; }
+	s32 getReadAmount()           { return readAmount.load(std::memory_order_acquire) + gsPack.readAmount; }
 	bool hasDataRemaining() const { return curOffset < curSize; }
 	bool isDone() const           { return isMTVU() ? !mtvu.fakePackets : (!hasDataRemaining() && (state == GIF_PATH_IDLE || state == GIF_PATH_WAIT)); }
 
@@ -251,15 +267,13 @@ struct Gif_Path {
 		curSize     += size;
 	}
 
-	// If completed a GS packet (with EOP) then returned GS_Packet.done = 1
+	// If completed a GS packet (with EOP) then set done to true
 	// MTVU: This function only should be called called on EE thread
-	GS_Packet ExecuteGSPacket() {
+	GS_Packet ExecuteGSPacket(bool &done) {
 		if (mtvu.fakePackets) { // For MTVU mode...
 			mtvu.fakePackets--;
-			GS_Packet fakePack;
-			fakePack.done =  1; // Fake packets don't get processed by pcsx2
-			fakePack.size =~0u; // Used to indicate that its a fake packet
-			return fakePack;
+			done = true;
+			return mtvu.fakePacket;
 		}
 		pxAssert(!isMTVU());
 		for(;;) {
@@ -276,6 +290,7 @@ struct Gif_Path {
 				}
 
 				gifTag.setTag(&buffer[curOffset], 1);
+
 				state = (GIF_PATH_STATE)(gifTag.tag.FLG + 1);
 
 				// We don't have enough data for a complete GS packet
@@ -307,9 +322,8 @@ struct Gif_Path {
 
 			if (gifTag.tag.EOP) {
 				GS_Packet t = gsPack;
-				t.done = 1;
+				done = true;
 
-				
 				dmaRewind = 0;
 				
 				gsPack.Reset();
@@ -378,22 +392,23 @@ struct Gif_Path {
 
 	// MTVU: Gets called after VU1 execution on MTVU thread
 	void FinishGSPacketMTVU() {
-		if (1) {
-			ScopedLock lock(mtvu.gsPackMutex);
-			AtomicExchangeAdd(readAmount, gsPack.size + gsPack.readAmount);
-			mtvu.gsPackQueue.push_back(gsPack);
-		}
+		// Performance note: fetch_add atomic operation might create some stall for atomic
+		// operation in gsPack.push
+		readAmount.fetch_add(gsPack.size + gsPack.readAmount, std::memory_order_acq_rel);
+		while (!mtvu.gsPackQueue.push(gsPack))
+			;
+
 		gsPack.Reset();
 		gsPack.offset = curOffset;
 	}
 
 	// MTVU: Gets called by MTGS thread
 	GS_Packet GetGSPacketMTVU() {
-		ScopedLock lock(mtvu.gsPackMutex);
-		if (mtvu.gsPackQueue.size()) {
-			GS_Packet t = mtvu.gsPackQueue[0];
-			return t; // XGkick GS packet(s)
+		// FIXME is the error path useful ?
+		if (!mtvu.gsPackQueue.empty()) {
+			return mtvu.gsPackQueue.front();
 		}
+
 		Console.Error("MTVU: Expected gsPackQueue to have elements!");
 		pxAssert(0);
 		return GS_Packet(); // gsPack.size will be 0
@@ -401,18 +416,13 @@ struct Gif_Path {
 
 	// MTVU: Gets called by MTGS thread
 	void PopGSPacketMTVU() {
-		ScopedLock lock(mtvu.gsPackMutex);
-		if (mtvu.gsPackQueue.size()) {
-			mtvu.gsPackQueue.pop_front();
-		}
+		mtvu.gsPackQueue.pop();
 	}
 
 	// MTVU: Returns the amount of pending
 	// GS Packets that MTGS hasn't yet processed
 	u32 GetPendingGSPackets() {
-		ScopedLock lock(mtvu.gsPackMutex);
-		u32 t = mtvu.gsPackQueue.size();
-		return t;
+		return mtvu.gsPackQueue.size();
 	}
 };
 
@@ -423,7 +433,7 @@ struct Gif_Unit {
 	tGIF_STAT& stat;
 	GIF_TRANSFER_TYPE lastTranType; // Last Transfer Type
 
-	Gif_Unit() : stat(gifRegs.stat) {
+	Gif_Unit() : gsSIGNAL(), gsFINISH(), stat(gifRegs.stat), lastTranType(GIF_TRANS_INVALID) {
 		gifPath[0].Init(GIF_PATH_1, _1mb*9, _1mb  + _1kb);
 		gifPath[1].Init(GIF_PATH_2, _1mb*9, _1mb  + _1kb);
 		gifPath[2].Init(GIF_PATH_3, _1mb*9, _1mb  + _1kb);
@@ -441,6 +451,11 @@ struct Gif_Unit {
 		if(!softReset) {
 			lastTranType = GIF_TRANS_INVALID;
 		}
+		//If the VIF has paused waiting for PATH3, recheck it after the reset has occurred (Eragon)
+		if (vif1Regs.stat.VGW) {
+			if (!(cpuRegs.interrupt & (1 << DMAC_VIF1)))
+				CPU_INT(DMAC_VIF1, 1);
+		}
 	}
 
 	// Resets Gif HW Regs
@@ -564,8 +579,9 @@ struct Gif_Unit {
 		for(;;) {
 			if (stat.APATH) { // Some Transfer is happening
 				Gif_Path& path   = gifPath[stat.APATH-1];
-				GS_Packet gsPack = path.ExecuteGSPacket();
-				if(!gsPack.done) {
+				bool done = false;
+				GS_Packet gsPack = path.ExecuteGSPacket(done);
+				if(!done) {
 					if (stat.APATH == 3 && CanDoP3Slice() && !gsSIGNAL.queued) {
 						if(!didPath3 && /*!Path3Masked() &&*/ checkPaths(1,1,0)) { // Path3 slicing
 							didPath3 = true;
@@ -592,9 +608,11 @@ struct Gif_Unit {
 					break; // Not finished with GS packet
 				}
 				//DevCon.WriteLn("Adding GS Packet for path %d", stat.APATH);
-				AddCompletedGSPacket(gsPack, (GIF_PATH)(stat.APATH-1));
+				if (gifPath[curPath].state == GIF_PATH_WAIT || gifPath[curPath].state == GIF_PATH_IDLE) {
+					AddCompletedGSPacket(gsPack, (GIF_PATH)(stat.APATH - 1));
+				}
+				
 			}
-
 			if (!gsSIGNAL.queued && !gifPath[0].isDone()) {
 				stat.APATH = 1;
 				stat.P1Q = 0;
@@ -620,11 +638,10 @@ struct Gif_Unit {
 				break;
 			}
 		}
-
 		//Some loaders/Refresh Rate selectors and things dont issue "End of Packet" commands
 		//So we look and see if the end of the last tag is all there, if so, stick it in the buffer for the GS :)
 		//(Invisible Screens on Terminator 3 and Growlanser 2/3)
-		if(gifPath[curPath].curOffset == gifPath[curPath].curSize) 
+		if(gifPath[curPath].curOffset == gifPath[curPath].curSize)
 		{
 			FlushToMTGS();
 		}
@@ -650,18 +667,20 @@ struct Gif_Unit {
 	bool CanDoPath2HL() const {
 		return (stat.APATH == 0 || stat.APATH == 2) && CanDoGif();
 	}
-	// Gif DMA - CHECK_GIFREVERSEHACK is a hack for Hot Wheels.
+	// Gif DMA
 	bool CanDoPath3() const {
 		return((stat.APATH == 0 && !Path3Masked()) || stat.APATH == 3) && CanDoGif();
 	}
 
 	bool CanDoP3Slice()const { return stat.IMT == 1 && gifPath[GIF_PATH_3].state == GIF_PATH_IMAGE; }
-	bool CanDoGif() const    { return stat.PSE == 0 && (CHECK_GIFREVERSEHACK ? 1 : stat.DIR == 0) && gsSIGNAL.queued == 0; }
+	bool CanDoGif() const    { return stat.PSE == 0 && stat.DIR == 0 && gsSIGNAL.queued == 0; }
 	//Mask stops the next packet which hasnt started from transferring
 	bool Path3Masked() const { return ((stat.M3R || stat.M3P) && (gifPath[GIF_PATH_3].state == GIF_PATH_IDLE || gifPath[GIF_PATH_3].state == GIF_PATH_WAIT)); }
 
 	void PrintInfo(bool printP1=1, bool printP2=1, bool printP3=1) {
 		u32 a = checkPaths(1,1,1), b = checkQueued(1,1,1);
+		(void)a; // Don't warn about unused variable
+		(void)b;
 		GUNIT_LOG("Gif Unit - LastTransfer = %s, Paths = [%d,%d,%d], Queued = [%d,%d,%d]",
 				   Gif_TransferStr[(lastTranType>>8)&0xf],
 				   !!(a&1),!!(a&2),!!(a&4),!!(b&1),!!(b&2),!!(b&4));
