@@ -16,9 +16,17 @@
 #include "PrecompiledHeader.h"
 #include "IopCommon.h"
 
+#include "Common.h"
+#include "ConsoleLogger.h"
 #include "Sio.h"
 #include "sio_internal.h"
 
+#ifndef DISABLE_RECORDING
+#	include "Recording/InputRecording.h"
+#	include "Recording/PadData.h"
+#	include "Recording/RecordingInputManager.h"
+#endif
+
 _sio sio;
 _mcd mcds[2][4];
 _mcd *mcd;
@@ -26,13 +34,15 @@ _mcd *mcd;
 SIO_MODE siomode = SIO_START;
 static void sioWrite8inl(u8 data);
 #define SIO_WRITE void inline
+#define SIO_FORCEINLINE __fi
 
 // Magic psx values from nocash info
 static const u8 memcard_psx[] = {0x5A, 0x5D, 0x5C, 0x5D, 0x04, 0x00, 0x00, 0x80};
 
 // Memory Card Specs for standard Sony 8mb carts:
 //    Flags (magic sio '+' thingie!), Sector size, eraseBlockSize (in pages), card size (in pages), xor checksum (superblock?), terminator (unused?).
-static const mc_command_0x26_tag mc_sizeinfo_8mb= {'+', 512, 16, 0x4000, 0x52, 0x5A};
+// FIXME variable commented out since it's not used atm.
+// static const mc_command_0x26_tag mc_sizeinfo_8mb= {'+', 512, 16, 0x4000, 0x52, 0x5A};
 
 // Ejection timeout management belongs in the MemoryCardFile plugin, except the plugin
 // interface is not yet complete.
@@ -53,35 +63,32 @@ wxString GetTimeMsStr(){
 }
 
 //allow timeout also for the mcd manager panel
+void SetForceMcdEjectTimeoutNow( uint port, uint slot )
+{
+	mcds[port][slot].ForceEjection_Timeout = FORCED_MCD_EJECTION_MAX_TRIES;
+}
 void SetForceMcdEjectTimeoutNow()
 {
-	for( u8 port=0; port<2; ++port ) 
-			for( u8 slot=0; slot<4; ++slot )
-				mcds[port][slot].ForceEjection_Timeout = FORCED_MCD_EJECTION_MAX_TRIES;
+	for ( uint port = 0; port < 2; ++port ) {
+		for ( uint slot = 0; slot < 4; ++slot ) {
+			SetForceMcdEjectTimeoutNow( port, slot );
+		}
+	}
 }
 
+void ClearMcdEjectTimeoutNow( uint port, uint slot )
+{
+	mcds[port][slot].ForceEjection_Timeout = 0;
+}
 void ClearMcdEjectTimeoutNow()
 {
-	for( u8 port=0; port<2; ++port ) 
-			for( u8 slot=0; slot<4; ++slot )
-				mcds[port][slot].ForceEjection_Timeout = 0;
+	for ( uint port = 0; port < 2; ++port ) {
+		for ( uint slot = 0; slot < 4; ++slot ) {
+			ClearMcdEjectTimeoutNow( port, slot );
+		}
+	}
 }
 
-// SIO Inline'd IRQs : Calls the SIO interrupt handlers directly instead of
-// feeding them through the IOP's branch test. (see SIO.H for details)
-
-#ifdef SIO_INLINE_IRQS
-#define SIO_INT() sioInterrupt()
-#define SIO_FORCEINLINE __fi
-#else
-__fi void SIO_INT()
-{
-	if( !(psxRegs.interrupt & (1<<IopEvt_SIO)) )
-		PSX_INT(IopEvt_SIO, 64 ); // PSXCLK/250000);
-}
-#define SIO_FORCEINLINE __fi
-#endif
-
 // Currently only check if pad wants mtap to be active.
 // Could lets PCSX2 have its own options, if anyone ever
 // wants to add support for using the extra memcard slots.
@@ -120,11 +127,32 @@ void sioInit()
 	sio.packetsize = 0;
 }
 
-void SIO_FORCEINLINE sioInterrupt()
-{
-	PAD_LOG("Sio Interrupt");
-	sio.StatReg|= IRQ;
-	psxHu32(0x1070)|=0x80;
+bool isR3000ATest = false;
+
+// Check the active game's type, and fire the matching interrupt.
+// The 3rd bit of the HW_IFCG register lets us know if PSX mode is active. 1 = PSX, 0 = PS2
+// Note that the R3000A's call to interrupts only calls the PS2 based (lack of) delays.
+SIO_FORCEINLINE void sioInterrupt() {
+	if ((psxHu32(HW_ICFG) & (1 << 3)) && !isR3000ATest) {
+		if (!(psxRegs.interrupt & (1 << IopEvt_SIO)))
+			PSX_INT(IopEvt_SIO, 64); // PSXCLK/250000);
+	} else {
+		PAD_LOG("Sio Interrupt");
+		sio.StatReg |= IRQ;
+		iopIntcIrq(7); //Should this be used instead of the one below?
+		//psxHu32(0x1070)|=0x80;
+	}
+
+	isR3000ATest = false;
+}
+
+// An offhand way for the R3000A to access the sioInterrupt function.
+// Following the design of the old preprocessor system, the R3000A should
+// never call the PSX delays (oddly enough), and only the PS2. So we need
+// an extra layer here to help control that.
+__fi void sioInterruptR() {
+	isR3000ATest = true;
+	sioInterrupt();
 }
 
 SIO_WRITE sioWriteStart(u8 data)
@@ -152,7 +180,7 @@ SIO_WRITE sioWriteStart(u8 data)
 	case 0x81: siomode = SIO_MEMCARD; break;
 
 	default:
-		DevCon.Warning("%s cmd: %02X??\n", __FUNCTION__, data);
+		DevCon.Warning("%s cmd: %02X??", __FUNCTION__, data);
 		DEVICE_UNPLUGGED();
 		siomode = SIO_DUMMY;
 		break;
@@ -161,22 +189,48 @@ SIO_WRITE sioWriteStart(u8 data)
 	sioWrite8inl(data);
 }
 
+int byteCnt = 0;
+
+//This is only for digital pad! .... a fast hacky fix for testing
+//#define IS_LAST_BYTE_IN_PACKET ((byteCnt >= 3) ? 1 : 0)
+//this should be done on the last byte, but it works like this fine for now... How does one know which number is the last byte anyway (or are there any more following it).
+//maybe in the end a small LUT will be necessary that has the number of bytes for each command...
+//On the real PS1, if the controller doesn't supply an /ACK signal after each byte but the last, the transmission falls through... but it seems it is actually the interrupt
+//what 'continues' a transfer.
+//On the real PS1, asserting /ACK after the last byte would cause the transfer to fail.
+#define IS_LAST_BYTE_IN_PACKET ((sio.bufCount >= 3) ? 1 : 0)
+
 SIO_WRITE sioWriteController(u8 data)
 {
+	//if (data == 0x01) byteCnt = 0;
 	switch(sio.bufCount)
 	{
 	case 0:
+		byteCnt = 0; //hope this gets only cleared on the first byte...
 		SIO_STAT_READY();
 		DEVICE_PLUGGED();
 		sio.buf[0] = PADstartPoll(sio.port + 1);
 		break;
 
-	default: 
+	default:
 		sio.buf[sio.bufCount] = PADpoll(data);
+
+#ifndef DISABLE_RECORDING
+		if (g_Conf->EmuOptions.EnableRecordingTools)
+		{
+			g_InputRecording.ControllerInterrupt(data, sio.port, sio.bufCount, sio.buf);
+			if (g_InputRecording.IsInterruptFrame())
+			{
+				g_RecordingInput.ControllerInterrupt(data, sio.port, sio.bufCount, sio.buf);
+			}
+
+			PadData::LogPadData(sio.port, sio.bufCount, sio.buf);
+		}
+#endif
 		break;
 	}
-
-	SIO_INT();
+	//Console.WriteLn( "SIO: sent = %02X  From pad data =  %02X  bufCnt %08X ", data, sio.buf[sio.bufCount], sio.bufCount);
+	sioInterrupt(); //Don't all commands(transfers) cause an interrupt?
 }
 
 SIO_WRITE sioWriteMultitap(u8 data)
@@ -225,7 +279,7 @@ SIO_WRITE sioWriteMultitap(u8 data)
 			break;
 
 		default:
-			DevCon.Warning("%s cmd: %02X??\n", __FUNCTION__, data);
+			DevCon.Warning("%s cmd: %02X??", __FUNCTION__, data);
 			sio.buf[3] = 0x00;
 			sio.buf[4] = 0x00;
 			sio.buf[5] = 0x00;
@@ -264,7 +318,7 @@ SIO_WRITE sioWriteMultitap(u8 data)
 	//default: sio.buf[sio.bufCount] = 0x00; break;
 	}
 
-	SIO_INT();
+	sioInterrupt();
 }
 
 SIO_WRITE MemcardResponse()
@@ -332,14 +386,14 @@ SIO_WRITE memcardErase(u8 data)
 				break;
 
 			default:
-				DevCon.Warning("%s cmd: %02X??\n", __FUNCTION__, data);
+				DevCon.Warning("%s cmd: %02X??", __FUNCTION__, data);
 				sio.bufCount = -1;
 				//sio.bufSize = 3;
 				//sio.bufCount = 4;
 				break;
 			}
 		}
-		SIO_INT();
+		sioInterrupt();
 		break;
 
 	default:
@@ -393,7 +447,7 @@ SIO_WRITE memcardWrite(u8 data)
 				}
 
 			default:
-				DevCon.Warning("%s cmd: %02X??\n", __FUNCTION__, data);
+				DevCon.Warning("%s cmd: %02X??", __FUNCTION__, data);
 				sio.bufCount = -1;
 				//sio.bufSize = 3;
 				//sio.bufCount = 4;
@@ -401,13 +455,15 @@ SIO_WRITE memcardWrite(u8 data)
 			}
 
 		}
-		SIO_INT();
+		sioInterrupt();
 		break;
 
 	case 2:
 		transfer_size = data;
 
-		sio.buf[data + 5] = mcd->term;
+		// Note: coverity wrongly detects a buffer overflow. Because data + 5 could be > 512...
+		// So let's add a mask-nop to avoid various useless reports.
+		sio.buf[(data & 0xFF) + 5] = mcd->term;
 		sio.bufSize = data + 5;
 		checksum_pos = data + 4;
 		break;
@@ -444,7 +500,8 @@ SIO_WRITE memcardWrite(u8 data)
 
 SIO_WRITE memcardRead(u8 data)
 {
-	static u8 checksum_pos = 0;
+	/*static u8 checksum_pos = 0;*/
+	// psxmode: check if memcard reads need checksum_pos as well as writes (function above!)
 	static u8 transfer_size = 0;
 	static bool once = false;
 
@@ -480,14 +537,14 @@ SIO_WRITE memcardRead(u8 data)
 				}
 
 			default:
-				DevCon.Warning("%s cmd: %02X??\n", __FUNCTION__, data);
+				DevCon.Warning("%s cmd: %02X??", __FUNCTION__, data);
 				sio.bufCount = -1;
 				//sio.bufSize = 3;
 				//sio.bufCount = 4;
 				break;
 			}
 		}
-		SIO_INT();
+		sioInterrupt();
 		break;
 
 	case 2:
@@ -600,7 +657,7 @@ SIO_WRITE sioWriteMemcard(u8 data)
 	case 0:
 		SIO_STAT_READY();
 		memcardInit();
-		SIO_INT();
+		sioInterrupt();
 		break;
 
 	case 1:
@@ -650,6 +707,7 @@ SIO_WRITE sioWriteMemcard(u8 data)
 		case 0x52: // PSX 'R'ead
 		case 0x53: // PSX 'S'tate
 		case 0x57: // PSX 'W'rite
+		case 0x58: // PSX Pocketstation
 			siomode = SIO_DUMMY;
 			break;
 
@@ -660,6 +718,7 @@ SIO_WRITE sioWriteMemcard(u8 data)
 		case 0x11: // On Boot/Probe
 		case 0x12: // On Write/Delete/Recheck?
 			sio2.packet.recvVal3 = 0x8C;
+			// Fall through
 
 		case 0x81: // Checked right after copy/delete
 		case 0xBF: // Wtf?? On game booting?
@@ -670,11 +729,11 @@ SIO_WRITE sioWriteMemcard(u8 data)
 			break;
 
 		default:
-			DevCon.Warning("%s cmd: %02X??\n", __FUNCTION__, data);
+			DevCon.Warning("%s cmd: %02X??", __FUNCTION__, data);
 			siomode = SIO_DUMMY;
 			break;
 		}
-		SIO_INT();
+		sioInterrupt();
 		break;
 
 	case 2:
@@ -696,12 +755,12 @@ SIO_WRITE sioWriteMemcardPSX(u8 data)
 	case 0: // Same init stuff...
 		SIO_STAT_READY();
 		memcardInit();
-		SIO_INT();
+		sioInterrupt();
 		break;
 
 	case 1:
-		sio.cmd = data; 
-		switch(data) 
+		sio.cmd = data;
+		switch(data)
 		{
 		case 0x53: // PSX 'S'tate // haven't seen it happen yet
 			sio.buf[1] = mcd->FLAG;
@@ -711,23 +770,25 @@ SIO_WRITE sioWriteMemcardPSX(u8 data)
 
 		case 0x52: // PSX 'R'ead / Probe
 		case 0x57: // PSX 'W'rite
+		case 0x58: // POCKETSTATION!! Grrrr // Lots of love to the PS2DEV/ps2sdk
 			sio.buf[1] = 0x00; //mcd->FLAG;
 			sio.buf[2] = 0x5A; // probe end, success "0x5A"
 			sio.buf[3] = 0x5D;
 			sio.buf[4] = 0x00;
 			break;
-
-		case 0x58: // POCKETSTATION!! Grrrr // Lots of love to the PS2DEV/ps2sdk 
-			DEVICE_UNPLUGGED(); // Check is for 0x01000 on stat
-			siomode = SIO_DUMMY;
-			break;
+		// Old handing for Pocketstation, effectively discarded the calls.
+		// Keeping it around for reference.
+		//case 0x58: // POCKETSTATION!! Grrrr // Lots of love to the PS2DEV/ps2sdk
+		//	DEVICE_UNPLUGGED(); // Check is for 0x01000 on stat
+		//	siomode = SIO_DUMMY;
+		//	break;
 
 		default:
 			//printf("%s cmd: %02X??\n", __FUNCTION__, data);
 			siomode = SIO_DUMMY;
 			break;
 		}
-		SIO_INT();
+		sioInterrupt();
 		break;
 
 	case 2: break;
@@ -742,7 +803,7 @@ SIO_WRITE sioWriteMemcardPSX(u8 data)
 		sio.buf[6] = data;
 		mcd->sectorAddr |= data;
 		mcd->goodSector = !(mcd->sectorAddr > 0x3FF);
-		mcd->transferAddr = 128 * mcd->sectorAddr; 
+		mcd->transferAddr = 128 * mcd->sectorAddr;
 		break;
 
 	case 6:
@@ -788,7 +849,7 @@ SIO_WRITE sioWriteMemcardPSX(u8 data)
 			sio.buf[136] = 0x5D;
 
 			// (47h=Good, 4Eh=BadChecksum, FFh=BadSector)
-			sio.buf[137] = data == xorcheck ? 0x47 : 0x4E; 
+			sio.buf[137] = data == xorcheck ? 0x47 : 0x4E;
 			if(!mcd->goodSector) sio.buf[137] = 0xFF;
 			else mcd->Write(&sio.buf[7], 0x80);
 			siomode = SIO_DUMMY;
@@ -802,11 +863,44 @@ SIO_WRITE sioWriteInfraRed(u8 data)
 	SIO_STAT_READY();
 	DEVICE_PLUGGED();
 	siomode = SIO_DUMMY;
-	SIO_INT();
+	sioInterrupt();
 }
 
+//This bit-field in the STATUS register contains the (inveted) state of the /ACK linre from the Controller / MC.
+//1 = /ACK_line_active_low
+//Should go into Sio.h
+#define ACK_INP 0x80
+
+//This is named RESET_ERR in sio_internal.h.
+#define CLR_INTR 0x0010
+//Set the ammount of received bytes that triggers an interrupt.
+//0=1, 1=2, 2=4, 3=8 receivedBytesIntTriger = 1<< ((ctrl & RX_BYTES_INT) >>8)
+#define RX_BYTES_INT 0x0300
+//Enable interrupt on TX ready and TX empty
+#define TX_INT_EN 0x0400
+//Trigger interrupt after receiving several (see above) bytes.
+#define RX_INT_EN 0x0800
+//Controll register: Enable the /ACK line trigerring the interrupt.
+#define ACK_INT_EN 0x1000
+//Selects slot 1 or 2
+#define SLOT_NR 0x2000
+
+void chkTriggerInt() {
+	//Conditions for triggerring an interrupt.
+	//this is not correct, but ... it can be fixed later
+	 sioInterrupt(); return;
+	if ((sio.StatReg & IRQ)) { sioInterrupt(); return; } //The interrupt flag in the main INTR_STAT reg should go active on multiple occasions. Set it here for now (hack), until the correct mechanism is made.
+	if ((sio.CtrlReg & ACK_INT_EN) && ((sio.StatReg & TX_RDY) || (sio.StatReg & TX_EMPTY))) { sioInterrupt(); return; }
+	if ((sio.CtrlReg & ACK_INT_EN) && (sio.StatReg & ACK_INP)) { sioInterrupt(); return; }
+	//The following one may be incorrect.
+	//if ((sio.CtrlReg & RX_INT_EN) && ((byteCnt >= (1<< ((sio.CtrlReg & RX_BYTES_INT) >>8))) ? 1:0) ) { sioInterrupt(); return; }
+	return;
+}
+
+
 static void sioWrite8inl(u8 data)
 {
+//	Console.WriteLn( "SIO DATA write %02X  mode %08X " , data, siomode);
 	switch(siomode)
 	{
 	case SIO_START: sioWriteStart(data); break;
@@ -822,12 +916,44 @@ static void sioWrite8inl(u8 data)
 	case SIO_MEMCARD_PSX: sioWriteMemcardPSX(data); break;
 	case SIO_DUMMY: break;
 	};
+	sio.StatReg |= RX_RDY; //Why not set the byte-received flag, when for EVERY sent byte, one is received... it's just how SPI is...
+	sio.StatReg |= TX_EMPTY; //The current byte *has* been sent, so it is empty.
+	if (IS_LAST_BYTE_IN_PACKET != 1) //The following should be set after each byte transfer but the last one.
+		sio.StatReg |= ACK_INP; //Signal that Controller (or MC) has brought the /ACK (Acknowledge) line active low.
+
+	sioInterrupt();
+	//chkTriggerInt();
+	//Console.WriteLn( "SIO0 WR DATA COMMON %02X  INT_STAT= %08X  IOPpc= %08X " , data, psxHu32(0x1070), psxRegs.pc);
+	byteCnt++;
 }
 
-void sioWriteCtrl16(u16 value) 
+int clrAckCnt =0;
+
+void sioStatRead() {
+
+if (clrAckCnt > 1) {  //This check can probably be removed...
+	sio.StatReg &= ~ACK_INP; //clear (goes inactive) /ACK line.
+	// sio.StatReg &= ~TX_RDY;
+	// sio.StatReg &= ~0x200; //irq
+	// if (byteCnt == 1)
+	// 	sio.StatReg &= ~RX_RDY;
+	clrAckCnt = 0;
+}
+	//The /ACK line should go active for >2us, in a time window between 12us and 100us after each byte is sent (received by the controller).
+	//If that doesn't happen, the controller is considered missing.
+	//The /ACK line must NOT go active after the last byte in the transmission! (Otherwise some err. may happen - tested.)
+if ((sio.StatReg & ACK_INP)) clrAckCnt++;
+
+	chkTriggerInt();
+	return;
+}
+
+void sioWriteCtrl16(u16 value)
 {
 	static u8 tcount[2];
 
+//	Console.WriteLn( "SIO0 WR CTRL %02X  IOPpc= %08X " , value, psxRegs.pc);
+
 	tcount[sio.port] = sio.bufCount;
 	sio.port = (value >> 13) & 1;
 
@@ -847,24 +973,29 @@ void sioWriteCtrl16(u16 value)
 		psxRegs.interrupt &= ~(1<<IopEvt_SIO);
 	}
 
+	if (sio.CtrlReg & CLR_INTR) sio.StatReg &= ~(IRQ | PARITY_ERR); //clear internal interrupt
 	sio.bufCount = tcount[sio.port];
+	//chkTriggerInt(); //necessary? - causes transfer to stup after the third byte
 }
 
-u8 sioRead8() 
+u8 sioRead8()
 {
 	u8 ret;
 
-	if(sio.StatReg & RX_RDY) 
+	if(sio.StatReg & RX_RDY)
 	{
 		ret = sio.buf[sio.bufCount];
 		if(sio.bufCount == sio.bufSize) SIO_STAT_EMPTY();
 		sio.bufCount++;
+		SIO_STAT_EMPTY(); //this should depend on the counter above... but it seems wrong (buffer never empties).
+		sio.StatReg &= ~TX_RDY; //all clear (transfer of byte ended)
 	}
 	else
 	{
 		ret = sio.ret;
 	}
-
+		//	sio.StatReg &= ~TX_RDY; //all clear (transfer of byte ended)
+//	Console.WriteLn( "SIO DATA read %02X  stat= %08X " , ret, sio.StatReg);
 	return ret;
 }
 
@@ -873,8 +1004,8 @@ void sioWrite8(u8 value)
 	sioWrite8inl(value);
 }
 
-void SIODMAWrite(u8 value)
-{
+void SIODMAWrite(u8 value) //Why does the SIO2 FIFO handler call this function...??
+{ //PS1 and PS2 modes are separate and the SIO0 and SIO2 aren't active at the sam time...
 	sioWrite8inl(value);
 }
 
@@ -886,18 +1017,14 @@ void sioNextFrame() {
 	}
 }
 
-// Used to figure out when a new game boots, so that memory cards can re-index themselves and only load data relevant to that game.
-wxString SioCurrentGameSerial = L"";
 void sioSetGameSerial( const wxString& serial ) {
-	if ( serial == SioCurrentGameSerial ) { return; }
-	SioCurrentGameSerial = serial;
-
 	for ( uint port = 0; port < 2; ++port ) {
 		for ( uint slot = 0; slot < 4; ++slot ) {
-			mcds[port][slot].ReIndex( serial );
+			if ( mcds[port][slot].ReIndex( serial ) ) {
+				SetForceMcdEjectTimeoutNow( port, slot );
+			}
 		}
 	}
-	SetForceMcdEjectTimeoutNow();
 }
 
 void SaveStateBase::sioFreeze()
@@ -936,7 +1063,7 @@ void SaveStateBase::sioFreeze()
 		//    it has a "rule" that the memcard should never be ejected during a song.  So by
 		//    ejecting it, the game freezes (which is actually good emulation, but annoying!)
 
-		for( u8 port=0; port<2; ++port ) 
+		for( u8 port=0; port<2; ++port )
 			for( u8 slot=0; slot<4; ++slot )
 		{
 			u64 checksum = mcds[port][slot].GetChecksum();
