@@ -20,13 +20,12 @@
 #include "newVif.h"
 #include "MTVU.h"
 
-#include "SamplProf.h"
-
 #include "Elfheader.h"
 
 #include "System/RecTypes.h"
 
 #include "Utilities/MemsetFast.inl"
+#include "Utilities/Perf.h"
 
 
 // --------------------------------------------------------------------------------------
@@ -37,16 +36,12 @@
 // Parameters:
 //   name - a nice long name that accurately describes the contents of this reserve.
 RecompiledCodeReserve::RecompiledCodeReserve( const wxString& name, uint defCommit )
-	: BaseVmReserveListener( name )
+	: VirtualMemoryReserve( name, defCommit )
 {
-	m_blocksize		= (1024 * 128) / __pagesize;
 	m_prot_mode		= PageAccess_Any();
-	m_def_commit	= defCommit / __pagesize;
-	
-	m_profiler_registered = false;
 }
 
-RecompiledCodeReserve::~RecompiledCodeReserve() throw()
+RecompiledCodeReserve::~RecompiledCodeReserve()
 {
 	_termProfiler();
 }
@@ -54,29 +49,21 @@ RecompiledCodeReserve::~RecompiledCodeReserve() throw()
 void RecompiledCodeReserve::_registerProfiler()
 {
 	if (m_profiler_name.IsEmpty() || !IsOk()) return;
-	ProfilerRegisterSource( m_profiler_name, m_baseptr, GetReserveSizeInBytes() );
-	m_profiler_registered = true;
+
+	Perf::any.map((uptr)m_baseptr, GetReserveSizeInBytes(), m_profiler_name.ToUTF8());
 }
 
 void RecompiledCodeReserve::_termProfiler()
 {
-	if (m_profiler_registered)
-		ProfilerTerminateSource( m_profiler_name );
-}
-
-uint RecompiledCodeReserve::_calcDefaultCommitInBlocks() const
-{
-	return (m_def_commit + m_blocksize - 1) / m_blocksize;
 }
 
 void* RecompiledCodeReserve::Reserve( size_t size, uptr base, uptr upper_bounds )
 {
 	if (!_parent::Reserve(size, base, upper_bounds)) return NULL;
-	_registerProfiler();
 
-	// Pre-Allocate the first block (to reduce the number of segmentation fault
-	// in debugger)
-	DoCommitAndProtect(0);
+	Commit();
+
+	_registerProfiler();
 
 	return m_baseptr;
 }
@@ -85,11 +72,24 @@ void RecompiledCodeReserve::Reset()
 {
 	_parent::Reset();
 
-	// Pre-Allocate the first block (to reduce the number of segmentation fault
-	// in debugger)
-	DoCommitAndProtect(0);
+	Commit();
 }
 
+bool RecompiledCodeReserve::Commit()
+{
+	bool status = _parent::Commit();
+
+	if (IsDevBuild && m_baseptr)
+	{
+		// Clear the recompiled code block to 0xcc (INT3) -- this helps disasm tools show
+		// the assembly dump more cleanly.  We don't clear the block on Release builds since
+		// it can add a noticeable amount of overhead to large block recompilations.
+
+		memset(m_baseptr, 0xCC, m_pages_commited * __pagesize);
+	}
+
+	return status;
+}
 
 // Sets the abbreviated name used by the profiler.  Name should be under 10 characters long.
 // After a name has been set, a profiler source will be automatically registered and cleared
@@ -101,23 +101,6 @@ RecompiledCodeReserve& RecompiledCodeReserve::SetProfilerName( const wxString& s
 	return *this;
 }
 
-void RecompiledCodeReserve::DoCommitAndProtect( uptr page )
-{
-	CommitBlocks(page, (m_pages_commited || !m_def_commit) ? 1 : _calcDefaultCommitInBlocks() );
-}
-
-void RecompiledCodeReserve::OnCommittedBlock( void* block )
-{
-	if (IsDevBuild)
-	{
-		// Clear the recompiled code block to 0xcc (INT3) -- this helps disasm tools show
-		// the assembly dump more cleanly.  We don't clear the block on Release builds since
-		// it can add a noticeable amount of overhead to large block recompilations.
-
-		memset(block, 0xCC, m_blocksize * __pagesize);
-	}
-}
-
 // This error message is shared by R5900, R3000, and microVU recompilers.  It is not used by the
 // SuperVU recompiler, since it has its own customized message.
 void RecompiledCodeReserve::ThrowIfNotOk() const
@@ -270,9 +253,6 @@ void SysLogMachineCaps()
 	if( x86caps.hasAVX2 )							features[0].Add( L"AVX2" );
 	if( x86caps.hasFMA)								features[0].Add( L"FMA" );
 
-	if( x86caps.hasMultimediaExtensionsExt )		features[1].Add( L"MMX2  " );
-	if( x86caps.has3DNOWInstructionExtensions )		features[1].Add( L"3DNOW " );
-	if( x86caps.has3DNOWInstructionExtensionsExt )	features[1].Add( L"3DNOW2" );
 	if( x86caps.hasStreamingSIMD4ExtensionsA )		features[1].Add( L"SSE4a " );
 
 	const wxString result[2] =
@@ -282,7 +262,11 @@ void SysLogMachineCaps()
 	};
 
 	Console.WriteLn( Color_StrongBlack,	L"x86 Features Detected:" );
-	Console.Indent().WriteLn( result[0] + (result[1].IsEmpty() ? L"" : (L"\n" + result[1])) );
+    Console.Indent().WriteLn(result[0] + (result[1].IsEmpty() ? L"" : (L"\n" + result[1])));
+#ifdef __M_X86_64
+    Console.Indent().WriteLn("Pcsx2 was compiled as 64-bits, which is unsupported and breaks all recompilers.");
+#endif
+
 	Console.Newline();
 }
 
@@ -290,19 +274,19 @@ template< typename CpuType >
 class CpuInitializer
 {
 public:
-	ScopedPtr<CpuType>			MyCpu;
-	ScopedExcept	ExThrown;
-	
+	std::unique_ptr<CpuType> MyCpu;
+	ScopedExcept ExThrown;
+
 	CpuInitializer();
-	virtual ~CpuInitializer() throw();
+	virtual ~CpuInitializer();
 
 	bool IsAvailable() const
 	{
 		return !!MyCpu;
 	}
 
-	CpuType* GetPtr() { return MyCpu.GetPtr(); }
-	const CpuType* GetPtr() const { return MyCpu.GetPtr(); }
+	CpuType* GetPtr() { return MyCpu.get(); }
+	const CpuType* GetPtr() const { return MyCpu.get(); }
 
 	operator CpuType*() { return GetPtr(); }
 	operator const CpuType*() const { return GetPtr(); }
@@ -317,25 +301,25 @@ template< typename CpuType >
 CpuInitializer< CpuType >::CpuInitializer()
 {
 	try {
-		MyCpu = new CpuType();
+		MyCpu = std::make_unique<CpuType>();
 		MyCpu->Reserve();
 	}
 	catch( Exception::RuntimeError& ex )
 	{
 		Console.Error( L"CPU provider error:\n\t" + ex.FormatDiagnosticMessage() );
-		MyCpu = NULL;
-		ExThrown = ex.Clone();
+		MyCpu = nullptr;
+		ExThrown = ScopedExcept(ex.Clone());
 	}
 	catch( std::runtime_error& ex )
 	{
 		Console.Error( L"CPU provider error (STL Exception)\n\tDetails:" + fromUTF8( ex.what() ) );
-		MyCpu = NULL;
-		ExThrown = new Exception::RuntimeError(ex);
+		MyCpu = nullptr;
+		ExThrown = ScopedExcept(new Exception::RuntimeError(ex));
 	}
 }
 
 template< typename CpuType >
-CpuInitializer< CpuType >::~CpuInitializer() throw()
+CpuInitializer< CpuType >::~CpuInitializer()
 {
 	try {
 		if (MyCpu)
@@ -364,7 +348,7 @@ public:
 
 public:
 	CpuInitializerSet() {}
-	virtual ~CpuInitializerSet() throw() {}
+	virtual ~CpuInitializerSet() = default;
 };
 
 
@@ -382,7 +366,7 @@ SysMainMemory::SysMainMemory()
 {
 }
 
-SysMainMemory::~SysMainMemory() throw()
+SysMainMemory::~SysMainMemory()
 {
 	try {
 		ReleaseAll();
@@ -400,9 +384,6 @@ void SysMainMemory::ReserveAll()
 	m_ee.Reserve();
 	m_iop.Reserve();
 	m_vu.Reserve();
-	
-	reserveNewVif(0);
-	reserveNewVif(1);
 }
 
 void SysMainMemory::CommitAll()
@@ -452,7 +433,7 @@ void SysMainMemory::DecommitAll()
 
 	closeNewVif(0);
 	closeNewVif(1);
-	
+
 	vtlb_Core_Free();
 }
 
@@ -484,14 +465,14 @@ SysCpuProviderPack::SysCpuProviderPack()
 	Console.WriteLn( Color_StrongBlue, "Reserving memory for recompilers..." );
 	ConsoleIndentScope indent(1);
 
-	CpuProviders = new CpuInitializerSet();
+	CpuProviders = std::make_unique<CpuInitializerSet>();
 
 	try {
 		recCpu.Reserve();
 	}
 	catch( Exception::RuntimeError& ex )
 	{
-		m_RecExceptionEE = ex.Clone();
+		m_RecExceptionEE = ScopedExcept(ex.Clone());
 		Console.Error( L"EE Recompiler Reservation Failed:\n" + ex.FormatDiagnosticMessage() );
 		recCpu.Shutdown();
 	}
@@ -501,7 +482,7 @@ SysCpuProviderPack::SysCpuProviderPack()
 	}
 	catch( Exception::RuntimeError& ex )
 	{
-		m_RecExceptionIOP = ex.Clone();
+		m_RecExceptionIOP = ScopedExcept(ex.Clone());
 		Console.Error( L"IOP Recompiler Reservation Failed:\n" + ex.FormatDiagnosticMessage() );
 		psxRec.Shutdown();
 	}
@@ -517,18 +498,18 @@ SysCpuProviderPack::SysCpuProviderPack()
 
 bool SysCpuProviderPack::IsRecAvailable_MicroVU0() const { return CpuProviders->microVU0.IsAvailable(); }
 bool SysCpuProviderPack::IsRecAvailable_MicroVU1() const { return CpuProviders->microVU1.IsAvailable(); }
-BaseException* SysCpuProviderPack::GetException_MicroVU0() const { return CpuProviders->microVU0.ExThrown; }
-BaseException* SysCpuProviderPack::GetException_MicroVU1() const { return CpuProviders->microVU1.ExThrown; }
+BaseException* SysCpuProviderPack::GetException_MicroVU0() const { return CpuProviders->microVU0.ExThrown.get(); }
+BaseException* SysCpuProviderPack::GetException_MicroVU1() const { return CpuProviders->microVU1.ExThrown.get(); }
 
 #ifndef DISABLE_SVU
 bool SysCpuProviderPack::IsRecAvailable_SuperVU0() const { return CpuProviders->superVU0.IsAvailable(); }
 bool SysCpuProviderPack::IsRecAvailable_SuperVU1() const { return CpuProviders->superVU1.IsAvailable(); }
-BaseException* SysCpuProviderPack::GetException_SuperVU0() const { return CpuProviders->superVU0.ExThrown; }
-BaseException* SysCpuProviderPack::GetException_SuperVU1() const { return CpuProviders->superVU1.ExThrown; }
+BaseException* SysCpuProviderPack::GetException_SuperVU0() const { return CpuProviders->superVU0.ExThrown.get(); }
+BaseException* SysCpuProviderPack::GetException_SuperVU1() const { return CpuProviders->superVU1.ExThrown.get(); }
 #endif
 
 
-void SysCpuProviderPack::CleanupMess() throw()
+void SysCpuProviderPack::CleanupMess() noexcept
 {
 	try
 	{
@@ -544,7 +525,7 @@ void SysCpuProviderPack::CleanupMess() throw()
 	DESTRUCTOR_CATCHALL
 }
 
-SysCpuProviderPack::~SysCpuProviderPack() throw()
+SysCpuProviderPack::~SysCpuProviderPack()
 {
 	CleanupMess();
 }
@@ -661,20 +642,26 @@ u8* SysMmapEx(uptr base, u32 size, uptr bounds, const char *caller)
 	return Mem;
 }
 
+wxString SysGetBiosDiscID()
+{
+	// FIXME: we should return a serial based on
+	// the BIOS being run (either a checksum of the BIOS roms, and/or a string based on BIOS
+	// region and revision).
+
+	return wxEmptyString;
+}
+
 // This function always returns a valid DiscID -- using the Sony serial when possible, and
 // falling back on the CRC checksum of the ELF binary if the PS2 software being run is
 // homebrew or some other serial-less item.
 wxString SysGetDiscID()
 {
 	if( !DiscSerial.IsEmpty() ) return DiscSerial;
-	
+
 	if( !ElfCRC )
 	{
-		// FIXME: system is currently running the BIOS, so it should return a serial based on
-		// the BIOS being run (either a checksum of the BIOS roms, and/or a string based on BIOS
-		// region and revision).
-		
-		return wxEmptyString;
+		// system is currently running the BIOS
+		return SysGetBiosDiscID();
 	}
 
 	return pxsFmt( L"%08x", ElfCRC );
