@@ -19,19 +19,15 @@
 #include "Gif_Unit.h"
 #include "Vif_Dma.h"
 
-u16 vifqwc = 0;
-
 static u32 qwctag(u32 mask)
 {
 	return (dmacRegs.rbor.ADDR + (mask & dmacRegs.rbsr.RMSK));
 }
 
-static u16 QWCinVIFMFIFO(u32 DrainADDR)
+static u16 QWCinVIFMFIFO(u32 DrainADDR, u16 qwc)
 {
 	u32 ret;
 	
-
-	SPR_LOG("VIF MFIFO Requesting %x QWC from the MFIFO Base %x MFIFO Top %x, SPR MADR %x Drain %x", vif1ch.qwc, dmacRegs.rbor.ADDR, dmacRegs.rbor.ADDR + dmacRegs.rbsr.RMSK + 16, spr0ch.madr, DrainADDR);
 	//Calculate what we have in the fifo.
 	if(DrainADDR <= spr0ch.madr)
 	{
@@ -45,26 +41,29 @@ static u16 QWCinVIFMFIFO(u32 DrainADDR)
 		//calculate from base to the SPR tag addr and what is left in the top of the ring
 		ret = ((spr0ch.madr - dmacRegs.rbor.ADDR) + (limit - DrainADDR)) >> 4;
 	}
-	SPR_LOG("%x Available of the %x requested", ret, vif1ch.qwc);
-	
+
+	VIF_LOG("VIF MFIFO Requesting %x QWC of %x Available from the MFIFO Base %x MFIFO Top %x, SPR MADR %x Drain %x", qwc, ret, dmacRegs.rbor.ADDR, dmacRegs.rbor.ADDR + dmacRegs.rbsr.RMSK + 16, spr0ch.madr, DrainADDR);
+
 	return ret;
 }
 static __fi bool mfifoVIF1rbTransfer()
 {
-	u32 maddr = dmacRegs.rbor.ADDR;
 	u32 msize = dmacRegs.rbor.ADDR + dmacRegs.rbsr.RMSK + 16;
-	u16 mfifoqwc = std::min(QWCinVIFMFIFO(vif1ch.madr), vif1ch.qwc);
+	u16 mfifoqwc = std::min(QWCinVIFMFIFO(vif1ch.madr, vif1ch.qwc), vif1ch.qwc);
 	u32 *src;
 	bool ret;
-
-	if(mfifoqwc == 0) return true; //Cant do anything, lets forget it 
+	
+	if (mfifoqwc == 0) {
+		DevCon.Warning("VIF MFIFO no QWC before transfer (in transfer function, bit late really)");
+		return true; //Cant do anything, lets forget it 
+	}
 
 	/* Check if the transfer should wrap around the ring buffer */
 	if ((vif1ch.madr + (mfifoqwc << 4)) > (msize))
 	{
 		int s1 = ((msize) - vif1ch.madr) >> 2;
 
-		SPR_LOG("Split MFIFO");
+		VIF_LOG("Split MFIFO");
 
 		/* it does, so first copy 's1' bytes from 'addr' to 'data' */
 		vif1ch.madr = qwctag(vif1ch.madr);
@@ -81,18 +80,19 @@ static __fi bool mfifoVIF1rbTransfer()
 		{
 			if(vif1.irqoffset.value != 0) DevCon.Warning("VIF1 MFIFO Offest != 0! vifoffset=%x", vif1.irqoffset.value);
             /* and second copy 's2' bytes from 'maddr' to '&data[s1]' */
-            vif1ch.madr = maddr;
+			//DevCon.Warning("Loopyloop");
+			vif1ch.tadr = qwctag(vif1ch.tadr);
+			vif1ch.madr = qwctag(vif1ch.madr);
 
-            src = (u32*)PSM(maddr);
+            src = (u32*)PSM(vif1ch.madr);
             if (src == NULL) return false;
             VIF1transfer(src, ((mfifoqwc << 2) - s1));
 		}
-		
 	}
 	else
 	{
-		SPR_LOG("Direct MFIFO");
-		
+		VIF_LOG("Direct MFIFO");
+
 		/* it doesn't, so just transfer 'qwc*4' words */
 		src = (u32*)PSM(vif1ch.madr);
 		if (src == NULL) return false;
@@ -101,7 +101,6 @@ static __fi bool mfifoVIF1rbTransfer()
 			ret = VIF1transfer(src + vif1.irqoffset.value, mfifoqwc * 4 - vif1.irqoffset.value);
 		else
 			ret = VIF1transfer(src, mfifoqwc << 2);
-
 	}
 	return ret;
 }
@@ -117,22 +116,28 @@ static __fi void mfifo_VIF1chain()
 
 	if (vif1ch.madr >= dmacRegs.rbor.ADDR &&
 	        vif1ch.madr < (dmacRegs.rbor.ADDR + dmacRegs.rbsr.RMSK + 16u))
-	{		
+	{
 		//if(vif1ch.madr == (dmacRegs.rbor.ADDR + dmacRegs.rbsr.RMSK + 16)) DevCon.Warning("Edge VIF1");
-		
-		vif1ch.madr = qwctag(vif1ch.madr);
-		mfifoVIF1rbTransfer();
-		vif1ch.tadr = qwctag(vif1ch.tadr);
-		vif1ch.madr = qwctag(vif1ch.madr);
-		if(QWCinVIFMFIFO(vif1ch.madr) == 0) vif1.inprogress |= 0x10;
+		if (QWCinVIFMFIFO(vif1ch.madr, vif1ch.qwc) == 0) {
+			VIF_LOG("VIF MFIFO Empty before transfer");
+			vif1.inprogress |= 0x10;
+			g_vif1Cycles += 4;
+			return;
+		}
 
-		//vifqwc -= startqwc - vif1ch.qwc;
+		mfifoVIF1rbTransfer();
+		vif1ch.madr = qwctag(vif1ch.madr);
+		//When transferring direct from the MFIFO, the TADR needs to be after the data last read
+		//FF7 DoC Expects the transfer to end with an Empty interrupt, so the TADR has to match SPR0_MADR
+		//It does an END tag (which normally doesn't increment TADR because it breaks Soul Calibur 2)
+		//with a QWC of 1 (rare) so we need to increment the TADR in the case of MFIFO.
+		vif1ch.tadr = vif1ch.madr;
 		
 	}
 	else
 	{
 		tDMA_TAG *pMem = dmaGetAddr(vif1ch.madr, !vif1ch.chcr.DIR);
-		SPR_LOG("Non-MFIFO Location");
+		VIF_LOG("Non-MFIFO Location");
 
 		//No need to exit on non-mfifo as it is indirect anyway, so it can be transferring this while spr refills the mfifo
 
@@ -159,7 +164,7 @@ void mfifoVifMaskMem(int id)
 				//DevCon.Warning("VIF MFIFO MADR below bottom of ring buffer, wrapping VIF MADR = %x Ring Bottom %x", vif1ch.madr, dmacRegs.rbor.ADDR);
 				vif1ch.madr = qwctag(vif1ch.madr);
 			}
-			if(vif1ch.madr > (dmacRegs.rbor.ADDR + dmacRegs.rbsr.RMSK)) //Usual scenario is the tag is near the end (Front Mission 4)
+			if(vif1ch.madr > (dmacRegs.rbor.ADDR + (u32)dmacRegs.rbsr.RMSK)) //Usual scenario is the tag is near the end (Front Mission 4)
 			{
 				//DevCon.Warning("VIF MFIFO MADR outside top of ring buffer, wrapping VIF MADR = %x Ring Top %x", vif1ch.madr, (dmacRegs.rbor.ADDR + dmacRegs.rbsr.RMSK)+16);
 				vif1ch.madr = qwctag(vif1ch.madr);
@@ -171,36 +176,21 @@ void mfifoVifMaskMem(int id)
 	}
 }
 
-void mfifoVIF1transfer(int qwc)
+void mfifoVIF1transfer()
 {
 	tDMA_TAG *ptag;
 
 	g_vif1Cycles = 0;
 
-	if (qwc > 0)
-	{
-		//vifqwc += qwc;
-		SPR_LOG("Added %x qw to mfifo,Vif CHCR %x Stalled %x done %x", qwc, vif1ch.chcr._u32, vif1.vifstalled.enabled, vif1.done);
-		if (vif1.inprogress & 0x10)
-		{
-			//Don't resume if stalled or already looping
-			if(vif1ch.chcr.STR == true && !(cpuRegs.interrupt & (1<<DMAC_MFIFO_VIF)) && !vif1Regs.stat.INT)
-			{
-				SPR_LOG("Data Added, Resuming");
-				//Need to simulate the time it takes to copy here, if the VIF resumes before the SPR has finished, it isn't happy.
-				CPU_INT(DMAC_MFIFO_VIF, qwc * BIAS);
-			}
-
-			//Apparently this is bad, i guess so, the data is going to memory rather than the FIFO
-			//vif1Regs.stat.FQC = 0x10; // FQC=16
-		}
-		vif1.inprogress &= ~0x10;
-
-		return;
-	}
-
 	if (vif1ch.qwc == 0)
 	{
+		if (QWCinVIFMFIFO(vif1ch.tadr, 1) == 0) {
+			VIF_LOG("VIF MFIFO Empty before tag");
+			vif1.inprogress |= 0x10;
+			g_vif1Cycles += 4;
+			return;
+		}
+
 		vif1ch.tadr = qwctag(vif1ch.tadr);
 		ptag = dmaGetAddr(vif1ch.tadr, false);
 
@@ -211,7 +201,7 @@ void mfifoVIF1transfer(int qwc)
 
 		if (vif1ch.chcr.TTE)
 		{
-            bool ret;
+			bool ret;
 
 			static __aligned16 u128 masked_tag;
 
@@ -230,7 +220,6 @@ void mfifoVIF1transfer(int qwc)
 				vif1.irqoffset.value = 2;
 				vif1.irqoffset.enabled = true;
 				ret = VIF1transfer((u32*)&masked_tag + 2, 2, true);  //Transfer Tag
-				//ret = VIF1transfer((u32*)ptag + 2, 2);  //Transfer Tag
 			}
 
 			if (!ret && vif1.irqoffset.enabled)
@@ -238,10 +227,10 @@ void mfifoVIF1transfer(int qwc)
 				vif1.inprogress &= ~1;
 				return;        //IRQ set by VIFTransfer
 				
-			} //else vif1.vifstalled.enabled = false;
+			}
 			g_vif1Cycles += 2;
 		}
-		
+
 		vif1.irqoffset.value = 0;
 		vif1.irqoffset.enabled = false;
 
@@ -249,10 +238,8 @@ void mfifoVIF1transfer(int qwc)
 
 		vif1ch.madr = ptag[1]._u32;
 
-		//vifqwc--;
-
-		SPR_LOG("dmaChain %8.8x_%8.8x size=%d, id=%d, madr=%lx, tadr=%lx mfifo qwc = %x spr0 madr = %x",
-        ptag[1]._u32, ptag[0]._u32, vif1ch.qwc, ptag->ID, vif1ch.madr, vif1ch.tadr, vifqwc, spr0ch.madr);
+		VIF_LOG("dmaChain %8.8x_%8.8x size=%d, id=%d, madr=%lx, tadr=%lx spr0 madr = %x",
+        ptag[1]._u32, ptag[0]._u32, vif1ch.qwc, ptag->ID, vif1ch.madr, vif1ch.tadr, spr0ch.madr);
 
 		vif1.done |= hwDmacSrcChainWithStack(vif1ch, ptag->ID);
 
@@ -264,12 +251,9 @@ void mfifoVIF1transfer(int qwc)
 			vif1.done = true;
 		}
 
-		
-		if(vif1ch.qwc > 0) 	vif1.inprogress |= 1;
-
 		vif1ch.tadr = qwctag(vif1ch.tadr);
 
-		if(QWCinVIFMFIFO(vif1ch.tadr) == 0) vif1.inprogress |= 0x10;
+		if(vif1ch.qwc > 0) 	vif1.inprogress |= 1;
 	}
 	else
 	{
@@ -277,7 +261,7 @@ void mfifoVIF1transfer(int qwc)
 	}
 	
 
-	SPR_LOG("mfifoVIF1transfer end %x madr %x, tadr %x vifqwc %x", vif1ch.chcr._u32, vif1ch.madr, vif1ch.tadr, vifqwc);
+	VIF_LOG("mfifoVIF1transfer end %x madr %x, tadr %x", vif1ch.chcr._u32, vif1ch.madr, vif1ch.tadr);
 }
 
 void vifMFIFOInterrupt()
@@ -285,6 +269,11 @@ void vifMFIFOInterrupt()
 	g_vif1Cycles = 0;
 	VIF_LOG("vif mfifo interrupt");
 
+	if (dmacRegs.ctrl.MFD != MFD_VIF1) {
+		vif1Interrupt();
+		return;
+	}
+
 	if( gifRegs.stat.APATH == 2  && gifUnit.gifPath[1].isDone())
 	{
 		gifRegs.stat.APATH = 0;
@@ -293,11 +282,6 @@ void vifMFIFOInterrupt()
 		if(gifUnit.checkPaths(1,0,1)) gifUnit.Execute(false, true);
 	}
 
-	if (dmacRegs.ctrl.MFD != MFD_VIF1) {
-		DevCon.Warning("Not in VIF MFIFO mode! Stopping VIF MFIFO");
-		return;
-	}
-
 	if (vif1ch.chcr.DIR) {
 		bool isDirect   = (vif1.cmd & 0x7f) == 0x50;
 		bool isDirectHL = (vif1.cmd & 0x7f) == 0x51;
@@ -308,7 +292,7 @@ void vifMFIFOInterrupt()
 			return;
 		}
 	}
-	if(vif1.waitforvu == true)
+	if(vif1.waitforvu)
 	{
 	//	DevCon.Warning("Waiting on VU1 MFIFO");
 		//CPU_INT(DMAC_MFIFO_VIF, 16);
@@ -321,7 +305,7 @@ void vifMFIFOInterrupt()
 	// Simulated GS transfer time done, clear the flags
 	
 	if (vif1.irq && vif1.vifstalled.enabled && vif1.vifstalled.value == VIF_IRQ_STALL) {
-		SPR_LOG("VIF MFIFO Code Interrupt detected");
+		VIF_LOG("VIF MFIFO Code Interrupt detected");
 		vif1Regs.stat.INT = true;
 
 		if (((vif1Regs.code >> 24) & 0x7f) != 0x7) {
@@ -335,7 +319,7 @@ void vifMFIFOInterrupt()
 			//vif1Regs.stat.FQC = 0; // FQC=0
 			//vif1ch.chcr.STR = false;
 			vif1Regs.stat.FQC = std::min((u16)0x10, vif1ch.qwc);
-			VIF_LOG("VIF1 MFIFO Stalled qwc = %x done = %x inprogress = %x", vif1ch.qwc,vif1.done, vif1.inprogress & 0x10);
+			VIF_LOG("VIF1 MFIFO Stalled qwc = %x done = %x inprogress = %x", vif1ch.qwc, vif1.done, vif1.inprogress & 0x10);
 			//Used to check if the MFIFO was empty, there's really no need if it's finished what it needed.
 			if((vif1ch.qwc > 0 || !vif1.done)) {
 				VIF_LOG("VIF1 MFIFO Stalled");
@@ -354,21 +338,15 @@ void vifMFIFOInterrupt()
 
 	if(vif1.inprogress & 0x10) {
 		FireMFIFOEmpty();
-		if(!(vif1.done && vif1ch.qwc == 0))return;
+		return;
 	}
 
 	vif1.vifstalled.enabled = false;
 
-	if (vif1.done == false || vif1ch.qwc) {
+	if (!vif1.done || vif1ch.qwc) {
 		switch(vif1.inprogress & 1) {
 			case 0: //Set up transfer
-				if (QWCinVIFMFIFO(vif1ch.tadr) == 0) {
-					vif1.inprogress |= 0x10;
-					CPU_INT(DMAC_MFIFO_VIF, 4 );	
-					return;
-				}
-				
-                mfifoVIF1transfer(0);
+				mfifoVIF1transfer();
 				vif1Regs.stat.FQC = std::min((u16)0x10, vif1ch.qwc);
 				
 			case 1: //Transfer data
@@ -382,11 +360,16 @@ void vifMFIFOInterrupt()
 				return;
 		}
 		return;
-	} 
+	}
 
 	vif1.vifstalled.enabled = false;
 	vif1.irqoffset.enabled = false;
 	vif1.done = 1;
+
+	if (spr0ch.madr == vif1ch.tadr) {
+		FireMFIFOEmpty();
+	}
+
 	g_vif1Cycles = 0;
 	vif1Regs.stat.FQC = std::min((u16)0x10, vif1ch.qwc);
 	vif1ch.chcr.STR = false;
