@@ -26,9 +26,14 @@
 
 #include "Debugger/DisassemblyDialog.h"
 
+#ifndef DISABLE_RECORDING
+#	include "Recording/VirtualPad.h"
+#endif
+
 #include <wx/cmdline.h>
 #include <wx/intl.h>
 #include <wx/stdpaths.h>
+#include <memory>
 
 using namespace pxSizerFlags;
 
@@ -71,6 +76,17 @@ void Pcsx2App::OpenMainFrame()
 	DisassemblyDialog* disassembly = new DisassemblyDialog( mainFrame );
 	m_id_Disassembler = disassembly->GetId();
 
+#ifndef DISABLE_RECORDING
+	VirtualPad* virtualPad0 = new VirtualPad(mainFrame, wxID_ANY, wxEmptyString, 0);
+	m_id_VirtualPad[0] = virtualPad0->GetId();
+	
+	VirtualPad *virtualPad1 = new VirtualPad(mainFrame, wxID_ANY, wxEmptyString, 1);
+	m_id_VirtualPad[1] = virtualPad1->GetId();
+
+	NewRecordingFrame* newRecordingFrame = new NewRecordingFrame(mainFrame);
+	m_id_NewRecordingFrame = newRecordingFrame->GetId();
+#endif
+	
 	if (g_Conf->EmuOptions.Debugger.ShowDebuggerOnStart)
 		disassembly->Show();
 
@@ -128,7 +144,7 @@ void Pcsx2App::AllocateCoreStuffs()
 		// FIXME : Some or all of SysCpuProviderPack should be run from the SysExecutor thread,
 		// so that the thread is safely blocked from being able to start emulation.
 
-		m_CpuProviders = new SysCpuProviderPack();
+		m_CpuProviders = std::make_unique<SysCpuProviderPack>();
 
 		if( m_CpuProviders->HadSomeFailures( g_Conf->EmuOptions.Cpu.Recompiler ) )
 		{
@@ -171,7 +187,7 @@ void Pcsx2App::AllocateCoreStuffs()
 #ifndef DISABLE_SVU
 				recOps.EnableVU0	= recOps.EnableVU0 && m_CpuProviders->IsRecAvailable_SuperVU0();
 #else
-				recOps.EnableVU1	= false;
+				recOps.EnableVU0	= false;
 #endif
 			}
 
@@ -214,7 +230,7 @@ void Pcsx2App::AllocateCoreStuffs()
 
 void Pcsx2App::OnInitCmdLine( wxCmdLineParser& parser )
 {
-	parser.SetLogo( AddAppName(" >>  %s  --  A Playstation2 Emulator for the PC  <<") + L"\n\n" +
+	parser.SetLogo( AddAppName(" >>  %s  --  A PlayStation 2 Emulator for the PC  <<") + L"\n\n" +
 		_("All options are for the current session only and will not be saved.\n")
 	);
 
@@ -235,18 +251,22 @@ void Pcsx2App::OnInitCmdLine( wxCmdLineParser& parser )
 	parser.AddSwitch( wxEmptyString,L"noguiprompt",	_("when nogui - prompt before exiting on suspend") );
 
 	parser.AddOption( wxEmptyString,L"elf",			_("executes an ELF image"), wxCMD_LINE_VAL_STRING );
+	parser.AddOption( wxEmptyString,L"irx",			_("executes an IRX image"), wxCMD_LINE_VAL_STRING );
 	parser.AddSwitch( wxEmptyString,L"nodisc",		_("boots an empty DVD tray; use to enter the PS2 system menu") );
 	parser.AddSwitch( wxEmptyString,L"usecd",		_("boots from the CDVD plugin (overrides IsoFile parameter)") );
 
 	parser.AddSwitch( wxEmptyString,L"nohacks",		_("disables all speedhacks") );
 	parser.AddOption( wxEmptyString,L"gamefixes",	_("use the specified comma or pipe-delimited list of gamefixes.") + fixlist, wxCMD_LINE_VAL_STRING );
 	parser.AddSwitch( wxEmptyString,L"fullboot",	_("disables fast booting") );
+	parser.AddOption( wxEmptyString,L"gameargs",	_("passes the specified space-delimited string of launch arguments to the game"), wxCMD_LINE_VAL_STRING);
 
 	parser.AddOption( wxEmptyString,L"cfgpath",		_("changes the configuration file path"), wxCMD_LINE_VAL_STRING );
 	parser.AddOption( wxEmptyString,L"cfg",			_("specifies the PCSX2 configuration file to use"), wxCMD_LINE_VAL_STRING );
 	parser.AddSwitch( wxEmptyString,L"forcewiz",	AddAppName(_("forces %s to start the First-time Wizard")) );
 	parser.AddSwitch( wxEmptyString,L"portable",	_("enables portable mode operation (requires admin/root access)") );
 
+	parser.AddSwitch( wxEmptyString,L"profiling",	_("update options to ease profiling (debug)") );
+
 	const PluginInfo* pi = tbl_PluginInfo; do {
 		parser.AddOption( wxEmptyString, pi->GetShortname().Lower(),
 			pxsFmt( _("specify the file to use as the %s plugin"), WX_STR(pi->GetShortname()) )
@@ -280,6 +300,8 @@ bool Pcsx2App::ParseOverrides( wxCmdLineParser& parser )
 
 	Overrides.DisableSpeedhacks = parser.Found(L"nohacks");
 
+	Overrides.ProfilingMode = parser.Found(L"profiling");
+
 	if (parser.Found(L"gamefixes", &dest))
 	{
 		Overrides.ApplyCustomGamefixes = true;
@@ -344,6 +366,7 @@ bool Pcsx2App::OnCmdLineParsed( wxCmdLineParser& parser )
 	if( parser.GetParamCount() >= 1 )
 	{
 		Startup.IsoFile		= parser.GetParam( 0 );
+		Startup.CdvdSource	= CDVD_SourceType::Iso;
 		Startup.SysAutoRun	= true;
 	}
 	else
@@ -352,15 +375,28 @@ bool Pcsx2App::OnCmdLineParsed( wxCmdLineParser& parser )
 		if (parser.Found(L"elf", &elf_file) && !elf_file.IsEmpty()) {
 			Startup.SysAutoRunElf = true;
 			Startup.ElfFile = elf_file;
+		} else if (parser.Found(L"irx", &elf_file) && !elf_file.IsEmpty()) {
+			Startup.SysAutoRunIrx = true;
+			Startup.ElfFile = elf_file;
 		}
 	}
-	
+
+	wxString game_args;
+	if (parser.Found(L"gameargs", &game_args) && !game_args.IsEmpty())
+		Startup.GameLaunchArgs = game_args;
+
 	if( parser.Found(L"usecd") )
 	{
-		Startup.CdvdSource	= CDVDsrc_Plugin;
+		Startup.CdvdSource	= CDVD_SourceType::Plugin;
 		Startup.SysAutoRun	= true;
 	}
 
+	if (parser.Found(L"nodisc"))
+	{
+		Startup.CdvdSource = CDVD_SourceType::NoDisc;
+		Startup.SysAutoRun = true;
+	}
+
 	return true;
 }
 
@@ -381,7 +417,7 @@ public:
 	{
 	}
 
-	virtual ~GameDatabaseLoaderThread() throw()
+	virtual ~GameDatabaseLoaderThread()
 	{
 		try {
 			_parent::Cancel();
@@ -418,7 +454,7 @@ bool Pcsx2App::OnInit()
 	pxDoAssert		= AppDoAssert;
 	pxDoOutOfMemory	= SysOutOfMemory_EmergencyResponse;
 
-	g_Conf = new AppConfig();
+	g_Conf = std::make_unique<AppConfig>();
     wxInitAllImageHandlers();
 
 	Console.WriteLn("Applying operating system default language...");
@@ -435,11 +471,8 @@ bool Pcsx2App::OnInit()
 
 	i18n_SetLanguagePath();
 
-#define pxAppMethodEventHandler(func) \
-	(wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent(pxInvokeAppMethodEventFunction, &func )
-
-	Connect( pxID_PadHandler_Keydown,	wxEVT_KEY_DOWN,		wxKeyEventHandler			(Pcsx2App::OnEmuKeyDown) );
-	Connect(							wxEVT_DESTROY,		wxWindowDestroyEventHandler	(Pcsx2App::OnDestroyWindow) );
+	Bind(wxEVT_KEY_DOWN, &Pcsx2App::OnEmuKeyDown, this, pxID_PadHandler_Keydown);
+	Bind(wxEVT_DESTROY, &Pcsx2App::OnDestroyWindow, this);
 
 	// User/Admin Mode Dual Setup:
 	//   PCSX2 now supports two fundamental modes of operation.  The default is Classic mode,
@@ -468,40 +501,49 @@ bool Pcsx2App::OnInit()
 		// PCSX2 has a lot of event handling logistics, so we *cannot* depend on wxWidgets automatic event
 		// loop termination code.  We have a much safer system in place that continues to process messages
 		// until all "important" threads are closed out -- not just until the main frame is closed(-ish).
-		m_timer_Termination = new wxTimer( this, wxID_ANY );
-		Connect( m_timer_Termination->GetId(), wxEVT_TIMER, wxTimerEventHandler(Pcsx2App::OnScheduledTermination) );
+		m_timer_Termination = std::make_unique<wxTimer>( this, wxID_ANY );
+		Bind(wxEVT_TIMER, &Pcsx2App::OnScheduledTermination, this, m_timer_Termination->GetId());
 		SetExitOnFrameDelete( false );
 
 
 		//   Start GUI and/or Direct Emulation
 		// -------------------------------------
+		pxSizerFlags::SetBestPadding();
 		if( Startup.ForceConsole ) g_Conf->ProgLogBox.Visible = true;
 		OpenProgramLog();
 		AllocateCoreStuffs();
 		if( m_UseGUI ) OpenMainFrame();
 
-		
+
 		(new GameDatabaseLoaderThread())->Start();
 
+		// By default no IRX injection
+		g_Conf->CurrentIRX = "";
+
 		if( Startup.SysAutoRun )
 		{
-			// Notes: Saving/remembering the Iso file is probably fine and desired, so using
-			// SysUpdateIsoSrcFile is good(ish).
-			// Saving the cdvd plugin override isn't desirable, so we don't assign it into g_Conf.
-
 			g_Conf->EmuOptions.UseBOOT2Injection = !Startup.NoFastBoot;
-			SysUpdateIsoSrcFile( Startup.IsoFile );
+			g_Conf->CdvdSource = Startup.CdvdSource;
+			if (Startup.CdvdSource == CDVD_SourceType::Iso)
+				SysUpdateIsoSrcFile( Startup.IsoFile );
 			sApp.SysExecute( Startup.CdvdSource );
+			g_Conf->CurrentGameArgs = Startup.GameLaunchArgs;
 		}
 		else if ( Startup.SysAutoRunElf )
 		{
 			g_Conf->EmuOptions.UseBOOT2Injection = true;
-			// Enable iop/ee logging
-			SysConsole.eeConsole.Enabled = true;
-			SysConsole.iopConsole.Enabled = true;
 
 			sApp.SysExecute( Startup.CdvdSource, Startup.ElfFile );
 		}
+		else if (Startup.SysAutoRunIrx )
+		{
+			g_Conf->EmuOptions.UseBOOT2Injection = true;
+
+			g_Conf->CurrentIRX = Startup.ElfFile;
+
+			// FIXME: ElfFile is an irx it will crash
+			sApp.SysExecute( Startup.CdvdSource, Startup.ElfFile );
+		}
 	}
 	// ----------------------------------------------------------------------------
 	catch( Exception::StartupAborted& ex )		// user-aborted, no popups needed.
