@@ -31,6 +31,11 @@
 
 #include "Debugger/DisassemblyDialog.h"
 
+#ifndef DISABLE_RECORDING
+#	include "Recording/RecordingControls.h"
+#	include "Recording/InputRecording.h"
+#endif
+
 #include "Utilities/IniInterface.h"
 #include "Utilities/AppTrait.h"
 
@@ -41,24 +46,30 @@
 #endif
 
 #ifdef __WXGTK__
-
-#if wxMAJOR_VERSION < 3
-#include <wx/gtk/win_gtk.h> // GTK_PIZZA interface (internal include removed in 3.0)
-#endif
-
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #endif
 
+// Safe to remove these lines when this is handled properly.
+#ifdef __WXMAC__
+// Great joy....
+#undef EBP
+#undef ESP
+#undef EDI
+#undef ESI
+#undef EDX
+#undef EAX
+#undef EBX
+#undef ECX
+#include <wx/osx/private.h>		// needed to implement the app!
+#endif
 
-IMPLEMENT_APP(Pcsx2App)
+wxIMPLEMENT_APP(Pcsx2App);
 
-DEFINE_EVENT_TYPE( pxEvt_LoadPluginsComplete );
-DEFINE_EVENT_TYPE( pxEvt_LogicalVsync );
+std::unique_ptr<AppConfig> g_Conf;
 
-DEFINE_EVENT_TYPE( pxEvt_ThreadTaskTimeout_SysExec );
-
-ScopedPtr<AppConfig>	g_Conf;
+AspectRatioType iniAR;
+bool switchAR;
 
 static bool HandlePluginError( BaseException& ex )
 {
@@ -89,7 +100,7 @@ public:
 	PluginErrorEvent( BaseException* ex=NULL ) : _parent( ex ) {}
 	PluginErrorEvent( const BaseException& ex ) : _parent( ex ) {}
 
-	virtual ~PluginErrorEvent() throw() { }
+	virtual ~PluginErrorEvent() = default;
 	virtual PluginErrorEvent *Clone() const { return new PluginErrorEvent(*this); }
 
 protected:
@@ -104,7 +115,7 @@ public:
 	PluginInitErrorEvent( BaseException* ex=NULL ) : _parent( ex ) {}
 	PluginInitErrorEvent( const BaseException& ex ) : _parent( ex ) {}
 
-	virtual ~PluginInitErrorEvent() throw() { }
+	virtual ~PluginInitErrorEvent() = default;
 	virtual PluginInitErrorEvent *Clone() const { return new PluginInitErrorEvent(*this); }
 
 protected:
@@ -157,7 +168,7 @@ public:
 	BIOSLoadErrorEvent(BaseException* ex = NULL) : _parent(ex) {}
 	BIOSLoadErrorEvent(const BaseException& ex) : _parent(ex) {}
 
-	virtual ~BIOSLoadErrorEvent() throw() { }
+	virtual ~BIOSLoadErrorEvent() = default;
 	virtual BIOSLoadErrorEvent *Clone() const { return new BIOSLoadErrorEvent(*this); }
 
 protected:
@@ -204,7 +215,7 @@ void Pcsx2App::PostMenuAction( MenuIdentifiers menu_id ) const
 	MainEmuFrame* mainFrame = GetMainFramePtr();
 	if( mainFrame == NULL ) return;
 
-	wxCommandEvent joe( wxEVT_COMMAND_MENU_SELECTED, menu_id );
+	wxCommandEvent joe( wxEVT_MENU, menu_id );
 	if( wxThread::IsMain() )
 		mainFrame->GetEventHandler()->ProcessEvent( joe );
 	else
@@ -221,13 +232,13 @@ void Pcsx2App::PostMenuAction( MenuIdentifiers menu_id ) const
 class Pcsx2AppMethodEvent : public pxActionEvent
 {
 	typedef pxActionEvent _parent;
-	DECLARE_DYNAMIC_CLASS_NO_ASSIGN(Pcsx2AppMethodEvent)
+	wxDECLARE_DYNAMIC_CLASS_NO_ASSIGN(Pcsx2AppMethodEvent);
 
 protected:
 	FnPtr_Pcsx2App	m_Method;
 
 public:
-	virtual ~Pcsx2AppMethodEvent() throw() { }
+	virtual ~Pcsx2AppMethodEvent() = default;
 	virtual Pcsx2AppMethodEvent *Clone() const { return new Pcsx2AppMethodEvent(*this); }
 
 	explicit Pcsx2AppMethodEvent( FnPtr_Pcsx2App method=NULL, SynchronousActionState* sema=NULL )
@@ -261,7 +272,7 @@ protected:
 };
 
 
-IMPLEMENT_DYNAMIC_CLASS( Pcsx2AppMethodEvent, pxActionEvent )
+wxIMPLEMENT_DYNAMIC_CLASS( Pcsx2AppMethodEvent, pxActionEvent );
 
 #ifdef __WXMSW__
 extern int TranslateVKToWXK( u32 keysym );
@@ -272,25 +283,24 @@ extern int TranslateGDKtoWXK( u32 keysym );
 void Pcsx2App::PadKeyDispatch( const keyEvent& ev )
 {
 	m_kevt.SetEventType( ( ev.evt == KEYPRESS ) ? wxEVT_KEY_DOWN : wxEVT_KEY_UP );
-	const bool isDown = (ev.evt == KEYPRESS);
 
 //returns 0 for normal keys and a WXK_* value for special keys
 #ifdef __WXMSW__
 	const int vkey = TranslateVKToWXK(ev.key);
+#elif defined( __WXMAC__ )
+	const int vkey = wxCharCodeWXToOSX( (wxKeyCode) ev.key );
 #elif defined( __WXGTK__ )
 	const int vkey = TranslateGDKtoWXK( ev.key );
 #else
 #	error Unsupported Target Platform.
 #endif
 
-	switch (vkey)
-	{
-		case WXK_SHIFT:		m_kevt.m_shiftDown		= isDown; return;
-		case WXK_CONTROL:	m_kevt.m_controlDown	= isDown; return;
-
-		case WXK_ALT:		// ALT/MENU are usually the same key?  I'm confused.
-		case WXK_MENU:		m_kevt.m_altDown		= isDown; return;
-	}
+	// Don't rely on current event handling to get the state of those specials keys.
+	// Typical linux bug: hit ctrl-alt key to switch the desktop. Key will be released
+	// outside of the window so the app isn't aware of the current key state.
+	m_kevt.m_shiftDown = wxGetKeyState(WXK_SHIFT);
+	m_kevt.m_controlDown = wxGetKeyState(WXK_CONTROL);
+	m_kevt.m_altDown = wxGetKeyState(WXK_MENU) || wxGetKeyState(WXK_ALT);
 
 	m_kevt.m_keyCode = vkey? vkey : ev.key;
 
@@ -415,7 +425,7 @@ public:
 		return Path::Combine( GetDataDir(), L"Langs" );
 	}
 
-#ifdef __linux__
+#ifdef __POSIX__
 	wxString GetUserLocalDataDir() const
 	{
 		// I got memory corruption inside wxGetEnv when I heavily toggle the GS renderer (F9). It seems wxGetEnv
@@ -448,11 +458,7 @@ public:
 
 };
 
-#if wxMAJOR_VERSION < 3
-wxStandardPathsBase& Pcsx2AppTraits::GetStandardPaths()
-#else
 wxStandardPaths& Pcsx2AppTraits::GetStandardPaths()
-#endif
 {
 	static Pcsx2StandardPaths stdPaths;
 	return stdPaths;
@@ -511,14 +517,29 @@ double FramerateManager::GetFramerate() const
 // times a second if not (ok, not quite, but you get the idea... I hope.)
 extern uint eecount_on_last_vdec;
 extern bool FMVstarted;
-extern bool renderswitch;
 extern bool EnableFMV;
+extern bool renderswitch;
+extern uint renderswitch_delay;
 
-void DoFmvSwitch()
+void DoFmvSwitch(bool on)
 {
-	ScopedCoreThreadPause paused_core( new SysExecEvent_SaveSinglePlugin(PluginId_GS) );
-	renderswitch = !renderswitch;
-	paused_core.AllowResume();
+	if (g_Conf->GSWindow.FMVAspectRatioSwitch != FMV_AspectRatio_Switch_Off) {
+		if (on) {
+			switchAR = true;
+			iniAR = g_Conf->GSWindow.AspectRatio;
+		} else {
+			switchAR = false;
+		}
+		if (GSFrame* gsFrame = wxGetApp().GetGsFramePtr())
+			if (GSPanel* viewport = gsFrame->GetViewport())
+				viewport->DoResize();
+	}
+
+	if (EmuConfig.Gamefixes.FMVinSoftwareHack) {
+		ScopedCoreThreadPause paused_core(new SysExecEvent_SaveSinglePlugin(PluginId_GS));
+		renderswitch = !renderswitch;
+		paused_core.AllowResume();
+	}
 }
 
 void Pcsx2App::LogicalVsync()
@@ -530,24 +551,26 @@ void Pcsx2App::LogicalVsync()
 	// Update / Calculate framerate!
 
 	FpsManager.DoFrame();
-	
-	if (EmuConfig.Gamefixes.FMVinSoftwareHack) {
-		if (EnableFMV == 1) {
-			Console.Warning("FMV on");
-			DoFmvSwitch();
-			EnableFMV = 0;
+
+	if (EmuConfig.Gamefixes.FMVinSoftwareHack || g_Conf->GSWindow.FMVAspectRatioSwitch != FMV_AspectRatio_Switch_Off) {
+		if (EnableFMV) {
+			DevCon.Warning("FMV on");
+			DoFmvSwitch(true);
+			EnableFMV = false;
 		}
 
-		if (FMVstarted){
+		if (FMVstarted) {
 			int diff = cpuRegs.cycle - eecount_on_last_vdec;
 			if (diff > 60000000 ) {
-				Console.Warning("FMV off");
-				DoFmvSwitch();
-				FMVstarted = 0;
+				DevCon.Warning("FMV off");
+				DoFmvSwitch(false);
+				FMVstarted = false;
 			}
 		}
 	}
 
+	renderswitch_delay >>= 1;
+
 	// Only call PADupdate here if we're using GSopen2.  Legacy GSopen plugins have the
 	// GS window belonging to the MTGS thread.
 	if( (PADupdate != NULL) && (GSopen2 != NULL) && (wxGetApp().GetGsFramePtr() != NULL) )
@@ -592,7 +615,28 @@ void Pcsx2App::HandleEvent(wxEvtHandler* handler, wxEventFunction func, wxEvent&
 
 void Pcsx2App::HandleEvent(wxEvtHandler* handler, wxEventFunction func, wxEvent& event)
 {
-	try {
+	try
+	{
+#ifndef DISABLE_RECORDING
+		if (g_Conf->EmuOptions.EnableRecordingTools)
+		{
+			if (g_RecordingControls.HasRecordingStopped())
+			{
+				// While stopping, GSFrame key event also stops, so get key input from here
+				// Along with that, you can not use the shortcut keys set in GSFrame
+				if (PADkeyEvent != NULL)
+				{
+					// Acquire key information, possibly calling it only once per frame
+					const keyEvent* ev = PADkeyEvent();
+					if (ev != NULL)
+					{
+						sApp.Recording_PadKeyDispatch(*ev);
+					}
+				}
+			}
+			g_RecordingControls.ResumeCoreThreadIfStarted();
+		}
+#endif
 		(handler->*func)(event);
 	}
 	// ----------------------------------------------------------------------------
@@ -633,10 +677,10 @@ void Pcsx2App::HandleEvent(wxEvtHandler* handler, wxEventFunction func, wxEvent&
 		// PCSX2. This probably happened in the BIOS error case above as well.
 		// So the idea is to explicitly close the gsFrame before the modal MessageBox appears and
 		// intercepts the close message. Only for wx3.0 though - it sometimes breaks linux wx2.8.
-#if wxMAJOR_VERSION >= 3
+
 		if (GSFrame* gsframe = wxGetApp().GetGsFramePtr())
 			gsframe->Close();
-#endif
+
 		Console.Error(ex.FormatDiagnosticMessage());
 
 		// Make sure it terminates properly for nogui users.
@@ -889,7 +933,7 @@ void Pcsx2App::PostIdleAppMethod( FnPtr_Pcsx2App method )
 
 SysMainMemory& Pcsx2App::GetVmReserve()
 {
-	if (!m_VmReserve) m_VmReserve = new SysMainMemory();
+	if (!m_VmReserve) m_VmReserve = std::unique_ptr<SysMainMemory>(new SysMainMemory());
 	return *m_VmReserve;
 }
 
@@ -954,20 +998,12 @@ void Pcsx2App::OpenGsPanel()
 	// unfortunately it creates a gray box in the middle of the window on some
 	// users.
 
-#if wxMAJOR_VERSION < 3
-	GtkWidget *child_window = gtk_bin_get_child(GTK_BIN(gsFrame->GetViewport()->GetHandle()));
-#else
 	GtkWidget *child_window = GTK_WIDGET(gsFrame->GetViewport()->GetHandle());
-#endif
 
 	gtk_widget_realize(child_window); // create the widget to allow to use GDK_WINDOW_* macro
 	gtk_widget_set_double_buffered(child_window, false); // Disable the widget double buffer, you will use the opengl one
 
-#if wxMAJOR_VERSION < 3
-	GdkWindow* draw_window = GTK_PIZZA(child_window)->bin_window;
-#else
 	GdkWindow* draw_window = gtk_widget_get_window(child_window);
-#endif
 
 #if GTK_MAJOR_VERSION < 3
 	Window Xwindow = GDK_WINDOW_XWINDOW(draw_window);
@@ -1023,6 +1059,13 @@ void Pcsx2App::OnProgramLogClosed( wxWindowID id )
 
 void Pcsx2App::OnMainFrameClosed( wxWindowID id )
 {
+#ifndef DISABLE_RECORDING
+	if (g_Conf->EmuOptions.EnableRecordingTools)
+	{
+		g_InputRecording.Stop();
+	}
+#endif
+
 	// Nothing threaded depends on the mainframe (yet) -- it all passes through the main wxApp
 	// message handler.  But that might change in the future.
 	if( m_id_MainFrame != id ) return;
@@ -1041,7 +1084,7 @@ protected:
 	wxString			m_elf_override;
 
 public:
-	virtual ~SysExecEvent_Execute() throw() {}
+	virtual ~SysExecEvent_Execute() = default;
 	SysExecEvent_Execute* Clone() const { return new SysExecEvent_Execute(*this); }
 
 	wxString GetEventName() const
@@ -1057,7 +1100,7 @@ public:
 	SysExecEvent_Execute()
 		: m_UseCDVDsrc(false)
 		, m_UseELFOverride(false)
-		, m_cdvdsrc_type(CDVDsrc_Iso)
+		, m_cdvdsrc_type(CDVD_SourceType::Iso)
 	{
 	}
 
@@ -1082,12 +1125,13 @@ protected:
 
 		CoreThread.ResetQuick();
 		symbolMap.Clear();
+		CBreakPoints::SetSkipFirst(0);
 
-		CDVDsys_SetFile( CDVDsrc_Iso, g_Conf->CurrentIso );
+		CDVDsys_SetFile(CDVD_SourceType::Iso, g_Conf->CurrentIso );
 		if( m_UseCDVDsrc )
 			CDVDsys_ChangeSource( m_cdvdsrc_type );
 		else if( CDVD == NULL )
-			CDVDsys_ChangeSource( CDVDsrc_NoDisc );
+			CDVDsys_ChangeSource(CDVD_SourceType::NoDisc);
 
 		if( m_UseELFOverride && !CoreThread.HasActiveMachine() )
 			CoreThread.SetElfOverride( m_elf_override );
