@@ -16,10 +16,9 @@
 #pragma once
 
 #include "App.h"
-
-BEGIN_DECLARE_EVENT_TYPES()
-	DECLARE_EVENT_TYPE(pxEvt_DockConsole, -1)
-END_DECLARE_EVENT_TYPES()
+#include <array>
+#include <map>
+#include <memory>
 
 static const bool EnableThreadedLoggingTest = false; //true;
 
@@ -36,7 +35,7 @@ class PipeRedirectionBase
 	DeclareNoncopyableObject( PipeRedirectionBase );
 
 public:
-	virtual ~PipeRedirectionBase() throw()=0;	// abstract destructor, forces abstract class behavior
+	virtual ~PipeRedirectionBase() =0;	// abstract destructor, forces abstract class behavior
 
 protected:
 	PipeRedirectionBase() {}
@@ -57,11 +56,7 @@ public:
 	pxLogConsole() {}
 
 protected:
-#if wxMAJOR_VERSION >= 3
 	virtual void DoLogRecord(wxLogLevel level, const wxString &message, const wxLogRecordInfo &info);
-#else
-	virtual void DoLog(wxLogLevel level, const wxChar *szString, time_t t);
-#endif
 };
 
 
@@ -75,7 +70,7 @@ class ConsoleTestThread : public Threading::pxThread
 	typedef pxThread _parent;
 
 protected:
-	volatile bool m_done;
+	std::atomic<bool> m_done;
 	void ExecuteTaskInThread();
 
 public:
@@ -84,43 +79,12 @@ public:
 	{
 	}
 
-	~ConsoleTestThread() throw()
+	~ConsoleTestThread()
 	{
 		m_done = true;
 	}
 };
 
-// --------------------------------------------------------------------------------------
-//  pxLogTextCtrl
-// --------------------------------------------------------------------------------------
-class pxLogTextCtrl : public wxTextCtrl,
-	public EventListener_CoreThread,
-	public EventListener_Plugins
-{
-protected:
-	ScopedPtr<ScopedCoreThreadPause> m_IsPaused;
-	bool m_FreezeWrites;
-
-public:
-	pxLogTextCtrl(wxWindow* parent);
-	virtual ~pxLogTextCtrl() throw();
-
-	bool HasWriteLock() const { return m_FreezeWrites; }
-	void ConcludeIssue();
-
-#ifdef __WXMSW__
-	virtual void WriteText(const wxString& text);
-#endif
-
-protected:
-	virtual void OnThumbTrack(wxScrollWinEvent& event);
-	virtual void OnThumbRelease(wxScrollWinEvent& event);
-	virtual void OnResize( wxSizeEvent& evt );
-
-	void DispatchEvent( const CoreThreadStatus& status );
-	void DispatchEvent( const PluginEventType& evt );
-};
-
 // --------------------------------------------------------------------------------------
 //  ConsoleLogFrame  --  Because the one built in wx is poop.
 // --------------------------------------------------------------------------------------
@@ -137,18 +101,15 @@ protected:
 		DeclareNoncopyableObject(ColorArray);
 
 	protected:
-		SafeArray<wxTextAttr>	m_table;
-		wxTextAttr				m_color_default;
+		std::array<wxTextAttr, ConsoleColors_Count> m_table;
 
 	public:
-		virtual ~ColorArray() throw();
+		virtual ~ColorArray() = default;
 		ColorArray( int fontsize=8 );
 
-		void Create( int fontsize );
-		void Cleanup();
-
 		void SetFont( const wxFont& font );
 		void SetFont( int fontsize );
+		u32 GetRGBA( const ConsoleColors color );
 
 		const wxTextAttr& operator[]( ConsoleColors coloridx ) const
 		{
@@ -171,29 +132,29 @@ protected:
 
 protected:
 	ConLogConfig&	m_conf;
-	pxLogTextCtrl&	m_TextCtrl;
+	wxTextCtrl&		m_TextCtrl;
 	wxTimer			m_timer_FlushLimiter;
 	wxTimer			m_timer_FlushUnlocker;
 	ColorArray		m_ColorTable;
 
 	int				m_flushevent_counter;
 	bool			m_FlushRefreshLocked;
-	
+
 	// ----------------------------------------------------------------------------
 	//  Queue State Management Vars
 	// ----------------------------------------------------------------------------
 
 	// Boolean indicating if a flush message is already in the Main message queue.  Used
 	// to prevent spamming the main thread with redundant messages.
-	volatile bool			m_pendingFlushMsg;
+	std::atomic<bool>			m_pendingFlushMsg;
 
 	// This is a counter of the number of threads waiting for the Queue to flush.
-	volatile int			m_WaitingThreadsForFlush;
+	std::atomic<int>			m_WaitingThreadsForFlush;
 
 	// Indicates to the main thread if a child thread is actively writing to the log.  If
 	// true the main thread will sleep briefly to allow the child a chance to accumulate
 	// more messages (helps avoid rapid successive flushes on high volume logging).
-	volatile bool			m_ThreadedLogInQueue;
+	std::atomic<bool>			m_ThreadedLogInQueue;
 
 	// Used by threads waiting on the queue to flush.
 	Semaphore				m_sem_QueueFlushed;
@@ -215,28 +176,23 @@ protected:
 	// Threaded log spammer, useful for testing console logging performance.
 	// (alternatively you can enable Disasm logging in any recompiler and achieve
 	// a similar effect)
-	ScopedPtr<ConsoleTestThread>	m_threadlogger;
-
-	// ----------------------------------------------------------------------------
-	//  Window and Menu Object Handles
-	// ----------------------------------------------------------------------------
-
-	ScopedArray<wxMenuItem*>	m_sourceChecks;
+	std::unique_ptr<ConsoleTestThread> m_threadlogger;
 
 public:
 	// ctor & dtor
 	ConsoleLogFrame( MainEmuFrame *pParent, const wxString& szTitle, ConLogConfig& options );
 	virtual ~ConsoleLogFrame();
 
-	virtual void DockedMove();
-
 	// Retrieves the current configuration options settings for this box.
 	// (settings change if the user moves the window or changes the font size)
 	const ConLogConfig& GetConfig() const { return m_conf; }
+	u32 GetRGBA( const ConsoleColors color ) { return m_ColorTable.GetRGBA( color ); }
 
 	bool Write( ConsoleColors color, const wxString& text );
 	bool Newline();
 
+	void UpdateLogList();
+
 protected:
 	// menu callbacks
 	void OnOpen (wxCommandEvent& event);
@@ -250,13 +206,13 @@ protected:
 
 	void OnToggleTheme(wxCommandEvent& event);
 	void OnFontSize(wxCommandEvent& event);
+	void OnAutoDock(wxCommandEvent& event);
 	void OnToggleSource(wxCommandEvent& event);
 	void OnToggleCDVDInfo(wxCommandEvent& event);
 
 	virtual void OnCloseWindow(wxCloseEvent& event);
 
 	void OnSetTitle( wxCommandEvent& event );
-	void OnDockedMove( wxCommandEvent& event );
 	void OnFlushUnlockerTimer( wxTimerEvent& evt );
 	void OnFlushEvent( wxCommandEvent& event );
 
@@ -266,6 +222,22 @@ protected:
 	void OnMoveAround( wxMoveEvent& evt );
 	void OnResize( wxSizeEvent& evt );
 	void OnActivate( wxActivateEvent& evt );
-	
+
 	void OnLoggingChanged();
 };
+
+void OSDlog(ConsoleColors color, bool console, const std::string& str);
+
+template<typename ... Args>
+void OSDlog(ConsoleColors color, bool console, const std::string& format, Args ... args) {
+	if (!GSosdLog && !console) return;
+
+	size_t size = snprintf( nullptr, 0, format.c_str(), args ... ) + 1; // Extra space for '\0'
+	std::vector<char> buf(size);
+	snprintf( buf.data(), size, format.c_str(), args ... );
+
+	OSDlog(color, console, buf.data());
+}
+
+void OSDmonitor(ConsoleColors color, const std::string key, const std::string value);
+
