@@ -54,10 +54,11 @@ public:
 	void showMemoryView() { setBottomTabPage(memory); };
 	void loadCycles();
 	void reloadSymbolMap();
+	void clearSymbolMap();
 	u32 getStepOutAddress();
 
 	void listBoxHandler(wxCommandEvent& event);
-	DECLARE_EVENT_TABLE()
+	wxDECLARE_EVENT_TABLE();
 private:
 	void setBottomTabPage(wxWindow* win);
 	void postEvent(wxEventType type, int value);
@@ -74,26 +75,28 @@ private:
 	ThreadList* threadList;
 	StackFramesList* stackFrames;
 	u32 lastCycles;
+	u32 symbolCount;
 };
 
 class DisassemblyDialog : public wxFrame
 {
 public:
 	DisassemblyDialog( wxWindow* parent=NULL );
-	virtual ~DisassemblyDialog() throw() {}
+	virtual ~DisassemblyDialog() = default;
 
 	static wxString GetNameStatic() { return L"DisassemblyDialog"; }
 	wxString GetDialogName() const { return GetNameStatic(); }
 	
 	void update();
 	void reset();
+	void populate();
 	void setDebugMode(bool debugMode, bool switchPC);
 	
-#ifdef WIN32
+#ifdef _WIN32
 	WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
 #endif
 
-	DECLARE_EVENT_TABLE()
+	wxDECLARE_EVENT_TABLE();
 protected:
 	void onBreakRunClicked(wxCommandEvent& evt);
 	void onStepOverClicked(wxCommandEvent& evt);
