@@ -15,9 +15,12 @@
 
 #include "PrecompiledHeader.h"
 
+#include "App.h"
 #include "CDVD/CDVD.h"
 #include "GS.h"
+#include "GSFrame.h"
 
+#include "ConsoleLogger.h"
 #include "MainFrame.h"
 #include "IsoDropTarget.h"
 
@@ -28,6 +31,11 @@
 
 #include "Utilities/IniInterface.h"
 
+#ifndef DISABLE_RECORDING
+#	include "Recording/InputRecording.h"
+#	include "Recording/VirtualPad.h"
+#endif
+
 using namespace Dialogs;
 
 void MainEmuFrame::Menu_SysSettings_Click(wxCommandEvent &event)
@@ -37,12 +45,9 @@ void MainEmuFrame::Menu_SysSettings_Click(wxCommandEvent &event)
 
 void MainEmuFrame::Menu_McdSettings_Click(wxCommandEvent &event)
 {
-	AppOpenDialog<McdConfigDialog>( this );
-}
-
-void MainEmuFrame::Menu_GameDatabase_Click(wxCommandEvent &event)
-{
-	AppOpenDialog<GameDatabaseDialog>( this );
+	ScopedCoreThreadClose closed_core;
+	closed_core.AllowResume();
+	AppOpenModalDialog<McdConfigDialog>(wxEmptyString, this);
 }
 
 void MainEmuFrame::Menu_WindowSettings_Click(wxCommandEvent &event)
@@ -64,12 +69,6 @@ void MainEmuFrame::Menu_SelectPluginsBios_Click(wxCommandEvent &event)
 	AppOpenDialog<ComponentsConfigDialog>( this );
 }
 
-void MainEmuFrame::Menu_Language_Click(wxCommandEvent &event)
-{
-	//AppOpenDialog<InterfaceConfigDialog>( this );
-	InterfaceConfigDialog(this).ShowModal();
-}
-
 void MainEmuFrame::Menu_ChangeLang(wxCommandEvent &event) // Always in English
 {
 	AppOpenDialog<InterfaceLanguageDialog>(this);
@@ -90,7 +89,7 @@ static void WipeSettings()
 	//wxRmdir( GetSettingsFolder().ToString() );
 
 	wxGetApp().GetRecentIsoManager().Clear();
-	g_Conf = new AppConfig();
+	g_Conf = std::unique_ptr<AppConfig>(new AppConfig());
 	sMainFrame.RemoveCdvdMenu();
 
 	sApp.WipeUserModeSettings();
@@ -98,8 +97,11 @@ static void WipeSettings()
 
 void MainEmuFrame::RemoveCdvdMenu()
 {
-	if( wxMenuItem* item = m_menuCDVD.FindItem(MenuId_IsoSelector) )
-		m_menuCDVD.Remove( item );
+	// Delete() keeps the sub menu and delete the menu item.
+	// Remove() does not delete the menu item.
+	if (m_menuItem_RecentIsoMenu)
+		m_menuCDVD.Delete(m_menuItem_RecentIsoMenu);
+	m_menuItem_RecentIsoMenu = nullptr;
 }
 
 void MainEmuFrame::Menu_ResetAllSettings_Click(wxCommandEvent &event)
@@ -130,7 +132,7 @@ wxWindowID SwapOrReset_Iso( wxWindow* owner, IScopedCoreThread& core_control, co
 {
 	wxWindowID result = wxID_CANCEL;
 
-	if( (g_Conf->CdvdSource == CDVDsrc_Iso) && (isoFilename == g_Conf->CurrentIso) )
+	if( (g_Conf->CdvdSource == CDVD_SourceType::Iso) && (isoFilename == g_Conf->CurrentIso) )
 	{
 		core_control.AllowResume();
 		return result;
@@ -147,7 +149,7 @@ wxWindowID SwapOrReset_Iso( wxWindow* owner, IScopedCoreThread& core_control, co
 		dialog += dialog.GetCharHeight();
 		dialog += dialog.Heading(_("Do you want to swap discs or boot the new image (via system reset)?"));
 
-		result = pxIssueConfirmation( dialog, MsgButtons().Reset().Cancel().Custom(_("Swap Disc"), "swap"), L"DragDrop.BootSwapIso" );
+		result = pxIssueConfirmation( dialog, MsgButtons().Reset().Cancel().Custom(_("Swap Disc"), "swap"));
 		if( result == wxID_CANCEL )
 		{
 			core_control.AllowResume();
@@ -155,12 +157,12 @@ wxWindowID SwapOrReset_Iso( wxWindow* owner, IScopedCoreThread& core_control, co
 		}
 	}
 
-	g_Conf->CdvdSource = CDVDsrc_Iso;
+	g_Conf->CdvdSource = CDVD_SourceType::Iso;
 	SysUpdateIsoSrcFile( isoFilename );
 	if( result == wxID_RESET )
 	{
 		core_control.DisallowResume();
-		sApp.SysExecute( CDVDsrc_Iso );
+		sApp.SysExecute(CDVD_SourceType::Iso);
 	}
 	else
 	{
@@ -170,7 +172,7 @@ wxWindowID SwapOrReset_Iso( wxWindow* owner, IScopedCoreThread& core_control, co
 		core_control.AllowResume();
 	}
 
-	GetMainFrame().EnableCdvdPluginSubmenu( g_Conf->CdvdSource == CDVDsrc_Plugin );
+	GetMainFrame().EnableCdvdPluginSubmenu( g_Conf->CdvdSource == CDVD_SourceType::Plugin);
 
 	return result;
 }
@@ -187,13 +189,13 @@ wxWindowID SwapOrReset_CdvdSrc( wxWindow* owner, CDVD_SourceType newsrc )
 
 		wxString changeMsg;
 		changeMsg.Printf(_("You've selected to switch the CDVD source from %s to %s."),
-			CDVD_SourceLabels[g_Conf->CdvdSource], CDVD_SourceLabels[newsrc] );
+			CDVD_SourceLabels[enum_cast(g_Conf->CdvdSource)], CDVD_SourceLabels[enum_cast(newsrc)] );
 
 		dialog += dialog.Heading(changeMsg + L"\n\n" +
 			_("Do you want to swap discs or boot the new image (system reset)?")
 		);
 
-		result = pxIssueConfirmation( dialog, MsgButtons().Reset().Cancel().Custom(_("Swap Disc"), "swap"), L"DragDrop.BootSwapIso" );
+		result = pxIssueConfirmation( dialog, MsgButtons().Reset().Cancel().Custom(_("Swap Disc"), "swap"));
 
 		if( result == wxID_CANCEL )
 		{
@@ -208,7 +210,9 @@ wxWindowID SwapOrReset_CdvdSrc( wxWindow* owner, CDVD_SourceType newsrc )
 
 	if( result != wxID_RESET )
 	{
-		Console.Indent().WriteLn( L"(CdvdSource) HotSwapping CDVD source types from %s to %s.", CDVD_SourceLabels[oldsrc], CDVD_SourceLabels[newsrc] );
+		Console.Indent().WriteLn(L"(CdvdSource) HotSwapping CDVD source types from %s to %s.",
+			WX_STR(wxString(CDVD_SourceLabels[enum_cast(oldsrc)])),
+			WX_STR(wxString(CDVD_SourceLabels[enum_cast(newsrc)])));
 		//CoreThread.ChangeCdvdSource();
 		sMainFrame.UpdateIsoSrcSelection();
 		core.AllowResume();
@@ -219,7 +223,7 @@ wxWindowID SwapOrReset_CdvdSrc( wxWindow* owner, CDVD_SourceType newsrc )
 		sApp.SysExecute( g_Conf->CdvdSource );
 	}
 
-	GetMainFrame().EnableCdvdPluginSubmenu( g_Conf->CdvdSource == CDVDsrc_Plugin );
+	GetMainFrame().EnableCdvdPluginSubmenu( g_Conf->CdvdSource == CDVD_SourceType::Plugin );
 
 	return result;
 }
@@ -260,8 +264,8 @@ bool MainEmuFrame::_DoSelectIsoBrowser( wxString& result )
 	
 	wxArrayString isoFilterTypes;
 
-	isoFilterTypes.Add(pxsFmt(_("All Supported (%s)"), WX_STR((isoSupportedLabel + L" .dump" + L" .gz"))));
-	isoFilterTypes.Add(isoSupportedList + L";*.dump" + L";*.gz");
+	isoFilterTypes.Add(pxsFmt(_("All Supported (%s)"), WX_STR((isoSupportedLabel + L" .dump" + L" .gz" + L" .cso"))));
+	isoFilterTypes.Add(isoSupportedList + L";*.dump" + L";*.gz" + L";*.cso");
 
 	isoFilterTypes.Add(pxsFmt(_("Disc Images (%s)"), WX_STR(isoSupportedLabel) ));
 	isoFilterTypes.Add(isoSupportedList);
@@ -269,13 +273,13 @@ bool MainEmuFrame::_DoSelectIsoBrowser( wxString& result )
 	isoFilterTypes.Add(pxsFmt(_("Blockdumps (%s)"), L".dump" ));
 	isoFilterTypes.Add(L"*.dump");
 
-	isoFilterTypes.Add(pxsFmt(_("Compressed (%s)"), L".gz"));
-	isoFilterTypes.Add(L"*.gz");
+	isoFilterTypes.Add(pxsFmt(_("Compressed (%s)"), L".gz .cso"));
+	isoFilterTypes.Add(L"*.gz;*.cso");
 
 	isoFilterTypes.Add(_("All Files (*.*)"));
 	isoFilterTypes.Add(L"*.*");
 	
-	wxFileDialog ctrl( this, _("Select CDVD source iso..."), g_Conf->Folders.RunIso.ToString(), wxEmptyString,
+	wxFileDialog ctrl( this, _("Select disc image, compressed disc image, or block-dump..."), g_Conf->Folders.RunIso.ToString(), wxEmptyString,
 		JoinString(isoFilterTypes, L"|"), wxFD_OPEN | wxFD_FILE_MUST_EXIST );
 
 	if( ctrl.ShowModal() != wxID_CANCEL )
@@ -309,7 +313,7 @@ void MainEmuFrame::_DoBootCdvd()
 {
 	ScopedCoreThreadPause paused_core;
 
-	if( g_Conf->CdvdSource == CDVDsrc_Iso )
+	if( g_Conf->CdvdSource == CDVD_SourceType::Iso )
 	{
 		bool selector = g_Conf->CurrentIso.IsEmpty();
 
@@ -329,7 +333,7 @@ void MainEmuFrame::_DoBootCdvd()
 			selector = true;
 		}
 
-		if( selector )
+		if( selector || g_Conf->AskOnBoot)
 		{
 			wxString result;
 			if( !_DoSelectIsoBrowser( result ) )
@@ -365,13 +369,13 @@ void MainEmuFrame::EnableCdvdPluginSubmenu(bool isEnable)
 
 void MainEmuFrame::Menu_CdvdSource_Click( wxCommandEvent &event )
 {
-	CDVD_SourceType newsrc = CDVDsrc_NoDisc;
+	CDVD_SourceType newsrc = CDVD_SourceType::NoDisc;
 
 	switch( event.GetId() )
 	{
-		case MenuId_Src_Iso:	newsrc = CDVDsrc_Iso;		break;
-		case MenuId_Src_Plugin:	newsrc = CDVDsrc_Plugin;	break;
-		case MenuId_Src_NoDisc: newsrc = CDVDsrc_NoDisc;	break;
+		case MenuId_Src_Iso:	newsrc = CDVD_SourceType::Iso;		break;
+		case MenuId_Src_Plugin:	newsrc = CDVD_SourceType::Plugin;	break;
+		case MenuId_Src_NoDisc: newsrc = CDVD_SourceType::NoDisc;	break;
 		jNO_DEFAULT
 	}
 
@@ -410,6 +414,42 @@ void MainEmuFrame::Menu_IsoBrowse_Click( wxCommandEvent &event )
 	AppSaveSettings();		// save the new iso selection; update menus!
 }
 
+void MainEmuFrame::Menu_IsoClear_Click(wxCommandEvent &event)
+{
+	wxDialogWithHelpers dialog(this, _("Confirm clearing ISO list"));
+	dialog += dialog.Heading(_("This will clear the ISO list. If an ISO is running it will remain in the list. Continue?"));
+
+	bool confirmed = pxIssueConfirmation(dialog, MsgButtons().YesNo()) == wxID_YES;
+
+	if (confirmed)
+	{
+		// If the CDVD mode is not ISO, or the system isn't running, wipe the CurrentIso field in INI file
+		if (g_Conf->CdvdSource != CDVD_SourceType::Iso || !SysHasValidState())
+			SysUpdateIsoSrcFile("");
+		wxGetApp().GetRecentIsoManager().Clear();
+		AppSaveSettings();
+	}
+}
+
+void MainEmuFrame::Menu_Ask_On_Boot_Click(wxCommandEvent &event)
+{
+	g_Conf->AskOnBoot = event.IsChecked();
+
+	if (SysHasValidState())
+		return;
+
+	wxGetApp().GetRecentIsoManager().EnableItems(!event.IsChecked());
+	FindItemInMenuBar(MenuId_IsoBrowse)->Enable(!event.IsChecked());
+}
+
+void MainEmuFrame::Menu_Debug_CreateBlockdump_Click(wxCommandEvent &event)
+{
+	g_Conf->EmuOptions.CdvdDumpBlocks = event.IsChecked();
+	if (g_Conf->EmuOptions.CdvdDumpBlocks && SysHasValidState())
+		Console.Warning("VM must be rebooted to create a useful block dump.");
+
+	AppSaveSettings();
+}
 
 void MainEmuFrame::Menu_MultitapToggle_Click( wxCommandEvent& )
 {
@@ -436,21 +476,78 @@ void MainEmuFrame::Menu_EnableBackupStates_Click( wxCommandEvent& )
 void MainEmuFrame::Menu_EnablePatches_Click( wxCommandEvent& )
 {
 	g_Conf->EmuOptions.EnablePatches = GetMenuBar()->IsChecked( MenuId_EnablePatches );
-    AppSaveSettings();
+	AppApplySettings();
+	AppSaveSettings();
 }
 
 void MainEmuFrame::Menu_EnableCheats_Click( wxCommandEvent& )
 {
 	g_Conf->EmuOptions.EnableCheats  = GetMenuBar()->IsChecked( MenuId_EnableCheats );
-    AppSaveSettings();
+	AppApplySettings();
+	AppSaveSettings();
 }
 
 void MainEmuFrame::Menu_EnableWideScreenPatches_Click( wxCommandEvent& )
 {
 	g_Conf->EmuOptions.EnableWideScreenPatches  = GetMenuBar()->IsChecked( MenuId_EnableWideScreenPatches );
+	AppApplySettings();
 	AppSaveSettings();
 }
 
+#ifndef DISABLE_RECORDING
+void MainEmuFrame::Menu_EnableRecordingTools_Click(wxCommandEvent&)
+{
+	bool checked = GetMenuBar()->IsChecked(MenuId_EnableRecordingTools);
+	// Confirm with User
+	if (checked)
+	{
+		if (!Msgbox::OkCancel(_("Please be aware that PCSX2's input recording features are still very much a work-in-progress.\n"
+			"As a result, there may be unforeseen bugs, performance implications and instability with certain games.\n\n"
+			"These tools are provided as-is and should be enabled under your own discretion."), "Enabling Recording Tools"))
+		{
+			checked = false;
+			m_menuSys.FindChildItem(MenuId_EnableRecordingTools)->Check(false);
+		}
+	}
+
+	// If still enabled, add the menu item, else, remove it
+	if (checked)
+	{
+		GetMenuBar()->Insert(TopLevelMenu_Recording, &m_menuRecording, _("&Recording"));
+		// Enable Recording Keybindings
+		if (GSFrame* gsFrame = wxGetApp().GetGsFramePtr())
+		{
+			if (GSPanel* viewport = gsFrame->GetViewport())
+			{
+				viewport->InitRecordingAccelerators();
+			}
+		}
+	}
+	else
+	{
+		GetMenuBar()->Remove(TopLevelMenu_Recording);
+		// Always turn controller logs off, but never turn it on by default
+		SysConsole.controlInfo.Enabled = checked;
+		// Return Keybindings Back to Normal
+		if (GSFrame* gsFrame = wxGetApp().GetGsFramePtr())
+		{
+			if (GSPanel* viewport = gsFrame->GetViewport())
+			{
+				viewport->InitDefaultAccelerators();
+			}
+		}
+	}
+
+	g_Conf->EmuOptions.EnableRecordingTools = checked;
+	SysConsole.recordingConsole.Enabled = checked;
+	// Enable Recording Logs
+	ConsoleLogFrame* progLog = wxGetApp().GetProgramLog();
+	progLog->UpdateLogList();
+	AppApplySettings();
+	AppSaveSettings();
+}
+#endif
+
 void MainEmuFrame::Menu_EnableHostFs_Click( wxCommandEvent& )
 {
 	g_Conf->EmuOptions.HostFs = GetMenuBar()->IsChecked( MenuId_EnableHostFs );
@@ -487,14 +584,32 @@ void MainEmuFrame::Menu_SaveStates_Click(wxCommandEvent &event)
 	States_FreezeCurrentSlot();
 }
 
-void MainEmuFrame::Menu_LoadStateOther_Click(wxCommandEvent &event)
+void MainEmuFrame::Menu_LoadStateFromFile_Click(wxCommandEvent &event)
 {
-   Console.WriteLn("If this were hooked up, it would load a savestate file.");
+	wxFileDialog loadStateDialog(this, _("Load State"), L"", L"",
+		L"Savestate files (*.p2s)|*.p2s", wxFD_OPEN);
+
+	if (loadStateDialog.ShowModal() == wxID_CANCEL)
+	{
+		return;
+	}
+
+	wxString path = loadStateDialog.GetPath();
+	StateCopy_LoadFromFile(path);
 }
 
-void MainEmuFrame::Menu_SaveStateOther_Click(wxCommandEvent &event)
+void MainEmuFrame::Menu_SaveStateToFile_Click(wxCommandEvent &event)
 {
-   Console.WriteLn("If this were hooked up, it would save a savestate file.");
+	wxFileDialog saveStateDialog(this, _("Save State"), L"", L"",
+		L"Savestate files (*.p2s)|*.p2s", wxFD_OPEN);
+
+	if (saveStateDialog.ShowModal() == wxID_CANCEL)
+	{
+		return;
+	}
+
+	wxString path = saveStateDialog.GetPath();
+	StateCopy_SaveToFile(path);
 }
 
 void MainEmuFrame::Menu_Exit_Click(wxCommandEvent &event)
@@ -505,7 +620,7 @@ void MainEmuFrame::Menu_Exit_Click(wxCommandEvent &event)
 class SysExecEvent_ToggleSuspend : public SysExecEvent
 {
 public:
-	virtual ~SysExecEvent_ToggleSuspend() throw() {}
+	virtual ~SysExecEvent_ToggleSuspend() = default;
 
 	wxString GetEventName() const { return L"ToggleSuspendResume"; }
 
@@ -531,17 +646,12 @@ void MainEmuFrame::Menu_SuspendResume_Click(wxCommandEvent &event)
 	GetSysExecutorThread().PostEvent( new SysExecEvent_ToggleSuspend() );
 }
 
-void MainEmuFrame::Menu_SysReset_Click(wxCommandEvent &event)
-{
-	UI_DisableSysReset();
-	sApp.SysExecute();
-}
-
 void MainEmuFrame::Menu_SysShutdown_Click(wxCommandEvent &event)
 {
 	//if( !SysHasValidState() && !CorePlugins.AreAnyInitialized() ) return;
 
 	UI_DisableSysShutdown();
+	Console.SetTitle("PCSX2 Program Log");
 	CoreThread.Reset();
 }
 
@@ -582,7 +692,7 @@ void MainEmuFrame::Menu_ShowConsole(wxCommandEvent &event)
 	// Use messages to relay open/close commands (thread-safe)
 
 	g_Conf->ProgLogBox.Visible = event.IsChecked();
-	wxCommandEvent evt( wxEVT_COMMAND_MENU_SELECTED, g_Conf->ProgLogBox.Visible ? wxID_OPEN : wxID_CLOSE );
+	wxCommandEvent evt( wxEVT_MENU, g_Conf->ProgLogBox.Visible ? wxID_OPEN : wxID_CLOSE );
 	wxGetApp().ProgramLog_PostEvent( evt );
 }
 
@@ -596,3 +706,175 @@ void MainEmuFrame::Menu_ShowAboutBox(wxCommandEvent &event)
 {
 	AppOpenDialog<AboutBoxDialog>( this );
 }
+
+void MainEmuFrame::Menu_Capture_Video_Record_Click(wxCommandEvent &event)
+{
+	ScopedCoreThreadPause paused_core;
+	paused_core.AllowResume();
+
+	m_capturingVideo = true;
+	VideoCaptureUpdate();
+}
+
+void MainEmuFrame::Menu_Capture_Video_Stop_Click(wxCommandEvent &event)
+{
+	ScopedCoreThreadPause paused_core;
+	paused_core.AllowResume();
+
+	m_capturingVideo = false;
+	VideoCaptureUpdate();
+}
+
+void MainEmuFrame::VideoCaptureUpdate()
+{
+	GetMTGS().WaitGS();		// make sure GS is in sync with the audio stream when we start.
+	if (m_capturingVideo)
+	{
+		// start recording
+
+		// make the recording setup dialog[s] pseudo-modal also for the main PCSX2 window
+		// (the GSdx dialog is already properly modal for the GS window)
+		bool needsMainFrameEnable = false;
+		if (GetMainFramePtr() && GetMainFramePtr()->IsEnabled())
+		{
+			needsMainFrameEnable = true;
+			GetMainFramePtr()->Disable();
+		}
+
+		if (GSsetupRecording)
+		{
+			// GSsetupRecording can be aborted/canceled by the user. Don't go on to record the audio if that happens.
+			if (GSsetupRecording(m_capturingVideo, NULL))
+			{
+				if (SPU2setupRecording)
+				{
+					SPU2setupRecording(m_capturingVideo, NULL);
+				}
+			}
+			else
+			{
+				// recording dialog canceled by the user. align our state
+				m_capturingVideo = false;
+			}
+		}
+		else
+		{
+			// the GS doesn't support recording.
+			if (SPU2setupRecording)
+			{
+				SPU2setupRecording(m_capturingVideo, NULL);
+			}
+		}
+
+		if (GetMainFramePtr() && needsMainFrameEnable)
+		{
+			GetMainFramePtr()->Enable();
+		}
+
+	}
+	else
+	{
+		// stop recording
+		if (GSsetupRecording)
+		{
+			GSsetupRecording(m_capturingVideo, NULL);
+		}
+		if (SPU2setupRecording)
+		{
+			SPU2setupRecording(m_capturingVideo, NULL);
+		}
+	}
+
+	if (m_capturingVideo)
+	{
+		m_submenuVideoCapture.FindItem(MenuId_Capture_Video_Record)->Enable(false);
+		m_submenuVideoCapture.FindItem(MenuId_Capture_Video_Stop)->Enable(true);
+	}
+	else
+	{
+		m_submenuVideoCapture.FindItem(MenuId_Capture_Video_Record)->Enable(true);
+		m_submenuVideoCapture.FindItem(MenuId_Capture_Video_Stop)->Enable(false);
+	}
+}
+
+void MainEmuFrame::Menu_Capture_Screenshot_Screenshot_Click(wxCommandEvent & event)
+{
+	if (!CoreThread.IsOpen())
+	{
+		return;
+	}
+	GSmakeSnapshot(g_Conf->Folders.Snapshots.ToAscii());
+}
+
+#ifndef DISABLE_RECORDING
+void MainEmuFrame::Menu_Recording_New_Click(wxCommandEvent &event)
+{
+	g_InputRecording.Stop();
+
+	NewRecordingFrame* NewRecordingFrame = wxGetApp().GetNewRecordingFramePtr();
+	if (NewRecordingFrame)
+	{
+		if (NewRecordingFrame->ShowModal() == wxID_CANCEL)
+		{
+			return;
+		}
+		// From Current Frame
+		if (NewRecordingFrame->GetFrom() == 0)
+		{
+			if (!CoreThread.IsOpen())
+			{
+				recordingConLog(L"[REC]: Game is not open, aborting new input recording.\n");
+				return;
+			}
+			g_InputRecording.Create(NewRecordingFrame->GetFile(), true, NewRecordingFrame->GetAuthor());
+		}
+		// From Power-On
+		else if (NewRecordingFrame->GetFrom() == 1)
+		{
+			g_InputRecording.Create(NewRecordingFrame->GetFile(), false, NewRecordingFrame->GetAuthor());
+		}
+	}
+	m_menuRecording.FindChildItem(MenuId_Recording_New)->Enable(false);
+	m_menuRecording.FindChildItem(MenuId_Recording_Stop)->Enable(true);
+}
+
+void MainEmuFrame::Menu_Recording_Play_Click(wxCommandEvent &event)
+{
+	g_InputRecording.Stop();
+	wxFileDialog openFileDialog(this, _("Select P2M2 record file."), L"", L"",
+		L"p2m2 file(*.p2m2)|*.p2m2", wxFD_OPEN);
+	if (openFileDialog.ShowModal() == wxID_CANCEL)
+	{
+		return;
+	}
+
+	wxString path = openFileDialog.GetPath();
+	g_InputRecording.Play(path, true);
+	m_menuRecording.FindChildItem(MenuId_Recording_New)->Enable(false);
+	m_menuRecording.FindChildItem(MenuId_Recording_Stop)->Enable(true);
+}
+
+void MainEmuFrame::Menu_Recording_Stop_Click(wxCommandEvent &event)
+{
+	g_InputRecording.Stop();
+	m_menuRecording.FindChildItem(MenuId_Recording_New)->Enable(true);
+	m_menuRecording.FindChildItem(MenuId_Recording_Stop)->Enable(false);
+}
+
+void MainEmuFrame::Menu_Recording_VirtualPad_Open_Click(wxCommandEvent &event)
+{
+	VirtualPad *vp = NULL;
+	if (event.GetId() == MenuId_Recording_VirtualPad_Port0)
+	{
+		vp = wxGetApp().GetVirtualPadPtr(0);
+	}
+	else if (event.GetId() == MenuId_Recording_VirtualPad_Port1)
+	{
+		vp = wxGetApp().GetVirtualPadPtr(1);
+	}
+	if (vp != NULL)
+	{
+		vp->Show();
+	}
+}
+#endif
