@@ -45,7 +45,7 @@ static int pxMessageDialog( const wxString& caption, const wxString& content, co
 // --------------------------------------------------------------------------------------
 //  BaseMessageBoxEvent Implementation
 // --------------------------------------------------------------------------------------
-IMPLEMENT_DYNAMIC_CLASS( BaseMessageBoxEvent, pxActionEvent )
+wxIMPLEMENT_DYNAMIC_CLASS( BaseMessageBoxEvent, pxActionEvent );
 
 BaseMessageBoxEvent::BaseMessageBoxEvent( const wxString& content, SynchronousActionState& instdata )
 	: m_Content( content )
@@ -60,7 +60,7 @@ BaseMessageBoxEvent::BaseMessageBoxEvent( const wxString& content, SynchronousAc
 }
 
 BaseMessageBoxEvent::BaseMessageBoxEvent( const BaseMessageBoxEvent& event )
-	: m_Content( event.m_Content )
+	: pxActionEvent(), m_Content( event.m_Content )
 {
 	m_state = event.m_state;
 }
@@ -81,7 +81,7 @@ int BaseMessageBoxEvent::_DoDialog() const
 // --------------------------------------------------------------------------------------
 //  pxMessageBoxEvent Implementation
 // --------------------------------------------------------------------------------------
-IMPLEMENT_DYNAMIC_CLASS( pxMessageBoxEvent, BaseMessageBoxEvent )
+wxIMPLEMENT_DYNAMIC_CLASS( pxMessageBoxEvent, BaseMessageBoxEvent );
 
 pxMessageBoxEvent::pxMessageBoxEvent( const wxString& title, const wxString& content, const MsgButtons& buttons, SynchronousActionState& instdata )
 	: BaseMessageBoxEvent( content, instdata )
@@ -97,13 +97,6 @@ pxMessageBoxEvent::pxMessageBoxEvent( const wxString& title, const wxString& con
 {
 }
 
-pxMessageBoxEvent::pxMessageBoxEvent( const pxMessageBoxEvent& event )
-	: BaseMessageBoxEvent( event )
-	, m_Title( event.m_Title )
-	, m_Buttons( event.m_Buttons )
-{
-}
-
 int pxMessageBoxEvent::_DoDialog() const
 {
 	return pxMessageDialog( m_Title, m_Content, m_Buttons );
@@ -112,7 +105,7 @@ int pxMessageBoxEvent::_DoDialog() const
 // --------------------------------------------------------------------------------------
 //  pxAssertionEvent Implementation
 // --------------------------------------------------------------------------------------
-IMPLEMENT_DYNAMIC_CLASS( pxAssertionEvent, BaseMessageBoxEvent )
+wxIMPLEMENT_DYNAMIC_CLASS( pxAssertionEvent, BaseMessageBoxEvent );
 
 pxAssertionEvent::pxAssertionEvent( const wxString& content, const wxString& trace, SynchronousActionState& instdata )
 	: BaseMessageBoxEvent( content, instdata )
@@ -126,12 +119,6 @@ pxAssertionEvent::pxAssertionEvent( const wxString& content, const wxString& tra
 {
 }
 
-pxAssertionEvent::pxAssertionEvent( const pxAssertionEvent& event )
-	: BaseMessageBoxEvent( event )
-	, m_Stacktrace( event.m_Stacktrace )
-{
-}
-
 pxAssertionEvent& pxAssertionEvent::SetStacktrace( const wxString& trace )
 {
 	m_Stacktrace = trace;
