@@ -23,6 +23,7 @@
 
 #include <wx/statline.h>
 #include <wx/dnd.h>
+#include <memory>
 
 #include "AppCommon.h"
 #include "ApplyState.h"
@@ -50,7 +51,7 @@ namespace Panels
 	public:
 		DirPickerPanel( wxWindow* parent, FoldersEnum_t folderid, const wxString& label, const wxString& dialogLabel );
 		DirPickerPanel( wxWindow* parent, FoldersEnum_t folderid, const wxString& dialogLabel );
-		virtual ~DirPickerPanel() throw() { }
+		virtual ~DirPickerPanel() = default;
 
 		void Reset();
 		wxDirName GetPath() const;
@@ -88,7 +89,7 @@ namespace Panels
 		DirPickerPanel*	m_dirpicker_custom;
 
 	public:
-		virtual ~DocsFolderPickerPanel() throw() { }
+		virtual ~DocsFolderPickerPanel() = default;
 		DocsFolderPickerPanel( wxWindow* parent, bool isFirstTime = true );
 
 		void Apply();
@@ -108,7 +109,7 @@ namespace Panels
 		wxComboBox*		m_picker;
 
 	public:
-		virtual ~LanguageSelectionPanel() throw() { }
+		virtual ~LanguageSelectionPanel() = default;
 		LanguageSelectionPanel( wxWindow* parent, bool showApply = true );
 
 		void Apply();
@@ -134,7 +135,7 @@ namespace Panels
 
 	public:
 		BaseAdvancedCpuOptions( wxWindow* parent );
-		virtual ~BaseAdvancedCpuOptions() throw() { }
+		virtual ~BaseAdvancedCpuOptions() = default;
 
 		void RestoreDefaults();
 
@@ -150,7 +151,7 @@ namespace Panels
 	{
 	public:
 		AdvancedOptionsFPU( wxWindow* parent );
-		virtual ~AdvancedOptionsFPU() throw() { }
+		virtual ~AdvancedOptionsFPU() = default;
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
 		void ApplyConfigToGui( AppConfig& configToApply, int flags=0 );
@@ -160,7 +161,7 @@ namespace Panels
 	{
 	public:
 		AdvancedOptionsVU( wxWindow* parent );
-		virtual ~AdvancedOptionsVU() throw() { }
+		virtual ~AdvancedOptionsVU() = default;
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
 		void ApplyConfigToGui( AppConfig& configToApply, int flags=0 );
@@ -177,11 +178,11 @@ namespace Panels
 		pxRadioPanel*		m_panel_RecIOP;
 		pxCheckBox*			m_check_EECacheEnable;
 		AdvancedOptionsFPU*	m_advancedOptsFpu;
-		wxButton *m_button_RestoreDefaults;
+		wxButton*			m_button_RestoreDefaults;
 
 	public:
 		CpuPanelEE( wxWindow* parent );
-		virtual ~CpuPanelEE() throw() {}
+		virtual ~CpuPanelEE() = default;
 
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
@@ -189,6 +190,7 @@ namespace Panels
 
 	protected:
 		void OnRestoreDefaults( wxCommandEvent& evt );
+		void EECache_Event( wxCommandEvent& evt );
 	};
 
 	class CpuPanelVU : public BaseApplicableConfigPanel_SpecificConfig
@@ -201,7 +203,7 @@ namespace Panels
 
 	public:
 		CpuPanelVU( wxWindow* parent );
-		virtual ~CpuPanelVU() throw() {}
+		virtual ~CpuPanelVU() = default;
 
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
@@ -224,7 +226,7 @@ namespace Panels
 
 	public:
 		FrameSkipPanel( wxWindow* parent );
-		virtual	~FrameSkipPanel() throw() {}
+		virtual	~FrameSkipPanel() = default;
 
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
@@ -251,7 +253,7 @@ namespace Panels
 
 	public:
 		FramelimiterPanel( wxWindow* parent );
-		virtual ~FramelimiterPanel() throw() {}
+		virtual ~FramelimiterPanel() = default;
 
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
@@ -264,7 +266,17 @@ namespace Panels
 	class GSWindowSettingsPanel : public BaseApplicableConfigPanel_SpecificConfig
 	{
 	protected:
+		// Exclusive mode is currently not used (true for svn r4399).
+		// PCSX2 has partial infrastructure for it:
+		//  - The plugin APIs have GSsetExclusive.
+		//  - GSdx seem to support it (it supports the API and has implementation), but I don't know if it ever got called.
+		//  - BUT, the configuration (AppConfig, and specifically GSWindowOptions) do NOT seem to have a place to store this value,
+		//    and PCSX2's code doesn't seem to use this API anywhere. So, no exclusive mode for now.
+		//    - avih
+
 		wxComboBox*		m_combo_AspectRatio;
+		wxComboBox*		m_combo_FMVAspectRatioSwitch;
+		wxComboBox*		m_combo_vsync;
 
 		wxTextCtrl*		m_text_Zoom;
 
@@ -273,15 +285,6 @@ namespace Panels
 		pxCheckBox*		m_check_VsyncEnable;
 		pxCheckBox*		m_check_Fullscreen;
 
-		//Exclusive mode is currently not used (true for svn r4399).
-		//PCSX2 has partial infrastructure for it:
-		//  - The plugin APIs have GSsetExclusive.
-		//  - GSdx seem to support it (it supports the API and has implementation), but I don't know if it ever got called.
-		//  - BUT, the configuration (AppConfig, and specifically GSWindowOptions) do NOT seem to have a place to store this value,
-		//         and PCSX2's code doesn't seem to use this API anywhere. So, no exclusive mode for now.
-		//           - avih
-		//pxCheckBox*		m_check_ExclusiveFS;
-
 		pxCheckBox*		m_check_HideMouse;
 		pxCheckBox*		m_check_DclickFullscreen;
 
@@ -290,7 +293,7 @@ namespace Panels
 
 	public:
 		GSWindowSettingsPanel( wxWindow* parent );
-		virtual ~GSWindowSettingsPanel() throw() {}
+		virtual ~GSWindowSettingsPanel() = default;
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
 		void ApplyConfigToGui( AppConfig& configToApply, int flags=0 );
@@ -300,15 +303,16 @@ namespace Panels
 	{
 	protected:
 		pxCheckBox*			m_check_SynchronousGS;
-		pxCheckBox*			m_check_DisableOutput;
+		wxButton*			m_restore_defaults;
 		FrameSkipPanel*		m_span;
 		FramelimiterPanel*	m_fpan;
 
 	public:
 		VideoPanel( wxWindow* parent );
-		virtual ~VideoPanel() throw() {}
+		virtual ~VideoPanel() = default;
 		void Apply();
 		void AppStatusEvent_OnSettingsApplied();
+		void Defaults_Click(wxCommandEvent& evt);
 		void ApplyConfigToGui( AppConfig& configToApply, int flags=0 );
 
 	protected:
@@ -326,12 +330,12 @@ namespace Panels
 		pxCheckBox*		m_check_Enable;
 		wxButton*		m_button_Defaults;
 
-		wxPanelWithHelpers* m_eeSliderPanel;
-		wxPanelWithHelpers* m_vuSliderPanel;
-		wxSlider*		m_slider_eecycle;
-		wxSlider*		m_slider_vustealer;
-		pxStaticText*	m_msg_eecycle;
-		pxStaticText*	m_msg_vustealer;
+		wxPanelWithHelpers* m_eeRateSliderPanel;
+		wxPanelWithHelpers* m_eeSkipSliderPanel;
+		wxSlider*		m_slider_eeRate;
+		wxSlider*		m_slider_eeSkip;
+		pxStaticText*	m_msg_eeRate;
+		pxStaticText*	m_msg_eeSkip;
 
 		pxCheckBox*		m_check_intc;
 		pxCheckBox*		m_check_waitloop;
@@ -340,7 +344,7 @@ namespace Panels
 		pxCheckBox*		m_check_vuThread;
 
 	public:
-		virtual ~SpeedHacksPanel() throw() {}
+		virtual ~SpeedHacksPanel() = default;
 		SpeedHacksPanel( wxWindow* parent );
 		void Apply();
 		void EnableStuff( AppConfig* configToUse=NULL );
@@ -348,10 +352,11 @@ namespace Panels
 		void ApplyConfigToGui( AppConfig& configToApply, int flags=0 );
 
 	protected:
-		const wxChar* GetEEcycleSliderMsg( int val );
-		const wxChar* GetVUcycleSliderMsg( int val );
+		const wxChar* GetEECycleRateSliderMsg( int val );
+		const wxChar* GetEECycleSkipSliderMsg( int val );
 		void SetEEcycleSliderMsg();
 		void SetVUcycleSliderMsg();
+		void TrigLayout();
 
 		void OnEnable_Toggled( wxCommandEvent& evt );
 		void Defaults_Click( wxCommandEvent& evt );
@@ -370,7 +375,7 @@ namespace Panels
 
 	public:
 		GameFixesPanel( wxWindow* parent );
-		virtual ~GameFixesPanel() throw() { }
+		virtual ~GameFixesPanel() = default;
 		void EnableStuff( AppConfig* configToUse=NULL );
 		void OnEnable_Toggled( wxCommandEvent& evt );
 		void Apply();
@@ -378,36 +383,6 @@ namespace Panels
 		void ApplyConfigToGui( AppConfig& configToApply, int flags=0 );
 	};
 
-	// --------------------------------------------------------------------------------------
-	//  GameDatabasePanel
-	// --------------------------------------------------------------------------------------
-	class GameDatabasePanel : public BaseApplicableConfigPanel
-	{
-	protected:
-		//wxTextCtrl*	searchBox;
-		//wxComboBox*	searchType;
-		//wxListBox*	searchList;
-		wxButton*	searchBtn;
-		wxTextCtrl*	serialBox;
-		wxTextCtrl*	nameBox;
-		wxTextCtrl*	regionBox;
-		wxTextCtrl*	compatBox;
-		wxTextCtrl*	commentBox;
-		wxTextCtrl*	patchesBox;
-		pxCheckBox*	gameFixes[GamefixId_COUNT];
-
-	public:
-		GameDatabasePanel( wxWindow* parent );
-		virtual ~GameDatabasePanel() throw() { }
-		void Apply();
-		void AppStatusEvent_OnSettingsApplied();
-
-	protected:
-		void PopulateFields( const wxString& serial=wxEmptyString );
-		bool WriteFieldsToDB();
-		void Search_Click( wxCommandEvent& evt );
-	};
-
 	class SettingsDirPickerPanel : public DirPickerPanel
 	{
 	public:
@@ -431,21 +406,6 @@ namespace Panels
 		StandardPathsPanel( wxWindow* parent );
 	};
 
-	// --------------------------------------------------------------------------------------
-	//  AppearanceThemesPanel
-	// --------------------------------------------------------------------------------------
-	class AppearanceThemesPanel : public BaseApplicableConfigPanel
-	{
-		typedef BaseApplicableConfigPanel _parent;
-
-	public:
-		virtual ~AppearanceThemesPanel() throw();
-		AppearanceThemesPanel( wxWindow* parent );
-
-		void Apply();
-		void AppStatusEvent_OnSettingsApplied();
-	};
-
 	// --------------------------------------------------------------------------------------
 	//  BaseSelectorPanel
 	// --------------------------------------------------------------------------------------
@@ -454,7 +414,7 @@ namespace Panels
 		typedef BaseApplicableConfigPanel _parent;
 
 	public:
-		virtual ~BaseSelectorPanel() throw();
+		virtual ~BaseSelectorPanel() = default;
 		BaseSelectorPanel( wxWindow* parent );
 
 		virtual void RefreshSelections();
@@ -484,42 +444,19 @@ namespace Panels
 		void OnShow(wxShowEvent& evt);
 	};
 
-	// --------------------------------------------------------------------------------------
-	//  ThemeSelectorPanel
-	// --------------------------------------------------------------------------------------
-	class ThemeSelectorPanel : public BaseSelectorPanel
-	{
-		typedef BaseSelectorPanel _parent;
-
-	protected:
-		ScopedPtr<wxArrayString>	m_ThemeList;
-		wxListBox*					m_ComboBox;
-		DirPickerPanel*				m_FolderPicker;
-
-	public:
-		virtual ~ThemeSelectorPanel() throw();
-		ThemeSelectorPanel( wxWindow* parent );
-
-	protected:
-		virtual void Apply();
-		virtual void AppStatusEvent_OnSettingsApplied();
-		virtual void DoRefresh();
-		virtual bool ValidateEnumerationStatus();	
-	};
-
 	// --------------------------------------------------------------------------------------
 	//  BiosSelectorPanel
 	// --------------------------------------------------------------------------------------
 	class BiosSelectorPanel : public BaseSelectorPanel
 	{
 	protected:
-		ScopedPtr<wxArrayString>	m_BiosList;
-		wxListBox*					m_ComboBox;
-		DirPickerPanel*				m_FolderPicker;
+		std::unique_ptr<wxArrayString> m_BiosList;
+		wxListBox* m_ComboBox;
+		DirPickerPanel* m_FolderPicker;
 
 	public:
 		BiosSelectorPanel( wxWindow* parent );
-		virtual ~BiosSelectorPanel() throw();
+		virtual ~BiosSelectorPanel() = default;
 
 	protected:
 		virtual void Apply();
@@ -555,13 +492,13 @@ namespace Panels
 		class EnumThread : public Threading::pxThread
 		{
 		public:
-			SafeList<EnumeratedPluginInfo> Results;		// array of plugin results.
+			std::vector<EnumeratedPluginInfo> Results;		// array of plugin results.
 
 		protected:
 			PluginSelectorPanel&	m_master;
 			ScopedBusyCursor		m_hourglass;
 		public:
-			virtual ~EnumThread() throw()
+			virtual ~EnumThread()
 			{
 				try {
 					pxThread::Cancel();
@@ -620,11 +557,11 @@ namespace Panels
 		ComboBoxPanel*	m_ComponentBoxes;
 		bool			m_Canceled;
 
-		ScopedPtr<wxArrayString>	m_FileList;	// list of potential plugin files
-		ScopedPtr<EnumThread>		m_EnumeratorThread;
+		std::unique_ptr<wxArrayString>	m_FileList;	// list of potential plugin files
+		std::unique_ptr<EnumThread>		m_EnumeratorThread;
 
 	public:
-		virtual ~PluginSelectorPanel() throw();
+		virtual ~PluginSelectorPanel();
 		PluginSelectorPanel( wxWindow* parent );
 
 		void CancelRefresh();		// used from destructor, stays non-virtual
