@@ -24,7 +24,12 @@
 #include "ZipTools/ThreadedZipTools.h"
 #include "Utilities/pxStreams.h"
 
+#include "ConsoleLogger.h"
+
 #include <wx/wfstream.h>
+#include <memory>
+
+#include "Patch.h"
 
 // Used to hold the current state backup (fullcopy of PS2 memory and plugin states).
 //static VmStateBuffer state_buffer( L"Public Savestate Buffer" );
@@ -40,10 +45,11 @@ static const wxChar* EntryFilename_InternalStructures	= L"PCSX2 Internal Structu
 class BaseSavestateEntry
 {
 protected:
-	BaseSavestateEntry() {}
-	virtual ~BaseSavestateEntry() throw() {}
+	BaseSavestateEntry() = default;
 
 public:
+	virtual ~BaseSavestateEntry() = default;
+
 	virtual wxString GetFilename() const=0;
 	virtual void FreezeIn( pxInputStream& reader ) const=0;
 	virtual void FreezeOut( SaveStateBase& writer ) const=0;
@@ -54,7 +60,7 @@ class MemorySavestateEntry : public BaseSavestateEntry
 {
 protected:
 	MemorySavestateEntry() {}
-	virtual ~MemorySavestateEntry() throw() {}
+	virtual ~MemorySavestateEntry() = default;
 
 public:
 	virtual void FreezeIn( pxInputStream& reader ) const;
@@ -77,7 +83,7 @@ public:
 		m_pid = pid;
 	}
 
-	virtual ~PluginSavestateEntry() throw() {}
+	virtual ~PluginSavestateEntry() = default;
 
 	virtual wxString GetFilename() const;
 	virtual void FreezeIn( pxInputStream& reader ) const;
@@ -141,10 +147,12 @@ void PluginSavestateEntry::FreezeOut( SaveStateBase& writer ) const
 class SavestateEntry_EmotionMemory : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_EmotionMemory() = default;
+
 	wxString GetFilename() const		{ return L"eeMemory.bin"; }
 	u8* GetDataPtr() const				{ return eeMem->Main; }
 	uint GetDataSize() const			{ return sizeof(eeMem->Main); }
-	
+
 	virtual void FreezeIn( pxInputStream& reader ) const
 	{
 		SysClearExecutionCache();
@@ -155,6 +163,8 @@ public:
 class SavestateEntry_IopMemory : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_IopMemory() = default;
+
 	wxString GetFilename() const		{ return L"iopMemory.bin"; }
 	u8* GetDataPtr() const				{ return iopMem->Main; }
 	uint GetDataSize() const			{ return sizeof(iopMem->Main); }
@@ -163,6 +173,8 @@ public:
 class SavestateEntry_HwRegs : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_HwRegs() = default;
+
 	wxString GetFilename() const		{ return L"eeHwRegs.bin"; }
 	u8* GetDataPtr() const				{ return eeHw; }
 	uint GetDataSize() const			{ return sizeof(eeHw); }
@@ -171,6 +183,8 @@ public:
 class SavestateEntry_IopHwRegs : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_IopHwRegs() = default;
+
 	wxString GetFilename() const		{ return L"iopHwRegs.bin"; }
 	u8* GetDataPtr() const				{ return iopHw; }
 	uint GetDataSize() const			{ return sizeof(iopHw); }
@@ -179,6 +193,8 @@ public:
 class SavestateEntry_Scratchpad : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_Scratchpad() = default;
+
 	wxString GetFilename() const		{ return L"Scratchpad.bin"; }
 	u8* GetDataPtr() const				{ return eeMem->Scratch; }
 	uint GetDataSize() const			{ return sizeof(eeMem->Scratch); }
@@ -187,6 +203,8 @@ public:
 class SavestateEntry_VU0mem : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_VU0mem() = default;
+
 	wxString GetFilename() const		{ return L"vu0Memory.bin"; }
 	u8* GetDataPtr() const				{ return vuRegs[0].Mem; }
 	uint GetDataSize() const			{ return VU0_MEMSIZE; }
@@ -195,6 +213,8 @@ public:
 class SavestateEntry_VU1mem : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_VU1mem() = default;
+
 	wxString GetFilename() const		{ return L"vu1Memory.bin"; }
 	u8* GetDataPtr() const				{ return vuRegs[1].Mem; }
 	uint GetDataSize() const			{ return VU1_MEMSIZE; }
@@ -203,6 +223,8 @@ public:
 class SavestateEntry_VU0prog : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_VU0prog() = default;
+
 	wxString GetFilename() const		{ return L"vu0MicroMem.bin"; }
 	u8* GetDataPtr() const				{ return vuRegs[0].Micro; }
 	uint GetDataSize() const			{ return VU0_PROGSIZE; }
@@ -211,6 +233,8 @@ public:
 class SavestateEntry_VU1prog : public MemorySavestateEntry
 {
 public:
+	virtual ~SavestateEntry_VU1prog() = default;
+
 	wxString GetFilename() const		{ return L"vu1MicroMem.bin"; }
 	u8* GetDataPtr() const				{ return vuRegs[1].Micro; }
 	uint GetDataSize() const			{ return VU1_PROGSIZE; }
@@ -226,42 +250,26 @@ public:
 //  would not be useful).
 //
 
-static const uint NumSavestateEntries = 9 + PluginId_Count;
+static const std::unique_ptr<BaseSavestateEntry> SavestateEntries[] = {
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_EmotionMemory),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_IopMemory),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_HwRegs),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_IopHwRegs),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_Scratchpad),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_VU0mem),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_VU1mem),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_VU0prog),
+	std::unique_ptr<BaseSavestateEntry>(new SavestateEntry_VU1prog),
 
-class SavestateEntryPack : public ScopedAlloc<const BaseSavestateEntry*>
-{
-	typedef ScopedAlloc<const BaseSavestateEntry*> _parent;
-
-public:
-	SavestateEntryPack()
-		: _parent( NumSavestateEntries )
-	{
-		uint i = 0;		// more convenient in case we re-arrange anything...
-
-		this->operator[](i++)	= new SavestateEntry_EmotionMemory;
-		this->operator[](i++)	= new SavestateEntry_IopMemory;
-		this->operator[](i++)	= new SavestateEntry_HwRegs;
-		this->operator[](i++)	= new SavestateEntry_IopHwRegs;
-		this->operator[](i++)	= new SavestateEntry_Scratchpad;
-		this->operator[](i++)	= new SavestateEntry_VU0mem;
-		this->operator[](i++)	= new SavestateEntry_VU1mem;
-		this->operator[](i++)	= new SavestateEntry_VU0prog;
-		this->operator[](i++)	= new SavestateEntry_VU1prog;
-		
-		this->operator[](i++)	= new PluginSavestateEntry( PluginId_GS );
-		this->operator[](i++)	= new PluginSavestateEntry( PluginId_PAD );
-		this->operator[](i++)	= new PluginSavestateEntry( PluginId_SPU2 );
-		this->operator[](i++)	= new PluginSavestateEntry( PluginId_CDVD );
-		this->operator[](i++)	= new PluginSavestateEntry( PluginId_USB );
-		this->operator[](i++)	= new PluginSavestateEntry( PluginId_FW );
-		this->operator[](i++)	= new PluginSavestateEntry( PluginId_DEV9 );
-	}
-	
-	using _parent::operator[];
+	std::unique_ptr<BaseSavestateEntry>(new PluginSavestateEntry( PluginId_GS )),
+	std::unique_ptr<BaseSavestateEntry>(new PluginSavestateEntry( PluginId_PAD )),
+	std::unique_ptr<BaseSavestateEntry>(new PluginSavestateEntry( PluginId_SPU2 )),
+	std::unique_ptr<BaseSavestateEntry>(new PluginSavestateEntry( PluginId_CDVD )),
+	std::unique_ptr<BaseSavestateEntry>(new PluginSavestateEntry( PluginId_USB )),
+	std::unique_ptr<BaseSavestateEntry>(new PluginSavestateEntry( PluginId_FW )),
+	std::unique_ptr<BaseSavestateEntry>(new PluginSavestateEntry( PluginId_DEV9 ))
 };
 
-static const SavestateEntryPack SavestateEntries;
-
 // It's bad mojo to have savestates trying to read and write from the same file at the
 // same time.  To prevent that we use this mutex lock, which is used by both the
 // CompressThread and the UnzipFromDisk events.  (note that CompressThread locks the
@@ -274,13 +282,13 @@ static void CheckVersion( pxInputStream& thr )
 {
 	u32 savever;
 	thr.Read( savever );
-	
+
 	// Major version mismatch.  Means we can't load this savestate at all.  Support for it
 	// was removed entirely.
 	if( savever > g_SaveVersion )
 		throw Exception::SaveStateLoadError( thr.GetStreamName() )
 			.SetDiagMsg(pxsFmt( L"Savestate uses an unsupported or unknown savestate version.\n(PCSX2 ver=%x, state ver=%x)", g_SaveVersion, savever ))
-			.SetUserMsg(_("Cannot load this savestate.  The state is an unsupported version."));
+			.SetUserMsg(_("Cannot load this savestate. The state is an unsupported version."));
 
 	// check for a "minor" version incompatibility; which happens if the savestate being loaded is a newer version
 	// than the emulator recognizes.  99% chance that trying to load it will just corrupt emulation or crash.
@@ -305,7 +313,7 @@ protected:
 public:
 	wxString GetEventName() const { return L"VM_Download"; }
 
-	virtual ~SysExecEvent_DownloadState() throw() {}
+	virtual ~SysExecEvent_DownloadState() = default;
 	SysExecEvent_DownloadState* Clone() const { return new SysExecEvent_DownloadState( *this ); }
 	SysExecEvent_DownloadState( ArchiveEntryList* dest_list=NULL )
 	{
@@ -314,7 +322,7 @@ public:
 
 	bool IsCriticalEvent() const { return true; }
 	bool AllowCancelOnExit() const { return false; }
-	
+
 protected:
 	void InvokeEvent()
 	{
@@ -335,7 +343,7 @@ protected:
 		internals.SetDataSize( saveme.GetCurrentPos() - internals.GetDataIndex() );
 		m_dest_list->Add( internals );
 
-		for (uint i=0; i<SavestateEntries.GetSize(); ++i)
+		for (uint i=0; i<ArraySize(SavestateEntries); ++i)
 		{
 			uint startpos = saveme.GetCurrentPos();
 			SavestateEntries[i]->FreezeOut( saveme );
@@ -367,10 +375,8 @@ public:
 		m_lock_Compress.Assign(mtx_CompressToDisk);
 	}
 
-	virtual ~VmStateCompressThread() throw()
-	{
-	}
-	
+	virtual ~VmStateCompressThread() = default;
+
 protected:
 	void OnStartInThread()
 	{
@@ -397,9 +403,7 @@ protected:
 public:
 	wxString GetEventName() const { return L"VM_ZipToDisk"; }
 
-	virtual ~SysExecEvent_ZipToDisk() throw()
-	{
-	}
+	virtual ~SysExecEvent_ZipToDisk() = default;
 
 	SysExecEvent_ZipToDisk* Clone() const { return new SysExecEvent_ZipToDisk( *this ); }
 
@@ -422,7 +426,7 @@ protected:
 	void InvokeEvent()
 	{
 		// Provisionals for scoped cleanup, in case of exception:
-		ScopedPtr<ArchiveEntryList> elist( m_src_list );
+		std::unique_ptr<ArchiveEntryList> elist(m_src_list);
 
 		wxString tempfile( m_filename + L".tmp" );
 
@@ -437,7 +441,7 @@ protected:
 		pxYield(4);
 
 		// Write the version and screenshot:
-		ScopedPtr<pxOutputStream> out( new pxOutputStream(tempfile, new wxZipOutputStream(woot)) );
+		std::unique_ptr<pxOutputStream> out(new pxOutputStream(tempfile, new wxZipOutputStream(woot)));
 		wxZipOutputStream* gzfp = (wxZipOutputStream*)out->GetWxStreamBase();
 
 		{
@@ -448,8 +452,8 @@ protected:
 			gzfp->CloseEntry();
 		}
 
-		ScopedPtr<wxImage> m_screenshot;
-		
+		std::unique_ptr<wxImage> m_screenshot;
+
 		if (m_screenshot)
 		{
 			wxZipEntry* vent = new wxZipEntry(EntryFilename_Screenshot);
@@ -460,18 +464,23 @@ protected:
 		}
 
 		(*new VmStateCompressThread())
-			.SetSource(elist)
-			.SetOutStream(out)
+			.SetSource(elist.get())
+			.SetOutStream(out.get())
 			.SetFinishedPath(m_filename)
 			.Start();
 
-		// No errors?  Release cleanup handlers:			
-		elist.DetachPtr();
-		out.DetachPtr();
+		// No errors?  Release cleanup handlers:
+		elist.release();
+		out.release();
 	}
-	
+
 	void CleanupEvent()
 	{
+#ifdef USE_NEW_SAVESLOTS_UI
+		// I have a feeling this doesn't need to be here, so I'm commenting this out for the moment.
+		// I'll remove it if it doesn't cause other issues.
+		//UI_UpdateSysControls();
+#endif
 	}
 };
 
@@ -486,11 +495,11 @@ class SysExecEvent_UnzipFromDisk : public SysExecEvent
 {
 protected:
 	wxString	m_filename;
-	
+
 public:
 	wxString GetEventName() const { return L"VM_UnzipFromDisk"; }
-	
-	virtual ~SysExecEvent_UnzipFromDisk() throw() {}
+
+	virtual ~SysExecEvent_UnzipFromDisk() = default;
 	SysExecEvent_UnzipFromDisk* Clone() const { return new SysExecEvent_UnzipFromDisk( *this ); }
 	SysExecEvent_UnzipFromDisk( const wxString& filename )
 		: m_filename( filename )
@@ -506,12 +515,12 @@ protected:
 
 		// Ugh.  Exception handling made crappy because wxWidgets classes don't support scoped pointers yet.
 
-		ScopedPtr<wxFFileInputStream> woot( new wxFFileInputStream(m_filename) );
+		std::unique_ptr<wxFFileInputStream> woot(new wxFFileInputStream(m_filename));
 		if (!woot->IsOk())
 			throw Exception::CannotCreateStream( m_filename ).SetDiagMsg(L"Cannot open file for reading.");
 
-		ScopedPtr<pxInputStream> reader( new pxInputStream(m_filename, new wxZipInputStream(woot)) );
-		woot.DetachPtr();
+		std::unique_ptr<pxInputStream> reader(new pxInputStream(m_filename, new wxZipInputStream(woot.get())));
+		woot.release();
 
 		if (!reader->IsOk())
 		{
@@ -526,16 +535,16 @@ protected:
 
 		bool foundVersion = false;
 		//bool foundScreenshot = false;
-		//bool foundEntry[numSavestateEntries] = false;
+		//bool foundEntry[ArraySize(SavestateEntries)] = false;
 
-		ScopedPtr<wxZipEntry> foundInternal;
-		ScopedPtr<wxZipEntry> foundEntry[NumSavestateEntries];
+		std::unique_ptr<wxZipEntry> foundInternal;
+		std::unique_ptr<wxZipEntry> foundEntry[ArraySize(SavestateEntries)];
 
 		while(true)
 		{
 			Threading::pxTestCancel();
 
-			ScopedPtr<wxZipEntry> entry( gzreader->GetNextEntry() );
+			std::unique_ptr<wxZipEntry> entry(gzreader->GetNextEntry());
 			if (!entry) break;
 
 			if (entry->GetName().CmpNoCase(EntryFilename_StateVersion) == 0)
@@ -549,7 +558,7 @@ protected:
 			if (entry->GetName().CmpNoCase(EntryFilename_InternalStructures) == 0)
 			{
 				DevCon.WriteLn( Color_Green, L" ... found '%s'", EntryFilename_InternalStructures);
-				foundInternal = entry.DetachPtr();
+				foundInternal = std::move(entry);
 				continue;
 			}
 
@@ -560,12 +569,12 @@ protected:
 				foundScreenshot = true;
 			}*/
 
-			for (uint i=0; i<NumSavestateEntries; ++i)
+			for (uint i=0; i<ArraySize(SavestateEntries); ++i)
 			{
 				if (entry->GetName().CmpNoCase(SavestateEntries[i]->GetFilename()) == 0)
 				{
 					DevCon.WriteLn( Color_Green, L" ... found '%s'", WX_STR(SavestateEntries[i]->GetFilename()) );
-					foundEntry[i] = entry.DetachPtr();
+					foundEntry[i] = std::move(entry);
 					break;
 				}
 			}
@@ -581,10 +590,10 @@ protected:
 
 		// Log any parts and pieces that are missing, and then generate an exception.
 		bool throwIt = false;
-		for (uint i=0; i<NumSavestateEntries; ++i)
+		for (uint i=0; i<ArraySize(SavestateEntries); ++i)
 		{
 			if (foundEntry[i]) continue;
-			
+
 			if (SavestateEntries[i]->IsRequired())
 			{
 				throwIt = true;
@@ -600,10 +609,12 @@ protected:
 		// We use direct Suspend/Resume control here, since it's desirable that emulation
 		// *ALWAYS* start execution after the new savestate is loaded.
 
+		PatchesVerboseReset();
+
 		GetCoreThread().Pause();
 		SysClearExecutionCache();
 
-		for (uint i=0; i<NumSavestateEntries; ++i)
+		for (uint i=0; i<ArraySize(SavestateEntries); ++i)
 		{
 			if (!foundEntry[i]) continue;
 
@@ -633,12 +644,12 @@ void StateCopy_SaveToFile( const wxString& file )
 {
 	UI_DisableStateActions();
 
-	ScopedPtr<ArchiveEntryList>	ziplist	(new ArchiveEntryList( new VmStateBuffer( L"Zippable Savestate" ) ));
-	
-	GetSysExecutorThread().PostEvent(new SysExecEvent_DownloadState	( ziplist ));
-	GetSysExecutorThread().PostEvent(new SysExecEvent_ZipToDisk		( ziplist, file ));
-	
-	ziplist.DetachPtr();
+	std::unique_ptr<ArchiveEntryList> ziplist(new ArchiveEntryList(new VmStateBuffer(L"Zippable Savestate")));
+
+	GetSysExecutorThread().PostEvent(new SysExecEvent_DownloadState(ziplist.get()));
+	GetSysExecutorThread().PostEvent(new SysExecEvent_ZipToDisk(ziplist.get(), file));
+
+	ziplist.release();
 }
 
 void StateCopy_LoadFromFile( const wxString& file )
@@ -659,15 +670,18 @@ void StateCopy_SaveToSlot( uint num )
 	if( wxFileExists( file ) && EmuConfig.BackupSavestate )
 	{
 		const wxString copy( SaveStateBase::GetFilename( num ) + pxsFmt( L".backup") );
-		
+
 		Console.Indent().WriteLn( Color_StrongGreen, L"Backing up existing state in slot %d.", num);
 		wxRenameFile( file, copy );
 	}
 
-	Console.WriteLn( Color_StrongGreen, "Saving savestate to slot %d...", num );
+	OSDlog( Color_StrongGreen, true, "Saving savestate to slot %d...", num );
 	Console.Indent().WriteLn( Color_StrongGreen, L"filename: %s", WX_STR(file) );
 
 	StateCopy_SaveToFile( file );
+#ifdef USE_NEW_SAVESLOTS_UI
+	UI_UpdateSysControls();
+#endif
 }
 
 void StateCopy_LoadFromSlot( uint slot, bool isFromBackup )
@@ -676,12 +690,15 @@ void StateCopy_LoadFromSlot( uint slot, bool isFromBackup )
 
 	if( !wxFileExists( file ) )
 	{
-		Console.Warning( L"Savestate slot %d%s is empty.", slot, isFromBackup?L" (backup)":L"" );
+		OSDlog(Color_StrongGreen, true, "Savestate slot %d%s is empty.", slot, isFromBackup ? " (backup)" : "");
 		return;
 	}
 
-	Console.WriteLn( Color_StrongGreen, L"Loading savestate from slot %d...%s", slot, WX_STR(wxString( isFromBackup?L" (backup)":L"" )) );
+	OSDlog( Color_StrongGreen, true, "Loading savestate from slot %d...%s", slot, isFromBackup?" (backup)":"" );
 	Console.Indent().WriteLn( Color_StrongGreen, L"filename: %s", WX_STR(file) );
 
 	StateCopy_LoadFromFile( file );
+#ifdef USE_NEW_SAVESLOTS_UI
+	UI_UpdateSysControls();
+#endif
 }
