@@ -51,7 +51,6 @@ void _deleteEEreg(int reg, int flush)
 	}
 	GPR_DEL_CONST(reg);
 	_deleteGPRtoXMMreg(reg, flush ? 0 : 2);
-	_deleteMMXreg(MMX_GPR+reg, flush ? 0 : 2);
 }
 
 void _flushEEreg(int reg)
@@ -62,24 +61,23 @@ void _flushEEreg(int reg)
 		return;
 	}
 	_deleteGPRtoXMMreg(reg, 1);
-	_deleteMMXreg(MMX_GPR + reg, 1);
 }
 
 // if not mmx, then xmm
 int eeProcessHILO(int reg, int mode, int mmx)
 {
 	// Fixme: MMX problem
-    int usemmx = 0/*mmx && _hasFreeMMXreg()*/;
+    int usemmx = 0;
 	if( (usemmx || _hasFreeXMMreg()) || !(g_pCurInstInfo->regs[reg]&EEINST_LASTUSE) ) {
-		//if( usemmx ) return _allocMMXreg(-1, MMX_GPR+reg, mode);
 		return _allocGPRtoXMMreg(-1, reg, mode);
 	}
 
 	return -1;
 }
 
-#define PROCESS_EE_SETMODES(mmreg) ((mmxregs[mmreg].mode&MODE_WRITE)?PROCESS_EE_MODEWRITES:0)
-#define PROCESS_EE_SETMODET(mmreg) ((mmxregs[mmreg].mode&MODE_WRITE)?PROCESS_EE_MODEWRITET:0)
+// Strangely this code is used on NOT-MMX path ...
+#define PROCESS_EE_SETMODES(mmreg) (/*(mmxregs[mmreg].mode&MODE_WRITE)*/ false ?PROCESS_EE_MODEWRITES:0)
+#define PROCESS_EE_SETMODET(mmreg) (/*(mmxregs[mmreg].mode&MODE_WRITE)*/ false ?PROCESS_EE_MODEWRITET:0)
 
 // ignores XMMINFO_READS, XMMINFO_READT, and XMMINFO_READD_LO from xmminfo
 // core of reg caching
@@ -91,7 +89,6 @@ void eeRecompileCode0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R5900FNP
 
 	if( GPR_IS_CONST2(_Rs_, _Rt_) ) {
 		if( xmminfo & XMMINFO_WRITED ) {
-			_deleteMMXreg(MMX_GPR+_Rd_, 2);
 			_deleteGPRtoXMMreg(_Rd_, 2);
 		}
 		if( xmminfo&XMMINFO_WRITED ) GPR_SET_CONST(_Rd_);
@@ -103,6 +100,7 @@ void eeRecompileCode0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R5900FNP
 
 	// test if should write xmm, mirror to mmx code
 	if( g_pCurInstInfo->info & EEINST_XMM ) {
+		pxAssert(0);
 
 		if( xmminfo & (XMMINFO_READLO|XMMINFO_WRITELO) ) _addNeededGPRtoXMMreg(XMMGPR_LO);
 		if( xmminfo & (XMMINFO_READHI|XMMINFO_WRITEHI) ) _addNeededGPRtoXMMreg(XMMGPR_HI);
@@ -134,7 +132,6 @@ void eeRecompileCode0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R5900FNP
 						_freeXMMreg(mmreg1);
 						if( GPR_IS_CONST1(_Rs_) ) info &= ~PROCESS_EE_MODEWRITET;
 						else info &= ~PROCESS_EE_MODEWRITES;
-						_deleteMMXreg(MMX_GPR+_Rd_, 2);
 						xmmregs[mmreg1].inuse = 1;
 						xmmregs[mmreg1].reg = _Rd_;
 						xmmregs[mmreg1].mode = moded;
@@ -184,7 +181,6 @@ void eeRecompileCode0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R5900FNP
 						if( !(xmminfo&XMMINFO_READD) && ((g_pCurInstInfo->regs[_Rt_] & EEINST_LASTUSE) || !EEINST_ISLIVEXMM(_Rt_)) ) {
 							_freeXMMreg(mmreg2);
 							info &= ~PROCESS_EE_MODEWRITET;
-							_deleteMMXreg(MMX_GPR+_Rd_, 2);
 							xmmregs[mmreg2].inuse = 1;
 							xmmregs[mmreg2].reg = _Rd_;
 							xmmregs[mmreg2].mode = moded;
@@ -193,7 +189,6 @@ void eeRecompileCode0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R5900FNP
 						else if( !(xmminfo&XMMINFO_READD) && ((g_pCurInstInfo->regs[_Rs_] & EEINST_LASTUSE) || !EEINST_ISLIVEXMM(_Rs_)) ) {
 							_freeXMMreg(mmreg1);
 							info &= ~PROCESS_EE_MODEWRITES;
-							_deleteMMXreg(MMX_GPR+_Rd_, 2);
 							xmmregs[mmreg1].inuse = 1;
 							xmmregs[mmreg1].reg = _Rd_;
 							xmmregs[mmreg1].mode = moded;
@@ -229,19 +224,13 @@ void eeRecompileCode0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R5900FNP
 	_deleteGPRtoXMMreg(_Rt_, 1);
 	if( xmminfo&XMMINFO_WRITED )
 		_deleteGPRtoXMMreg(_Rd_, (xmminfo&XMMINFO_READD)?0:2);
-	_deleteMMXreg(MMX_GPR+_Rs_, 1);
-	_deleteMMXreg(MMX_GPR+_Rt_, 1);
-	if( xmminfo&XMMINFO_WRITED )
-		_deleteMMXreg(MMX_GPR+_Rd_, (xmminfo&XMMINFO_READD)?0:2);
 
 	// don't delete, fn will take care of them
 //	if( xmminfo & (XMMINFO_READLO|XMMINFO_WRITELO) ) {
 //		_deleteGPRtoXMMreg(XMMGPR_LO, (xmminfo&XMMINFO_READLO)?1:0);
-//		_deleteMMXreg(MMX_GPR+MMX_LO, (xmminfo&XMMINFO_READLO)?1:0);
 //	}
 //	if( xmminfo & (XMMINFO_READHI|XMMINFO_WRITEHI) ) {
 //		_deleteGPRtoXMMreg(XMMGPR_HI, (xmminfo&XMMINFO_READHI)?1:0);
-//		_deleteMMXreg(MMX_GPR+MMX_HI, (xmminfo&XMMINFO_READHI)?1:0);
 //	}
 
 	if( GPR_IS_CONST1(_Rs_) ) {
@@ -267,7 +256,6 @@ void eeRecompileCode1(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 	if ( ! _Rt_ ) return;
 
 	if( GPR_IS_CONST1(_Rs_) ) {
-		_deleteMMXreg(MMX_GPR+_Rt_, 2);
 		_deleteGPRtoXMMreg(_Rt_, 2);
 		GPR_SET_CONST(_Rt_);
 		constcode();
@@ -276,6 +264,7 @@ void eeRecompileCode1(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 
 	// test if should write xmm, mirror to mmx code
 	if( g_pCurInstInfo->info & EEINST_XMM ) {
+		pxAssert(0);
 
 		// no const regs
 		mmreg1 = _allocCheckGPRtoXMM(g_pCurInstInfo, _Rs_, MODE_READ);
@@ -291,7 +280,6 @@ void eeRecompileCode1(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 				if( (g_pCurInstInfo->regs[_Rs_] & EEINST_LASTUSE) || !EEINST_ISLIVEXMM(_Rs_) ) {
 					_freeXMMreg(mmreg1);
 					info &= ~PROCESS_EE_MODEWRITES;
-					_deleteMMXreg(MMX_GPR+_Rt_, 2);
 					xmmregs[mmreg1].inuse = 1;
 					xmmregs[mmreg1].reg = _Rt_;
 					xmmregs[mmreg1].mode = MODE_WRITE|MODE_READ;
@@ -312,8 +300,6 @@ void eeRecompileCode1(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 	// regular x86
 	_deleteGPRtoXMMreg(_Rs_, 1);
 	_deleteGPRtoXMMreg(_Rt_, 2);
-	_deleteMMXreg(MMX_GPR+_Rs_, 1);
-	_deleteMMXreg(MMX_GPR+_Rt_, 2);
 
 	noconstcode(0);
 	GPR_DEL_CONST(_Rt_);
@@ -326,7 +312,6 @@ void eeRecompileCode2(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 	if ( ! _Rd_ ) return;
 
 	if( GPR_IS_CONST1(_Rt_) ) {
-		_deleteMMXreg(MMX_GPR+_Rd_, 2);
 		_deleteGPRtoXMMreg(_Rd_, 2);
 		GPR_SET_CONST(_Rd_);
 		constcode();
@@ -335,6 +320,7 @@ void eeRecompileCode2(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 
 	// test if should write xmm, mirror to mmx code
 	if( g_pCurInstInfo->info & EEINST_XMM ) {
+		pxAssert(0);
 
 		// no const regs
 		mmreg1 = _allocCheckGPRtoXMM(g_pCurInstInfo, _Rt_, MODE_READ);
@@ -350,7 +336,6 @@ void eeRecompileCode2(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 				if( (g_pCurInstInfo->regs[_Rt_] & EEINST_LASTUSE) || !EEINST_ISLIVE64(_Rt_) ) {
 					_freeXMMreg(mmreg1);
 					info &= ~PROCESS_EE_MODEWRITET;
-					_deleteMMXreg(MMX_GPR+_Rd_, 2);
 					xmmregs[mmreg1].inuse = 1;
 					xmmregs[mmreg1].reg = _Rd_;
 					xmmregs[mmreg1].mode = MODE_WRITE|MODE_READ;
@@ -371,8 +356,6 @@ void eeRecompileCode2(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 	// regular x86
 	_deleteGPRtoXMMreg(_Rt_, 1);
 	_deleteGPRtoXMMreg(_Rd_, 2);
-	_deleteMMXreg(MMX_GPR+_Rt_, 1);
-	_deleteMMXreg(MMX_GPR+_Rd_, 2);
 
 	noconstcode(0);
 	GPR_DEL_CONST(_Rd_);
@@ -417,9 +400,6 @@ void eeRecompileCodeConst0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R59
 	_deleteGPRtoXMMreg(_Rs_, 1);
 	_deleteGPRtoXMMreg(_Rt_, 1);
 	_deleteGPRtoXMMreg(_Rd_, 0);
-	_deleteMMXreg(MMX_GPR+_Rs_, 1);
-	_deleteMMXreg(MMX_GPR+_Rt_, 1);
-	_deleteMMXreg(MMX_GPR+_Rd_, 0);
 
 	if( GPR_IS_CONST2(_Rs_, _Rt_) ) {
 		GPR_SET_CONST(_Rd_);
@@ -446,8 +426,8 @@ void eeRecompileCodeConst0(R5900FNPTR constcode, R5900FNPTR_INFO constscode, R59
 // rt = rs op imm16
 void eeRecompileCodeConst1(R5900FNPTR constcode, R5900FNPTR_INFO noconstcode)
 {
-    if ( ! _Rt_ )
-        return;
+	if ( ! _Rt_ )
+		return;
 
 	// for now, don't support xmm
 
@@ -527,15 +507,15 @@ int eeRecompileCodeXMM(int xmminfo)
 	// flush consts
 	if( xmminfo & XMMINFO_READT ) {
 		if( GPR_IS_CONST1( _Rt_ ) && !(g_cpuFlushedConstReg&(1<<_Rt_)) ) {
-			MOV32ItoM((uptr)&cpuRegs.GPR.r[ _Rt_ ].UL[ 0 ], g_cpuConstRegs[_Rt_].UL[0]);
-			MOV32ItoM((uptr)&cpuRegs.GPR.r[ _Rt_ ].UL[ 1 ], g_cpuConstRegs[_Rt_].UL[1]);
+			xMOV(ptr32[&cpuRegs.GPR.r[ _Rt_ ].UL[ 0 ]], g_cpuConstRegs[_Rt_].UL[0]);
+			xMOV(ptr32[&cpuRegs.GPR.r[ _Rt_ ].UL[ 1 ]], g_cpuConstRegs[_Rt_].UL[1]);
 			g_cpuFlushedConstReg |= (1<<_Rt_);
 		}
 	}
 	if( xmminfo & XMMINFO_READS) {
 		if( GPR_IS_CONST1( _Rs_ ) && !(g_cpuFlushedConstReg&(1<<_Rs_)) ) {
-			MOV32ItoM((uptr)&cpuRegs.GPR.r[ _Rs_ ].UL[ 0 ], g_cpuConstRegs[_Rs_].UL[0]);
-			MOV32ItoM((uptr)&cpuRegs.GPR.r[ _Rs_ ].UL[ 1 ], g_cpuConstRegs[_Rs_].UL[1]);
+			xMOV(ptr32[&cpuRegs.GPR.r[ _Rs_ ].UL[ 0 ]], g_cpuConstRegs[_Rs_].UL[0]);
+			xMOV(ptr32[&cpuRegs.GPR.r[ _Rs_ ].UL[ 1 ]], g_cpuConstRegs[_Rs_].UL[1]);
 			g_cpuFlushedConstReg |= (1<<_Rs_);
 		}
 	}
@@ -573,7 +553,6 @@ int eeRecompileCodeXMM(int xmminfo)
 		if( regd < 0 ) {
 			if( !(xmminfo&XMMINFO_READD) && (xmminfo & XMMINFO_READT) && (_Rt_ == 0 || (g_pCurInstInfo->regs[_Rt_] & EEINST_LASTUSE) || !EEINST_ISLIVEXMM(_Rt_)) ) {
 				_freeXMMreg(EEREC_T);
-				_deleteMMXreg(MMX_GPR+_Rd_, 2);
 				xmmregs[EEREC_T].inuse = 1;
 				xmmregs[EEREC_T].reg = _Rd_;
 				xmmregs[EEREC_T].mode = readd;
@@ -581,7 +560,6 @@ int eeRecompileCodeXMM(int xmminfo)
 			}
 			else if( !(xmminfo&XMMINFO_READD) && (xmminfo & XMMINFO_READS) && (_Rs_ == 0 || (g_pCurInstInfo->regs[_Rs_] & EEINST_LASTUSE) || !EEINST_ISLIVEXMM(_Rs_)) ) {
 				_freeXMMreg(EEREC_S);
-				_deleteMMXreg(MMX_GPR+_Rd_, 2);
 				xmmregs[EEREC_S].inuse = 1;
 				xmmregs[EEREC_S].reg = _Rd_;
 				xmmregs[EEREC_S].mode = readd;
@@ -661,7 +639,6 @@ void eeFPURecompileCode(R5900FNPTR_INFO xmmcode, R5900FNPTR fpucode, int xmminfo
 					_freeXMMreg(mmregt);
 					info &= ~PROCESS_EE_MODEWRITET;
 				}
-				_deleteMMXreg(MMX_FPU+XMMFPU_ACC, 2);
 				xmmregs[mmregt].inuse = 1;
 				xmmregs[mmregt].reg = 0;
 				xmmregs[mmregt].mode = readacc;
@@ -673,7 +650,6 @@ void eeFPURecompileCode(R5900FNPTR_INFO xmmcode, R5900FNPTR fpucode, int xmminfo
 					_freeXMMreg(mmregs);
 					info &= ~PROCESS_EE_MODEWRITES;
 				}
-				_deleteMMXreg(MMX_FPU+XMMFPU_ACC, 2);
 				xmmregs[mmregs].inuse = 1;
 				xmmregs[mmregs].reg = 0;
 				xmmregs[mmregs].mode = readacc;
@@ -697,7 +673,6 @@ void eeFPURecompileCode(R5900FNPTR_INFO xmmcode, R5900FNPTR fpucode, int xmminfo
 					_freeXMMreg(mmregt);
 					info &= ~PROCESS_EE_MODEWRITET;
 				}
-				_deleteMMXreg(MMX_FPU+_Fd_, 2);
 				xmmregs[mmregt].inuse = 1;
 				xmmregs[mmregt].reg = _Fd_;
 				xmmregs[mmregt].mode = readd;
@@ -708,7 +683,6 @@ void eeFPURecompileCode(R5900FNPTR_INFO xmmcode, R5900FNPTR fpucode, int xmminfo
 					_freeXMMreg(mmregs);
 					info &= ~PROCESS_EE_MODEWRITES;
 				}
-				_deleteMMXreg(MMX_FPU+_Fd_, 2);
 				xmmregs[mmregs].inuse = 1;
 				xmmregs[mmregs].reg = _Fd_;
 				xmmregs[mmregs].mode = readd;
@@ -717,7 +691,6 @@ void eeFPURecompileCode(R5900FNPTR_INFO xmmcode, R5900FNPTR fpucode, int xmminfo
 			else if( (xmminfo&XMMINFO_READACC) && mmregacc >= 0 && (FPUINST_LASTUSE(XMMFPU_ACC) || !FPUINST_ISLIVE(XMMFPU_ACC)) ) {
 				if( FPUINST_ISLIVE(XMMFPU_ACC) )
 					_freeXMMreg(mmregacc);
-				_deleteMMXreg(MMX_FPU+_Fd_, 2);
 				xmmregs[mmregacc].inuse = 1;
 				xmmregs[mmregacc].reg = _Fd_;
 				xmmregs[mmregacc].mode = readd;
