@@ -24,17 +24,42 @@
 #include "GSPng.h"
 #include "GSUtil.h"
 
-#ifdef _WINDOWS
+#ifdef _WIN32
+
+class CPinInfo : public PIN_INFO {
+public:
+	CPinInfo() { pFilter = NULL; }
+	~CPinInfo() { if (pFilter) pFilter->Release(); }
+};
+
+class CFilterInfo : public FILTER_INFO {
+public:
+	CFilterInfo() { pGraph = NULL; }
+	~CFilterInfo() { if (pGraph) pGraph->Release(); }
+};
+
+#define BeginEnumFilters(pFilterGraph, pEnumFilters, pBaseFilter) \
+	{CComPtr<IEnumFilters> pEnumFilters; \
+	if(pFilterGraph && SUCCEEDED(pFilterGraph->EnumFilters(&pEnumFilters))) \
+	{ \
+		for(CComPtr<IBaseFilter> pBaseFilter; S_OK == pEnumFilters->Next(1, &pBaseFilter, 0); pBaseFilter = NULL) \
+		{ \
+
+#define EndEnumFilters }}}
+
+#define BeginEnumPins(pBaseFilter, pEnumPins, pPin) \
+	{CComPtr<IEnumPins> pEnumPins; \
+	if(pBaseFilter && SUCCEEDED(pBaseFilter->EnumPins(&pEnumPins))) \
+	{ \
+		for(CComPtr<IPin> pPin; S_OK == pEnumPins->Next(1, &pPin, 0); pPin = NULL) \
+		{ \
+
+#define EndEnumPins }}}
 
 //
 // GSSource
 //
-
-#ifdef __INTEL_COMPILER
 interface __declspec(uuid("59C193BB-C520-41F3-BC1D-E245B80A86FA"))
-#else
-[uuid("59C193BB-C520-41F3-BC1D-E245B80A86FA")] interface
-#endif
 IGSSource : public IUnknown
 {
 	STDMETHOD(DeliverNewSegment)() PURE;
@@ -42,11 +67,7 @@ IGSSource : public IUnknown
 	STDMETHOD(DeliverEOS)() PURE;
 };
 
-#ifdef __INTEL_COMPILER
 class __declspec(uuid("F8BB6F4F-0965-4ED4-BA74-C6A01E6E6C77"))
-#else
-[uuid("F8BB6F4F-0965-4ED4-BA74-C6A01E6E6C77")] class
-#endif
 GSSource : public CBaseFilter, private CCritSec, public IGSSource
 {
 	GSVector2i m_size;
@@ -63,7 +84,7 @@ GSSource : public CBaseFilter, private CCritSec, public IGSSource
 	class GSSourceOutputPin : public CBaseOutputPin
 	{
 		GSVector2i m_size;
-		vector<CMediaType> m_mts;
+		std::vector<CMediaType> m_mts;
 
 	public:
 		GSSourceOutputPin(const GSVector2i& size, REFERENCE_TIME atpf, CBaseFilter* pFilter, CCritSec* pLock, HRESULT& hr, int colorspace)
@@ -137,9 +158,9 @@ GSSource : public CBaseFilter, private CCritSec, public IGSSource
 
 	    HRESULT CheckMediaType(const CMediaType* pmt)
 		{
-			for(vector<CMediaType>::iterator i = m_mts.begin(); i != m_mts.end(); i++)
+			for(const auto &mt : m_mts)
 			{
-				if(i->majortype == pmt->majortype && i->subtype == pmt->subtype)
+				if(mt.majortype == pmt->majortype && mt.subtype == pmt->subtype)
 				{
 					return S_OK;
 				}
@@ -341,15 +362,6 @@ public:
 	}
 };
 
-#define BeginEnumPins(pBaseFilter, pEnumPins, pPin) \
-	{CComPtr<IEnumPins> pEnumPins; \
-	if(pBaseFilter && SUCCEEDED(pBaseFilter->EnumPins(&pEnumPins))) \
-	{ \
-		for(CComPtr<IPin> pPin; S_OK == pEnumPins->Next(1, &pPin, 0); pPin = NULL) \
-		{ \
-
-#define EndEnumPins }}}
-
 static IPin* GetFirstPin(IBaseFilter* pBF, PIN_DIRECTION dir)
 {
 	if(!pBF) return(NULL);
@@ -380,8 +392,11 @@ GSCapture::GSCapture()
 	: m_capturing(false), m_frame(0)
 	  , m_out_dir("/tmp/GSdx_Capture") // FIXME Later add an option
 {
-	m_out_dir = theApp.GetConfig("capture_out_dir", "/tmp/GSdx_Capture");
-	m_threads = theApp.GetConfig("capture_threads", 4);
+	m_out_dir = theApp.GetConfigS("capture_out_dir");
+	m_threads = theApp.GetConfigI("capture_threads");
+#if defined(__unix__)
+	m_compression_level = theApp.GetConfigI("png_compression_level");
+#endif
 }
 
 GSCapture::~GSCapture()
@@ -389,16 +404,16 @@ GSCapture::~GSCapture()
 	EndCapture();
 }
 
-bool GSCapture::BeginCapture(float fps, GSVector2i recomendedResolution, float aspect)
+bool GSCapture::BeginCapture(float fps, GSVector2i recommendedResolution, float aspect)
 {
-	printf("Recomended resolution: %d x %d, DAR for muxing: %.4f\n", recomendedResolution.x, recomendedResolution.y, aspect);
+	printf("Recommended resolution: %d x %d, DAR for muxing: %.4f\n", recommendedResolution.x, recommendedResolution.y, aspect);
 	std::lock_guard<std::recursive_mutex> lock(m_lock);
 
 	ASSERT(fps != 0);
 
 	EndCapture();
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 
 	GSCaptureDlg dlg;
 
@@ -407,7 +422,7 @@ bool GSCapture::BeginCapture(float fps, GSVector2i recomendedResolution, float a
 	m_size.x = (dlg.m_width + 7) & ~7;
 	m_size.y = (dlg.m_height + 7) & ~7;
 
-	wstring fn(dlg.m_filename.begin(), dlg.m_filename.end());
+	std::wstring fn{dlg.m_filename.begin(), dlg.m_filename.end()};
 
 	//
 
@@ -452,8 +467,8 @@ bool GSCapture::BeginCapture(float fps, GSVector2i recomendedResolution, float a
 	{
 		CFilterInfo fi;
 		pBF->QueryFilterInfo(&fi);
-		wstring s(fi.achName);
-		printf("Filter [%p]: %s\n", pBF.p, string(s.begin(), s.end()).c_str());
+		std::wstring s{fi.achName};
+		printf("Filter [%p]: %s\n", pBF.p, std::string{s.begin(), s.end()}.c_str());
 
 		BeginEnumPins(pBF, pEP, pPin)
 		{
@@ -462,13 +477,8 @@ bool GSCapture::BeginCapture(float fps, GSVector2i recomendedResolution, float a
 
 			CPinInfo pi;
 			pPin->QueryPinInfo(&pi);
-			wstring s(pi.achName);
-			printf("- Pin [%p - %p]: %s (%s)\n", pPin.p, pPinTo.p, string(s.begin(), s.end()).c_str(), pi.dir ? "out" : "in");
-
-			BeginEnumMediaTypes(pPin, pEMT, pmt)
-			{
-			}
-			EndEnumMediaTypes(pmt)
+			std::wstring s{pi.achName};
+			printf("- Pin [%p - %p]: %s (%s)\n", pPin.p, pPinTo.p, std::string{s.begin(), s.end()}.c_str(), pi.dir ? "out" : "in");
 		}
 		EndEnumPins
 	}
@@ -478,22 +488,19 @@ bool GSCapture::BeginCapture(float fps, GSVector2i recomendedResolution, float a
 
 	CComQIPtr<IGSSource>(m_src)->DeliverNewSegment();
 
-#elif __linux__
+#elif defined(__unix__)
 	// Note I think it doesn't support multiple depth creation
 	GSmkdir(m_out_dir.c_str());
 
 	// Really cheap recording
 	m_frame = 0;
 	// Add option !!!
-	m_size.x = theApp.GetConfig("capture_resx", 1280);
-	m_size.y = theApp.GetConfig("capture_resy", 1024);
+	m_size.x = theApp.GetConfigI("CaptureWidth");
+	m_size.y = theApp.GetConfigI("CaptureHeight");
 
-#ifdef __linux__
 	for(int i = 0; i < m_threads; i++) {
-		m_workers.push_back(new GSPng::Worker());
+		m_workers.push_back(std::unique_ptr<GSPng::Worker>(new GSPng::Worker(&GSPng::Process)));
 	}
-#endif
-
 #endif
 
 	m_capturing = true;
@@ -512,7 +519,7 @@ bool GSCapture::DeliverFrame(const void* bits, int pitch, bool rgba)
 		return false;
 	}
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 
 	if(m_src)
 	{
@@ -521,11 +528,11 @@ bool GSCapture::DeliverFrame(const void* bits, int pitch, bool rgba)
 		return true;
 	}
 
-#elif __linux__
+#elif defined(__unix__)
 
 	std::string out_file = m_out_dir + format("/frame.%010d.png", m_frame);
-	//GSPng::Save(GSPng::RGB_PNG, out_file, (char*)bits, m_size.x, m_size.y, pitch);
-	m_workers[m_frame%m_threads]->Push(shared_ptr<GSPng::Transaction>(new GSPng::Transaction(GSPng::RGB_PNG, out_file, (char*)bits, m_size.x, m_size.y, pitch)));
+	//GSPng::Save(GSPng::RGB_PNG, out_file, (uint8*)bits, m_size.x, m_size.y, pitch, m_compression_level);
+	m_workers[m_frame%m_threads]->Push(std::make_shared<GSPng::Transaction>(GSPng::RGB_PNG, out_file, static_cast<const uint8*>(bits), m_size.x, m_size.y, pitch, m_compression_level));
 
 	m_frame++;
 
@@ -538,7 +545,7 @@ bool GSCapture::EndCapture()
 {
 	std::lock_guard<std::recursive_mutex> lock(m_lock);
 
-#ifdef _WINDOWS
+#ifdef _WIN32
 
 	if(m_src)
 	{
@@ -554,10 +561,8 @@ bool GSCapture::EndCapture()
 		m_graph = NULL;
 	}
 
-#elif __linux__
-	for(size_t i = 0; i < m_workers.size(); i++) {
-		m_workers[i]->Wait();
-	}
+#elif defined(__unix__)
+	m_workers.clear();
 
 	m_frame = 0;
 
