@@ -26,22 +26,23 @@
 
 #include "stdafx.h"
 #include "GSLocalMemory.h"
+#include "GSdx.h"
 
 #define ASSERT_BLOCK(r, w, h) \
-	ASSERT((r).width() >= w && (r).height() >= h && !((r).left & (w - 1)) && !((r).top & (h - 1)) && !((r).right & (w - 1)) && !((r).bottom & (h - 1))); \
+	ASSERT((r).width() >= (w) && (r).height() >= (h) && !((r).left & ((w) - 1)) && !((r).top & ((h) - 1)) && !((r).right & ((w) - 1)) && !((r).bottom & ((h) - 1))); \
 
 #define FOREACH_BLOCK_START(r, w, h, bpp) \
 	ASSERT_BLOCK(r, w, h); \
-	GSVector4i _r = r >> 3; \
-	uint8* _dst = dst - _r.left * bpp; \
-	int _offset = dstpitch * h; \
-	for(int y = _r.top; y < _r.bottom; y += h >> 3, _dst += _offset) \
+	GSVector4i _r = (r) >> 3; \
+	uint8* _dst = dst - _r.left * (bpp); \
+	int _offset = dstpitch * (h); \
+	for(int y = _r.top; y < _r.bottom; y += (h) >> 3, _dst += _offset) \
 	{ \
-		uint32 _base = o->block.row[y]; \
-		for(int x = _r.left; x < _r.right; x += w >> 3) \
+		uint32 _base = off->block.row[y]; \
+		for(int x = _r.left; x < _r.right; x += (w) >> 3) \
 		{ \
-			const uint8* src = BlockPtr(_base + o->block.col[x]); \
-			uint8* dst = &_dst[x * bpp]; \
+			const uint8* src = BlockPtr(_base + off->block.col[x]); \
+			uint8* read_dst = &_dst[x * (bpp)]; \
 
 #define FOREACH_BLOCK_END }}
 
@@ -83,7 +84,27 @@ GSLocalMemory::psm_t GSLocalMemory::m_psm[64];
 GSLocalMemory::GSLocalMemory()
 	: m_clut(this)
 {
-	m_vm8 = (uint8*)vmalloc(m_vmsize * 2, false);
+	m_use_fifo_alloc = theApp.GetConfigB("UserHacks") && theApp.GetConfigB("wrap_gs_mem");
+	switch (theApp.GetCurrentRendererType()) {
+		case GSRendererType::OGL_SW:
+		case GSRendererType::DX1011_SW:
+			m_use_fifo_alloc = true;
+			break;
+		default: break;
+	}
+
+	if (m_use_fifo_alloc)
+		m_vm8 = (uint8*)fifo_alloc(m_vmsize, 4);
+	else
+		m_vm8 = nullptr;
+
+	// Either we don't use fifo alloc or we get an error.
+	if (m_vm8 == nullptr)
+	{
+		m_vm8 = (uint8*)vmalloc(m_vmsize * 4, false);
+		m_use_fifo_alloc = false;
+	}
+
 	m_vm16 = (uint16*)m_vm8;
 	m_vm32 = (uint32*)m_vm8;
 
@@ -222,8 +243,10 @@ GSLocalMemory::GSLocalMemory()
 		for(int j = 0; j < 8; j++) m_psm[i].rowOffset[j] = rowOffset32;
 		m_psm[i].blockOffset = blockOffset32;
 		m_psm[i].msk = 0xff;
+		m_psm[i].depth = 0;
 	}
 
+	m_psm[PSM_PSGPU24].pa = &GSLocalMemory::PixelAddress16;
 	m_psm[PSM_PSMCT16].pa = &GSLocalMemory::PixelAddress16;
 	m_psm[PSM_PSMCT16S].pa = &GSLocalMemory::PixelAddress16S;
 	m_psm[PSM_PSMT8].pa = &GSLocalMemory::PixelAddress8;
@@ -233,6 +256,7 @@ GSLocalMemory::GSLocalMemory()
 	m_psm[PSM_PSMZ16].pa = &GSLocalMemory::PixelAddress16Z;
 	m_psm[PSM_PSMZ16S].pa = &GSLocalMemory::PixelAddress16SZ;
 
+	m_psm[PSM_PSGPU24].bn = &GSLocalMemory::BlockNumber16;
 	m_psm[PSM_PSMCT16].bn = &GSLocalMemory::BlockNumber16;
 	m_psm[PSM_PSMCT16S].bn = &GSLocalMemory::BlockNumber16S;
 	m_psm[PSM_PSMT8].bn = &GSLocalMemory::BlockNumber8;
@@ -342,6 +366,7 @@ GSLocalMemory::GSLocalMemory()
 	m_psm[PSM_PSMZ16S].wi = &GSLocalMemory::WriteImage<PSM_PSMZ16S, 16, 8, 16>;
 
 	m_psm[PSM_PSMCT24].rtx = &GSLocalMemory::ReadTexture24;
+	m_psm[PSM_PSGPU24].rtx = &GSLocalMemory::ReadTextureGPU24;
 	m_psm[PSM_PSMCT16].rtx = &GSLocalMemory::ReadTexture16;
 	m_psm[PSM_PSMCT16S].rtx = &GSLocalMemory::ReadTexture16;
 	m_psm[PSM_PSMT8].rtx = &GSLocalMemory::ReadTexture8;
@@ -393,12 +418,14 @@ GSLocalMemory::GSLocalMemory()
 	m_psm[PSM_PSMZ16].rtxbP = &GSLocalMemory::ReadTextureBlock16;
 	m_psm[PSM_PSMZ16S].rtxbP = &GSLocalMemory::ReadTextureBlock16;
 
+	m_psm[PSM_PSGPU24].bpp = 16;
 	m_psm[PSM_PSMCT16].bpp = m_psm[PSM_PSMCT16S].bpp = 16;
 	m_psm[PSM_PSMT8].bpp = 8;
 	m_psm[PSM_PSMT4].bpp = 4;
 	m_psm[PSM_PSMZ16].bpp = m_psm[PSM_PSMZ16S].bpp = 16;
 
 	m_psm[PSM_PSMCT24].trbpp = 24;
+	m_psm[PSM_PSGPU24].trbpp = 16;
 	m_psm[PSM_PSMCT16].trbpp = m_psm[PSM_PSMCT16S].trbpp = 16;
 	m_psm[PSM_PSMT8].trbpp = m_psm[PSM_PSMT8H].trbpp = 8;
 	m_psm[PSM_PSMT4].trbpp = m_psm[PSM_PSMT4HL].trbpp = m_psm[PSM_PSMT4HH].trbpp = 4;
@@ -414,16 +441,20 @@ GSLocalMemory::GSLocalMemory()
 	m_psm[PSM_PSMCT16].fmt = m_psm[PSM_PSMZ16].fmt = 2;
 	m_psm[PSM_PSMCT16S].fmt = m_psm[PSM_PSMZ16S].fmt = 2;
 
+
+	m_psm[PSM_PSGPU24].bs = GSVector2i(16, 8);
 	m_psm[PSM_PSMCT16].bs = m_psm[PSM_PSMCT16S].bs = GSVector2i(16, 8);
 	m_psm[PSM_PSMT8].bs = GSVector2i(16, 16);
 	m_psm[PSM_PSMT4].bs = GSVector2i(32, 16);
 	m_psm[PSM_PSMZ16].bs = m_psm[PSM_PSMZ16S].bs = GSVector2i(16, 8);
 
+	m_psm[PSM_PSGPU24].pgs = GSVector2i(64, 64);
 	m_psm[PSM_PSMCT16].pgs = m_psm[PSM_PSMCT16S].pgs = GSVector2i(64, 64);
 	m_psm[PSM_PSMT8].pgs = GSVector2i(128, 64);
 	m_psm[PSM_PSMT4].pgs = GSVector2i(128, 128);
 	m_psm[PSM_PSMZ16].pgs = m_psm[PSM_PSMZ16S].pgs = GSVector2i(64, 64);
 
+	for(int i = 0; i < 8; i++) m_psm[PSM_PSGPU24].rowOffset[i] = rowOffset16;
 	for(int i = 0; i < 8; i++) m_psm[PSM_PSMCT16].rowOffset[i] = rowOffset16;
 	for(int i = 0; i < 8; i++) m_psm[PSM_PSMCT16S].rowOffset[i] = rowOffset16S;
 	for(int i = 0; i < 8; i++) m_psm[PSM_PSMT8].rowOffset[i] = rowOffset8[((i + 2) >> 2) & 1];
@@ -433,6 +464,7 @@ GSLocalMemory::GSLocalMemory()
 	for(int i = 0; i < 8; i++) m_psm[PSM_PSMZ16].rowOffset[i] = rowOffset16Z;
 	for(int i = 0; i < 8; i++) m_psm[PSM_PSMZ16S].rowOffset[i] = rowOffset16SZ;
 
+	m_psm[PSM_PSGPU24].blockOffset = blockOffset16;
 	m_psm[PSM_PSMCT16].blockOffset = blockOffset16;
 	m_psm[PSM_PSMCT16S].blockOffset = blockOffset16S;
 	m_psm[PSM_PSMT8].blockOffset = blockOffset8;
@@ -447,19 +479,27 @@ GSLocalMemory::GSLocalMemory()
 	m_psm[PSM_PSMT8H].msk = 0xc0;
 	m_psm[PSM_PSMT4HL].msk = 0x40;
 	m_psm[PSM_PSMT4HH].msk = 0x80;
+
+	m_psm[PSM_PSMZ32].depth  = 1;
+	m_psm[PSM_PSMZ24].depth  = 1;
+	m_psm[PSM_PSMZ16].depth  = 1;
+	m_psm[PSM_PSMZ16S].depth = 1;
 }
 
 GSLocalMemory::~GSLocalMemory()
 {
-	vmfree(m_vm8, m_vmsize * 2);
+	if (m_use_fifo_alloc)
+		fifo_free(m_vm8, m_vmsize, 4);
+	else
+		vmfree(m_vm8, m_vmsize * 4);
 
-	for_each(m_omap.begin(), m_omap.end(), aligned_free_second());
-	for_each(m_pomap.begin(), m_pomap.end(), aligned_free_second());
-	for_each(m_po4map.begin(), m_po4map.end(), aligned_free_second());
+	for(auto &i : m_omap) delete i.second;
+	for(auto &i : m_pomap) _aligned_free(i.second);
+	for(auto &i : m_po4map) _aligned_free(i.second);
 
-	for(hash_map<uint64, vector<GSVector2i>*>::iterator i = m_p2tmap.begin(); i != m_p2tmap.end(); i++)
+	for(auto &i : m_p2tmap)
 	{
-		delete [] i->second;
+		delete [] i.second;
 	}
 }
 
@@ -467,18 +507,18 @@ GSOffset* GSLocalMemory::GetOffset(uint32 bp, uint32 bw, uint32 psm)
 {
 	uint32 hash = bp | (bw << 14) | (psm << 20);
 
-	hash_map<uint32, GSOffset*>::iterator i = m_omap.find(hash);
+	auto i = m_omap.find(hash);
 
 	if(i != m_omap.end())
 	{
 		return i->second;
 	}
 
-	GSOffset* o = new GSOffset(bp, bw, psm);
+	GSOffset* off = new GSOffset(bp, bw, psm);
 
-	m_omap[hash] = o;
+	m_omap[hash] = off;
 
-	return o;
+	return off;
 }
 
 GSPixelOffset* GSLocalMemory::GetPixelOffset(const GIFRegFRAME& FRAME, const GIFRegZBUF& ZBUF)
@@ -498,21 +538,21 @@ GSPixelOffset* GSLocalMemory::GetPixelOffset(const GIFRegFRAME& FRAME, const GIF
 
 	uint32 hash = (FRAME.FBP << 0) | (ZBUF.ZBP << 9) | (bw << 18) | (fpsm_hash << 24) | (zpsm_hash << 28);
 
-	hash_map<uint32, GSPixelOffset*>::iterator i = m_pomap.find(hash);
+	auto it = m_pomap.find(hash);
 
-	if(i != m_pomap.end())
+	if(it != m_pomap.end())
 	{
-		return i->second;
+		return it->second;
 	}
 
-	GSPixelOffset* o = (GSPixelOffset*)_aligned_malloc(sizeof(GSPixelOffset), 32);
+	GSPixelOffset* off = (GSPixelOffset*)_aligned_malloc(sizeof(GSPixelOffset), 32);
 
-	o->hash = hash;
-	o->fbp = fbp;
-	o->zbp = zbp;
-	o->fpsm = fpsm;
-	o->zpsm = zpsm;
-	o->bw = bw;
+	off->hash = hash;
+	off->fbp = fbp;
+	off->zbp = zbp;
+	off->fpsm = fpsm;
+	off->zpsm = zpsm;
+	off->bw = bw;
 
 	pixelAddress fpa = m_psm[fpsm].pa;
 	pixelAddress zpa = m_psm[zpsm].pa;
@@ -522,19 +562,19 @@ GSPixelOffset* GSLocalMemory::GetPixelOffset(const GIFRegFRAME& FRAME, const GIF
 
 	for(int i = 0; i < 2048; i++)
 	{
-		o->row[i].x = (int)fpa(0, i, fbp, bw) << fs;
-		o->row[i].y = (int)zpa(0, i, zbp, bw) << zs;
+		off->row[i].x = (int)fpa(0, i, fbp, bw) << fs;
+		off->row[i].y = (int)zpa(0, i, zbp, bw) << zs;
 	}
 
 	for(int i = 0; i < 2048; i++)
 	{
-		o->col[i].x = m_psm[fpsm].rowOffset[0][i] << fs;
-		o->col[i].y = m_psm[zpsm].rowOffset[0][i] << zs;
+		off->col[i].x = m_psm[fpsm].rowOffset[0][i] << fs;
+		off->col[i].y = m_psm[zpsm].rowOffset[0][i] << zs;
 	}
 
-	m_pomap[hash] = o;
+	m_pomap[hash] = off;
 
-	return o;
+	return off;
 }
 
 GSPixelOffset4* GSLocalMemory::GetPixelOffset4(const GIFRegFRAME& FRAME, const GIFRegZBUF& ZBUF)
@@ -554,21 +594,21 @@ GSPixelOffset4* GSLocalMemory::GetPixelOffset4(const GIFRegFRAME& FRAME, const G
 
 	uint32 hash = (FRAME.FBP << 0) | (ZBUF.ZBP << 9) | (bw << 18) | (fpsm_hash << 24) | (zpsm_hash << 28);
 
-	hash_map<uint32, GSPixelOffset4*>::iterator i = m_po4map.find(hash);
+	auto it = m_po4map.find(hash);
 
-	if(i != m_po4map.end())
+	if(it != m_po4map.end())
 	{
-		return i->second;
+		return it->second;
 	}
 
-	GSPixelOffset4* o = (GSPixelOffset4*)_aligned_malloc(sizeof(GSPixelOffset4), 32);
+	GSPixelOffset4* off = (GSPixelOffset4*)_aligned_malloc(sizeof(GSPixelOffset4), 32);
 
-	o->hash = hash;
-	o->fbp = fbp;
-	o->zbp = zbp;
-	o->fpsm = fpsm;
-	o->zpsm = zpsm;
-	o->bw = bw;
+	off->hash = hash;
+	off->fbp = fbp;
+	off->zbp = zbp;
+	off->fpsm = fpsm;
+	off->zpsm = zpsm;
+	off->bw = bw;
 
 	pixelAddress fpa = m_psm[fpsm].pa;
 	pixelAddress zpa = m_psm[zpsm].pa;
@@ -578,32 +618,32 @@ GSPixelOffset4* GSLocalMemory::GetPixelOffset4(const GIFRegFRAME& FRAME, const G
 
 	for(int i = 0; i < 2048; i++)
 	{
-		o->row[i].x = (int)fpa(0, i, fbp, bw) << fs;
-		o->row[i].y = (int)zpa(0, i, zbp, bw) << zs;
+		off->row[i].x = (int)fpa(0, i, fbp, bw) << fs;
+		off->row[i].y = (int)zpa(0, i, zbp, bw) << zs;
 	}
 
 	for(int i = 0; i < 512; i++)
 	{
-		o->col[i].x = m_psm[fpsm].rowOffset[0][i * 4] << fs;
-		o->col[i].y = m_psm[zpsm].rowOffset[0][i * 4] << zs;
+		off->col[i].x = m_psm[fpsm].rowOffset[0][i * 4] << fs;
+		off->col[i].y = m_psm[zpsm].rowOffset[0][i * 4] << zs;
 	}
 
-	m_po4map[hash] = o;
+	m_po4map[hash] = off;
 
-	return o;
+	return off;
 }
 
 static bool cmp_vec2x(const GSVector2i& a, const GSVector2i& b) {return a.x < b.x;}
 
-vector<GSVector2i>* GSLocalMemory::GetPage2TileMap(const GIFRegTEX0& TEX0)
+std::vector<GSVector2i>* GSLocalMemory::GetPage2TileMap(const GIFRegTEX0& TEX0)
 {
 	uint64 hash = TEX0.u64 & 0x3ffffffffull; // TBP0 TBW PSM TW TH
 
-	hash_map<uint64, vector<GSVector2i>*>::iterator i = m_p2tmap.find(hash);
+	auto it = m_p2tmap.find(hash);
 
-	if(i != m_p2tmap.end())
+	if(it != m_p2tmap.end())
 	{
-		return i->second;
+		return it->second;
 	}
 
 	GSVector2i bs = m_psm[TEX0.PSM].bs;
@@ -611,45 +651,40 @@ vector<GSVector2i>* GSLocalMemory::GetPage2TileMap(const GIFRegTEX0& TEX0)
 	int tw = std::max<int>(1 << TEX0.TW, bs.x);
 	int th = std::max<int>(1 << TEX0.TH, bs.y);
 
-	const GSOffset* o = GetOffset(TEX0.TBP0, TEX0.TBW, TEX0.PSM);
+	const GSOffset* off = GetOffset(TEX0.TBP0, TEX0.TBW, TEX0.PSM);
 
-	hash_map<uint32, hash_set<uint32> > tmp; // key = page, value = y:x, 7 bits each, max 128x128 tiles for the worst case (1024x1024 32bpp 8x8 blocks)
+	std::unordered_map<uint32, std::unordered_set<uint32>> tmp; // key = page, value = y:x, 7 bits each, max 128x128 tiles for the worst case (1024x1024 32bpp 8x8 blocks)
 
 	for(int y = 0; y < th; y += bs.y)
 	{
-		uint32 base = o->block.row[y >> 3];
+		uint32 base = off->block.row[y >> 3];
 
 		for(int x = 0, i = y << 7; x < tw; x += bs.x, i += bs.x)
 		{
-			uint32 page = (base + o->block.col[x >> 3]) >> 5;
+			uint32 page = ((base + off->block.col[x >> 3]) >> 5) % MAX_PAGES;
 
-			if(page < MAX_PAGES)
-			{
-				tmp[page].insert(i >> 3); // ((y << 7) | x) >> 3
-			}
+			tmp[page].insert(i >> 3); // ((y << 7) | x) >> 3
 		}
 	}
 
 	// combine the lower 5 bits of the address into a 9:5 pointer:mask form, so the "valid bits" can be tested against an uint32 array
 
-	vector<GSVector2i>* p2t = new vector<GSVector2i>[MAX_PAGES];
+	auto p2t = new std::vector<GSVector2i>[MAX_PAGES];
 
-	for(hash_map<uint32, hash_set<uint32> >::iterator i = tmp.begin(); i != tmp.end(); i++)
+	for(const auto &i : tmp)
 	{
-		uint32 page = i->first;
+		uint32 page = i.first;
 
-		hash_set<uint32>& tiles = i->second;
+		auto& tiles = i.second;
 
-		hash_map<uint32, uint32> m;
+		std::unordered_map<uint32, uint32> m;
 
-		for(hash_set<uint32>::iterator j = tiles.begin(); j != tiles.end(); j++)
+		for(const auto addr : tiles)
 		{
-			uint32 addr = *j;
-
 			uint32 row = addr >> 5;
 			uint32 col = 1 << (addr & 31);
 
-			hash_map<uint32, uint32>::iterator k = m.find(row);
+			auto k = m.find(row);
 
 			if(k != m.end())
 			{
@@ -661,11 +696,14 @@ vector<GSVector2i>* GSLocalMemory::GetPage2TileMap(const GIFRegTEX0& TEX0)
 			}
 		}
 
+		// Allocate vector with initial size
+		p2t[page].reserve(m.size());
+
 		// sort by x and flip the mask (it will be used to erase a lot of bits in a loop, [x] &= ~y)
 
-		for(hash_map<uint32, uint32>::iterator j = m.begin(); j != m.end(); j++)
+		for(const auto &j : m)
 		{
-			p2t[page].push_back(GSVector2i(j->first, ~j->second));
+			p2t[page].push_back(GSVector2i(j.first, ~j.second));
 		}
 
 		std::sort(p2t[page].begin(), p2t[page].end(), cmp_vec2x);
@@ -764,7 +802,7 @@ void GSLocalMemory::WriteImageLeftRight(int l, int r, int y, int h, const uint8*
 template<int psm, int bsx, int bsy, int trbpp>
 void GSLocalMemory::WriteImageTopBottom(int l, int r, int y, int h, const uint8* src, int srcpitch, const GIFRegBITBLTBUF& BITBLTBUF)
 {
-	__aligned(uint8, 32) buff[64]; // merge buffer for one column
+	alignas(32) uint8 buff[64]; // merge buffer for one column
 
 	uint32 bp = BITBLTBUF.DBP;
 	uint32 bw = BITBLTBUF.DBW;
@@ -777,7 +815,7 @@ void GSLocalMemory::WriteImageTopBottom(int l, int r, int y, int h, const uint8*
 
 	if(y2 > 0)
 	{
-		int h2 = min(h, csy - y2);
+		int h2 = std::min(h, csy - y2);
 
 		for(int x = l; x < r; x += bsx)
 		{
@@ -930,7 +968,7 @@ void GSLocalMemory::WriteImage(int& tx, int& ty, const uint8* src, int len, GIFR
 
 	if(tx != l)
 	{
-		int n = min(len, (r - tx) * trbpp >> 3);
+		int n = std::min(len, (r - tx) * trbpp >> 3);
 		WriteImageX(tx, ty, src, n, BITBLTBUF, TRXPOS, TRXREG);
 		src += n;
 		len -= n;
@@ -969,7 +1007,7 @@ void GSLocalMemory::WriteImage(int& tx, int& ty, const uint8* src, int len, GIFR
 			// top part
 
 			{
-				int h2 = min(h, bsy - (ty & (bsy - 1)));
+				int h2 = std::min(h, bsy - (ty & (bsy - 1)));
 
 				if(h2 < bsy)
 				{
@@ -1031,8 +1069,10 @@ void GSLocalMemory::WriteImage(int& tx, int& ty, const uint8* src, int len, GIFR
 }
 
 
-#define IsTopLeftAligned(dsax, tx, ty, bw, bh) \
-	((((int)dsax) & ((bw)-1)) == 0 && ((tx) & ((bw)-1)) == 0 && ((int)dsax) == (tx) && ((ty) & ((bh)-1)) == 0)
+static bool IsTopLeftAligned(int dsax, int tx, int ty, int bw, int bh)
+{
+	return ((dsax & (bw-1)) == 0 && (tx & (bw-1)) == 0 && dsax == tx && (ty & (bh-1)) == 0);
+}
 
 void GSLocalMemory::WriteImage24(int& tx, int& ty, const uint8* src, int len, GIFRegBITBLTBUF& BITBLTBUF, GIFRegTRXPOS& TRXPOS, GIFRegTRXREG& TRXREG)
 {
@@ -1422,10 +1462,10 @@ void GSLocalMemory::ReadImageX(int& tx, int& ty, uint8* dst, int len, GIFRegBITB
 
 			for(int ex8 = ex - 8; len >= 8 && x <= ex8; len -= 8, x += 8, pd += 8)
 			{
-				int o = offset[x];
+				int off = offset[x];
 
-				GSVector4i::store<false>(&pd[0], GSVector4i::load(&ps[o + 0], &ps[o + 4]));
-				GSVector4i::store<false>(&pd[4], GSVector4i::load(&ps[o + 8], &ps[o + 12]));
+				GSVector4i::store<false>(&pd[0], GSVector4i::load(&ps[off + 0], &ps[off + 4]));
+				GSVector4i::store<false>(&pd[4], GSVector4i::load(&ps[off + 8], &ps[off + 12]));
 
 				for(int i = 0; i < 8; i++) ASSERT(pd[i] == ps[offset[x + i]]);
 			}
@@ -1608,22 +1648,22 @@ void GSLocalMemory::ReadImageX(int& tx, int& ty, uint8* dst, int len, GIFRegBITB
 
 ///////////////////
 
-void GSLocalMemory::ReadTexture32(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture32(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	FOREACH_BLOCK_START(r, 8, 8, 32)
 	{
-		GSBlock::ReadBlock32(src, dst, dstpitch);
+		GSBlock::ReadBlock32(src, read_dst, dstpitch);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture24(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture24(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	if(TEXA.AEM)
 	{
 		FOREACH_BLOCK_START(r, 8, 8, 32)
 		{
-			GSBlock::ReadAndExpandBlock24<true>(src, dst, dstpitch, TEXA);
+			GSBlock::ReadAndExpandBlock24<true>(src, read_dst, dstpitch, TEXA);
 		}
 		FOREACH_BLOCK_END
 	}
@@ -1631,19 +1671,38 @@ void GSLocalMemory::ReadTexture24(const GSOffset* RESTRICT o, const GSVector4i&
 	{
 		FOREACH_BLOCK_START(r, 8, 8, 32)
 		{
-			GSBlock::ReadAndExpandBlock24<false>(src, dst, dstpitch, TEXA);
+			GSBlock::ReadAndExpandBlock24<false>(src, read_dst, dstpitch, TEXA);
 		}
 		FOREACH_BLOCK_END
 	}
 }
 
-void GSLocalMemory::ReadTexture16(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTextureGPU24(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+{
+	FOREACH_BLOCK_START(r, 16, 8, 16)
+	{
+		GSBlock::ReadBlock16(src, read_dst, dstpitch);
+	}
+	FOREACH_BLOCK_END
+
+	// Convert packed RGB scanline to 32 bits RGBA
+	ASSERT(dstpitch >= r.width() * 4);
+	for(int y = r.top; y < r.bottom; y ++) {
+		uint8* line = dst + y * dstpitch;
+
+		for(int x = r.right; x >= r.left; x--) {
+			*(uint32*)&line[x * 4] = *(uint32*)&line[x * 3] & 0xFFFFFF;
+		}
+	}
+}
+
+void GSLocalMemory::ReadTexture16(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	if(TEXA.AEM)
 	{
 		FOREACH_BLOCK_START(r, 16, 8, 32)
 		{
-			GSBlock::ReadAndExpandBlock16<true>(src, dst, dstpitch, TEXA);
+			GSBlock::ReadAndExpandBlock16<true>(src, read_dst, dstpitch, TEXA);
 		}
 		FOREACH_BLOCK_END
 	}
@@ -1651,63 +1710,63 @@ void GSLocalMemory::ReadTexture16(const GSOffset* RESTRICT o, const GSVector4i&
 	{
 		FOREACH_BLOCK_START(r, 16, 8, 32)
 		{
-			GSBlock::ReadAndExpandBlock16<false>(src, dst, dstpitch, TEXA);
+			GSBlock::ReadAndExpandBlock16<false>(src, read_dst, dstpitch, TEXA);
 		}
 		FOREACH_BLOCK_END
 	}
 }
 
-void GSLocalMemory::ReadTexture8(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture8(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	const uint32* pal = m_clut;
 
 	FOREACH_BLOCK_START(r, 16, 16, 32)
 	{
-		GSBlock::ReadAndExpandBlock8_32(src, dst, dstpitch, pal);
+		GSBlock::ReadAndExpandBlock8_32(src, read_dst, dstpitch, pal);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture4(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture4(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	const uint64* pal = m_clut;
 
 	FOREACH_BLOCK_START(r, 32, 16, 32)
 	{
-		GSBlock::ReadAndExpandBlock4_32(src, dst, dstpitch, pal);
+		GSBlock::ReadAndExpandBlock4_32(src, read_dst, dstpitch, pal);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture8H(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture8H(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	const uint32* pal = m_clut;
 
 	FOREACH_BLOCK_START(r, 8, 8, 32)
 	{
-		GSBlock::ReadAndExpandBlock8H_32(src, dst, dstpitch, pal);
+		GSBlock::ReadAndExpandBlock8H_32(src, read_dst, dstpitch, pal);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture4HL(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture4HL(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	const uint32* pal = m_clut;
 
 	FOREACH_BLOCK_START(r, 8, 8, 32)
 	{
-		GSBlock::ReadAndExpandBlock4HL_32(src, dst, dstpitch, pal);
+		GSBlock::ReadAndExpandBlock4HL_32(src, read_dst, dstpitch, pal);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture4HH(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture4HH(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	const uint32* pal = m_clut;
 
 	FOREACH_BLOCK_START(r, 8, 8, 32)
 	{
-		GSBlock::ReadAndExpandBlock4HH_32(src, dst, dstpitch, pal);
+		GSBlock::ReadAndExpandBlock4HH_32(src, read_dst, dstpitch, pal);
 	}
 	FOREACH_BLOCK_END
 }
@@ -1786,9 +1845,9 @@ void GSLocalMemory::ReadTextureBlock4HH(uint32 bp, uint8* dst, int dstpitch, con
 
 ///////////////////
 
-void GSLocalMemory::ReadTexture(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
-	const psm_t& psm = m_psm[o->psm];
+	const psm_t& psm = m_psm[off->psm];
 
 	readTexel rt = psm.rt;
 	readTexture rtx = psm.rtx;
@@ -1799,9 +1858,9 @@ void GSLocalMemory::ReadTexture(const GSOffset* RESTRICT o, const GSVector4i& r,
 	{
 		GIFRegTEX0 TEX0;
 
-		TEX0.TBP0 = o->bp;
-		TEX0.TBW = o->bw;
-		TEX0.PSM = o->psm;
+		TEX0.TBP0 = off->bp;
+		TEX0.TBW = off->bw;
+		TEX0.PSM = off->psm;
 
 		GSVector4i cr = r.ralign<Align_Inside>(psm.bs);
 
@@ -1854,59 +1913,59 @@ void GSLocalMemory::ReadTexture(const GSOffset* RESTRICT o, const GSVector4i& r,
 
 			if(!cr.rempty())
 			{
-				(this->*rtx)(o, cr, dst + (cr.left - r.left) * sizeof(32), dstpitch, TEXA);
+				(this->*rtx)(off, cr, dst + (cr.left - r.left) * sizeof(uint32), dstpitch, TEXA);
 			}
 		}
 	}
 	else
 	{
-		(this->*rtx)(o, r, dst, dstpitch, TEXA);
+		(this->*rtx)(off, r, dst, dstpitch, TEXA);
 	}
 }
 
 // 32/8
 
-void GSLocalMemory::ReadTexture8P(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture8P(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	FOREACH_BLOCK_START(r, 16, 16, 8)
 	{
-		GSBlock::ReadBlock8(src, dst, dstpitch);
+		GSBlock::ReadBlock8(src, read_dst, dstpitch);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture4P(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture4P(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	FOREACH_BLOCK_START(r, 32, 16, 8)
 	{
-		GSBlock::ReadBlock4P(src, dst, dstpitch);
+		GSBlock::ReadBlock4P(src, read_dst, dstpitch);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture8HP(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture8HP(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	FOREACH_BLOCK_START(r, 8, 8, 8)
 	{
-		GSBlock::ReadBlock8HP(src, dst, dstpitch);
+		GSBlock::ReadBlock8HP(src, read_dst, dstpitch);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture4HLP(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture4HLP(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	FOREACH_BLOCK_START(r, 8, 8, 8)
 	{
-		GSBlock::ReadBlock4HLP(src, dst, dstpitch);
+		GSBlock::ReadBlock4HLP(src, read_dst, dstpitch);
 	}
 	FOREACH_BLOCK_END
 }
 
-void GSLocalMemory::ReadTexture4HHP(const GSOffset* RESTRICT o, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
+void GSLocalMemory::ReadTexture4HHP(const GSOffset* RESTRICT off, const GSVector4i& r, uint8* dst, int dstpitch, const GIFRegTEXA& TEXA)
 {
 	FOREACH_BLOCK_START(r, 8, 8, 8)
 	{
-		GSBlock::ReadBlock4HHP(src, dst, dstpitch);
+		GSBlock::ReadBlock4HHP(src, read_dst, dstpitch);
 	}
 	FOREACH_BLOCK_END
 }
@@ -1948,9 +2007,9 @@ void GSLocalMemory::ReadTextureBlock4HHP(uint32 bp, uint8* dst, int dstpitch, co
 
 //
 
-#include "GSTextureSW.h"
+#include "Renderers/SW/GSTextureSW.h"
 
-void GSLocalMemory::SaveBMP(const string& fn, uint32 bp, uint32 bw, uint32 psm, int w, int h)
+void GSLocalMemory::SaveBMP(const std::string& fn, uint32 bp, uint32 bw, uint32 psm, int w, int h)
 {
 	int pitch = w * 4;
 	int size = pitch * h;
@@ -2010,10 +2069,14 @@ GSOffset::GSOffset(uint32 _bp, uint32 _bw, uint32 _psm)
 	{
 		pixel.col[i] = GSLocalMemory::m_psm[_psm].rowOffset[i];
 	}
+
+	pages_as_bit.fill(nullptr);
 }
 
 GSOffset::~GSOffset()
 {
+	for(auto buffer: pages_as_bit)
+		_aligned_free(buffer);
 }
 
 uint32* GSOffset::GetPages(const GSVector4i& rect, uint32* pages, GSVector4i* bbox)
@@ -2024,12 +2087,12 @@ uint32* GSOffset::GetPages(const GSVector4i& rect, uint32* pages, GSVector4i* bb
 
 	if(bbox != NULL) *bbox = r;
 
-	// worst case: 
+	// worst case:
 	// bp page-aligned: (w * h) / (64 * 32)
 	// bp block-aligned: (w * h) / (8 * 8)
 
 	int size = r.width() * r.height();
-	
+
 	int limit = MAX_PAGES + 1;
 
 	if(pages == NULL)
@@ -2039,7 +2102,7 @@ uint32* GSOffset::GetPages(const GSVector4i& rect, uint32* pages, GSVector4i* bb
 		pages = new uint32[limit];
 	}
 
-	__aligned(uint32, 16) tmp[16];
+	alignas(16) uint32 tmp[16];
 
 	((GSVector4i*)tmp)[0] = GSVector4i::zero();
 	((GSVector4i*)tmp)[1] = GSVector4i::zero();
@@ -2052,26 +2115,23 @@ uint32* GSOffset::GetPages(const GSVector4i& rect, uint32* pages, GSVector4i* bb
 	bs.y >>= 3;
 
 	uint32* RESTRICT p = pages;
-	
+
 	for(int y = r.top; y < r.bottom; y += bs.y)
 	{
 		uint32 base = block.row[y];
 
 		for(int x = r.left; x < r.right; x += bs.x)
 		{
-			uint32 n = (base + block.col[x]) >> 5;
+			uint32 n = ((base + block.col[x]) >> 5) % MAX_PAGES;
 
-			if(n < MAX_PAGES)
+			uint32& row = tmp[n >> 5];
+			uint32 col = 1 << (n & 31);
+
+			if((row & col) == 0)
 			{
-				uint32& row = tmp[n >> 5];
-				uint32 col = 1 << (n & 31);
+				row |= col;
 
-				if((row & col) == 0)
-				{
-					row |= col;
-
-					*p++ = n;
-				}
+				*p++ = n;
 			}
 		}
 	}
@@ -2083,24 +2143,42 @@ uint32* GSOffset::GetPages(const GSVector4i& rect, uint32* pages, GSVector4i* bb
 	return pages;
 }
 
-GSVector4i* GSOffset::GetPagesAsBits(const GSVector4i& rect, GSVector4i* pages, GSVector4i* bbox)
+uint32* GSOffset::GetPagesAsBits(const GIFRegTEX0& TEX0)
 {
-	if(pages == NULL)
-	{
-		pages = (GSVector4i*)_aligned_malloc(sizeof(GSVector4i) * 4, 16);
-	}
+	// Performance note:
+	// GSOffset is per bp/bw/psm
+	// Pages coverage depends also on TW and Th (8bits). Therefore we will save them as a small array.
+	// It is faster than a hash cache and it reduces the GetPagesAsBits overhead.
 
-	pages[0] = GSVector4i::zero();
-	pages[1] = GSVector4i::zero();
-	pages[2] = GSVector4i::zero();
-	pages[3] = GSVector4i::zero();
+	int hash_key = (TEX0.u64 >> 26) & 0xFF;
+	uint32* pages = pages_as_bit[hash_key];
+
+	if (pages)
+		return pages;
+
+	// Aligned on 64 bytes to store the full bitmap in a single cache line
+	pages = (uint32*)_aligned_malloc(MAX_PAGES/8, 64);
+	pages_as_bit[hash_key] = pages;
+
+	GetPagesAsBits(GSVector4i(0, 0, 1 << TEX0.TW, 1 << TEX0.TH), pages);
+
+	return pages;
+}
+
+
+void* GSOffset::GetPagesAsBits(const GSVector4i& rect, void* pages)
+{
+	ASSERT(pages != nullptr);
+
+	((GSVector4i*)pages)[0] = GSVector4i::zero();
+	((GSVector4i*)pages)[1] = GSVector4i::zero();
+	((GSVector4i*)pages)[2] = GSVector4i::zero();
+	((GSVector4i*)pages)[3] = GSVector4i::zero();
 
 	GSVector2i bs = (bp & 31) == 0 ? GSLocalMemory::m_psm[psm].pgs : GSLocalMemory::m_psm[psm].bs;
 
 	GSVector4i r = rect.ralign<Align_Outside>(bs);
 
-	if(bbox != NULL) *bbox = r;
-
 	r = r.sra32(3);
 
 	bs.x >>= 3;
@@ -2112,15 +2190,11 @@ GSVector4i* GSOffset::GetPagesAsBits(const GSVector4i& rect, GSVector4i* pages,
 
 		for(int x = r.left; x < r.right; x += bs.x)
 		{
-			uint32 n = (base + block.col[x]) >> 5;
+			uint32 n = ((base + block.col[x]) >> 5) % MAX_PAGES;
 
-			if(n < MAX_PAGES)
-			{
-				((uint32*)pages)[n >> 5] |= 1 << (n & 31);
-			}
+			((uint32*)pages)[n >> 5] |= 1 << (n & 31);
 		}
 	}
 
 	return pages;
-
-}
\ No newline at end of file
+}
