@@ -1,9 +1,9 @@
 /* Copyright (c) 2007 MITSUNARI Shigeo
 * All rights reserved.
-* 
+*
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
-* 
+*
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
@@ -12,7 +12,7 @@
 * Neither the name of the copyright owner nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
-* 
+*
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -37,7 +37,7 @@
 */
 #include "xbyak.h"
 
-#ifdef _WIN32
+#ifdef _MSC_VER
 	#if (_MSC_VER < 1400) && defined(XBYAK32)
 		static inline __declspec(naked) void __cpuid(int[4], int)
 		{
@@ -82,11 +82,28 @@ extern "C" unsigned __int64 __xgetbv(int);
 
 namespace Xbyak { namespace util {
 
+/* GCC uses AVX/SSE4 operation to handle the uint64 type.
+ *
+ * It is quite annoying because the purpose of the code is to test the support
+ * of AVX/SSEn
+ *
+ * So far, we don't need other ISA on i386 so I hacked the code to limit the
+ * type to 32 bits. If we want to support AVX512 we might need to shuffle the
+ * code a bit.
+ *
+ * Extra note: it would be waste to use AVX512 on 32 bits, registers are
+ * limited to 8 instead of 32.
+ */
+
 /**
 	CPU detection class
 */
 class Cpu {
-	unsigned int type_;
+#ifdef XBYAK64
+	uint64 type_;
+#else
+	uint32 type_;
+#endif
 	unsigned int get32bitAsBE(const char *x) const
 	{
 		return x[0] | (x[1] << 8) | (x[2] << 16) | (x[3] << 24);
@@ -124,9 +141,12 @@ public:
 	int extFamily;
 	int displayFamily; // family + extFamily
 	int displayModel; // model + extModel
+	/*
+		data[] = { eax, ebx, ecx, edx }
+	*/
 	static inline void getCpuid(unsigned int eaxIn, unsigned int data[4])
 	{
-#ifdef _WIN32
+#ifdef _MSC_VER
 		__cpuid(reinterpret_cast<int*>(data), eaxIn);
 #else
 		__cpuid(eaxIn, data[0], data[1], data[2], data[3]);
@@ -134,7 +154,7 @@ public:
 	}
 	static inline void getCpuidEx(unsigned int eaxIn, unsigned int ecxIn, unsigned int data[4])
 	{
-#ifdef _WIN32
+#ifdef _MSC_VER
 		__cpuidex(reinterpret_cast<int*>(data), eaxIn, ecxIn);
 #else
 		__cpuid_count(eaxIn, ecxIn, data[0], data[1], data[2], data[3]);
@@ -152,42 +172,68 @@ public:
 		return ((uint64)edx << 32) | eax;
 #endif
 	}
-	enum Type {
-		NONE = 0,
-		tMMX = 1 << 0,
-		tMMX2 = 1 << 1,
-		tCMOV = 1 << 2,
-		tSSE = 1 << 3,
-		tSSE2 = 1 << 4,
-		tSSE3 = 1 << 5,
-		tSSSE3 = 1 << 6,
-		tSSE41 = 1 << 7,
-		tSSE42 = 1 << 8,
-		tPOPCNT = 1 << 9,
-		tAESNI = 1 << 10,
-		tSSE5 = 1 << 11,
-		tOSXSAVE = 1 << 12,
-		tPCLMULQDQ = 1 << 13,
-		tAVX = 1 << 14,
-		tFMA = 1 << 15,
+#ifdef XBYAK64
+	typedef uint64 Type;
+#else
+	typedef uint32 Type;
+#endif
+	static const Type NONE = 0;
+	static const Type tMMX = 1 << 0;
+	static const Type tMMX2 = 1 << 1;
+	static const Type tCMOV = 1 << 2;
+	static const Type tSSE = 1 << 3;
+	static const Type tSSE2 = 1 << 4;
+	static const Type tSSE3 = 1 << 5;
+	static const Type tSSSE3 = 1 << 6;
+	static const Type tSSE41 = 1 << 7;
+	static const Type tSSE42 = 1 << 8;
+	static const Type tPOPCNT = 1 << 9;
+	static const Type tAESNI = 1 << 10;
+	static const Type tSSE5 = 1 << 11;
+	static const Type tOSXSAVE = 1 << 12;
+	static const Type tPCLMULQDQ = 1 << 13;
+	static const Type tAVX = 1 << 14;
+	static const Type tFMA = 1 << 15;
 
-		t3DN = 1 << 16,
-		tE3DN = 1 << 17,
-		tSSE4a = 1 << 18,
-		tRDTSCP = 1 << 19,
-		tAVX2 = 1 << 20,
-		tGPR1 = 1 << 21, // andn, bextr, blsi, blsmk, blsr, tzcnt
-		tGPR2 = 1 << 22, // bzhi, mulx, pdep, pext, rorx, sarx, shlx, shrx
-		tLZCNT = 1 << 23,
+	static const Type t3DN = 1 << 16;
+	static const Type tE3DN = 1 << 17;
+	static const Type tSSE4a = 1 << 18;
+	static const Type tRDTSCP = 1 << 19;
+	static const Type tAVX2 = 1 << 20;
+	static const Type tBMI1 = 1 << 21; // andn, bextr, blsi, blsmsk, blsr, tzcnt
+	static const Type tBMI2 = 1 << 22; // bzhi, mulx, pdep, pext, rorx, sarx, shlx, shrx
+	static const Type tLZCNT = 1 << 23;
+
+	static const Type tINTEL = 1 << 24;
+	static const Type tAMD = 1 << 25;
+
+	static const Type tENHANCED_REP = 1 << 26; // enhanced rep movsb/stosb
+	static const Type tRDRAND = 1 << 27;
+	static const Type tADX = 1 << 28; // adcx, adox
+	static const Type tRDSEED = 1 << 29; // rdseed
+	static const Type tSMAP = 1 << 30; // stac
+#ifdef XBYAK64
+	static const Type tHLE = uint64(1) << 31; // xacquire, xrelease, xtest
+	static const Type tRTM = uint64(1) << 32; // xbegin, xend, xabort
+	static const Type tF16C = uint64(1) << 33; // vcvtph2ps, vcvtps2ph
+	static const Type tMOVBE = uint64(1) << 34; // mobve
+	static const Type tAVX512F = uint64(1) << 35;
+	static const Type tAVX512DQ = uint64(1) << 36;
+	static const Type tAVX512IFMA = uint64(1) << 37;
+	static const Type tAVX512PF = uint64(1) << 38;
+	static const Type tAVX512ER = uint64(1) << 39;
+	static const Type tAVX512CD = uint64(1) << 40;
+	static const Type tAVX512BW = uint64(1) << 41;
+	static const Type tAVX512VL = uint64(1) << 42;
+	static const Type tAVX512VBMI = uint64(1) << 43;
+#endif
 
-		tINTEL = 1 << 24,
-		tAMD = 1 << 25
-	};
 	Cpu()
 		: type_(NONE)
 	{
 		unsigned int data[4];
 		getCpuid(0, data);
+		const unsigned int maxNum = data[0];
 		static const char intel[] = "ntel";
 		static const char amd[] = "cAMD";
 		if (data[2] == get32bitAsBE(amd)) {
@@ -210,10 +256,15 @@ public:
 		if (data[2] & (1U << 9)) type_ |= tSSSE3;
 		if (data[2] & (1U << 19)) type_ |= tSSE41;
 		if (data[2] & (1U << 20)) type_ |= tSSE42;
+#ifdef XBYAK64
+		if (data[2] & (1U << 22)) type_ |= tMOVBE;
+		if (data[2] & (1U << 29)) type_ |= tF16C;
+#endif
 		if (data[2] & (1U << 23)) type_ |= tPOPCNT;
 		if (data[2] & (1U << 25)) type_ |= tAESNI;
 		if (data[2] & (1U << 1)) type_ |= tPCLMULQDQ;
 		if (data[2] & (1U << 27)) type_ |= tOSXSAVE;
+		if (data[2] & (1U << 30)) type_ |= tRDRAND;
 
 		if (data[3] & (1U << 15)) type_ |= tCMOV;
 		if (data[3] & (1U << 23)) type_ |= tMMX;
@@ -226,12 +277,39 @@ public:
 			if ((bv & 6) == 6) {
 				if (data[2] & (1U << 28)) type_ |= tAVX;
 				if (data[2] & (1U << 12)) type_ |= tFMA;
+#ifdef XBYAK64
+				if (((bv >> 5) & 7) == 7) {
+					getCpuid(7, data);
+					if (data[1] & (1U << 16)) type_ |= tAVX512F;
+					if (type_ & tAVX512F) {
+						getCpuidEx(7, 0, data);
+						if (data[1] & (1U << 17)) type_ |= tAVX512DQ;
+						if (data[1] & (1U << 21)) type_ |= tAVX512IFMA;
+						if (data[1] & (1U << 26)) type_ |= tAVX512PF;
+						if (data[1] & (1U << 27)) type_ |= tAVX512ER;
+						if (data[1] & (1U << 28)) type_ |= tAVX512CD;
+						if (data[1] & (1U << 30)) type_ |= tAVX512BW;
+						if (data[1] & (1U << 31)) type_ |= tAVX512VL;
+						if (data[2] & (1U << 1)) type_ |= tAVX512VBMI;
+					}
+				}
+#endif
 			}
 		}
-		getCpuidEx(7, 0, data);
-		if (type_ & tAVX && data[1] & 0x20) type_ |= tAVX2;
-		if (data[1] & (1U << 3)) type_ |= tGPR1;
-		if (data[1] & (1U << 8)) type_ |= tGPR2;
+		if (maxNum >= 7) {
+			getCpuidEx(7, 0, data);
+			if (type_ & tAVX && data[1] & 0x20) type_ |= tAVX2;
+			if (data[1] & (1U << 3)) type_ |= tBMI1;
+			if (data[1] & (1U << 8)) type_ |= tBMI2;
+			if (data[1] & (1U << 9)) type_ |= tENHANCED_REP;
+			if (data[1] & (1U << 18)) type_ |= tRDSEED;
+			if (data[1] & (1U << 19)) type_ |= tADX;
+			if (data[1] & (1U << 20)) type_ |= tSMAP;
+#ifdef XBYAK64
+			if (data[1] & (1U << 4)) type_ |= tHLE;
+			if (data[1] & (1U << 11)) type_ |= tRTM;
+#endif
+		}
 		setFamily();
 	}
 	void putFamily()
@@ -289,11 +367,12 @@ class Pack {
 	const Xbyak::Reg64 *tbl_[maxTblNum];
 	size_t n_;
 public:
-	Pack() : n_(0) {}
+	Pack() : tbl_(), n_(0) {}
 	Pack(const Xbyak::Reg64 *tbl, size_t n) { init(tbl, n); }
 	Pack(const Pack& rhs)
 		: n_(rhs.n_)
 	{
+		if (n_ > maxTblNum) throw Error(ERR_INTERNAL);
 		for (size_t i = 0; i < n_; i++) tbl_[i] = rhs.tbl_[i];
 	}
 	Pack(const Xbyak::Reg64& t0)
@@ -320,7 +399,7 @@ public:
 	{
 		if (n_ == 10) {
 			fprintf(stderr, "ERR Pack::can't append\n");
-			throw ERR_BAD_PARAMETER;
+			throw Error(ERR_BAD_PARAMETER);
 		}
 		tbl_[n_++] = &t;
 		return *this;
@@ -329,7 +408,7 @@ public:
 	{
 		if (n > maxTblNum) {
 			fprintf(stderr, "ERR Pack::init bad n=%d\n", (int)n);
-			throw ERR_BAD_PARAMETER;
+			throw Error(ERR_BAD_PARAMETER);
 		}
 		n_ = n;
 		for (size_t i = 0; i < n; i++) {
@@ -340,7 +419,7 @@ public:
 	{
 		if (n >= n_) {
 			fprintf(stderr, "ERR Pack bad n=%d\n", (int)n);
-			throw ERR_BAD_PARAMETER;
+			throw Error(ERR_BAD_PARAMETER);
 		}
 		return *tbl_[n];
 	}
@@ -353,7 +432,7 @@ public:
 		if (num == size_t(-1)) num = n_ - pos;
 		if (pos + num > n_) {
 			fprintf(stderr, "ERR Pack::sub bad pos=%d, num=%d\n", (int)pos, (int)num);
-			throw ERR_BAD_PARAMETER;
+			throw Error(ERR_BAD_PARAMETER);
 		}
 		Pack pack;
 		pack.n_ = num;
@@ -362,6 +441,13 @@ public:
 		}
 		return pack;
 	}
+	void put() const
+	{
+		for (size_t i = 0; i < n_; i++) {
+			printf("%s ", tbl_[i]->toString());
+		}
+		printf("\n");
+	}
 };
 
 class StackFrame {
@@ -420,27 +506,27 @@ public:
 		, t(t_)
 	{
 		using namespace Xbyak;
-		if (pNum < 0 || pNum > 4) throw ERR_BAD_PNUM;
+		if (pNum < 0 || pNum > 4) throw Error(ERR_BAD_PNUM);
 		const int allRegNum = pNum + tNum_ + (useRcx_ ? 1 : 0) + (useRdx_ ? 1 : 0);
-		if (allRegNum < pNum || allRegNum > 14) throw ERR_BAD_TNUM;
-		const Reg64& rsp = code->rsp;
-		const AddressFrame& ptr = code->ptr;
+		if (allRegNum < pNum || allRegNum > 14) throw Error(ERR_BAD_TNUM);
+		const Reg64& _rsp = code->rsp;
+		const AddressFrame& _ptr = code->ptr;
 		saveNum_ = (std::max)(0, allRegNum - noSaveNum);
 		const int *tbl = getOrderTbl() + noSaveNum;
 		P_ = saveNum_ + (stackSizeByte + 7) / 8;
 		if (P_ > 0 && (P_ & 1) == 0) P_++; // here (rsp % 16) == 8, then increment P_ for 16 byte alignment
 		P_ *= 8;
-		if (P_ > 0) code->sub(rsp, P_);
+		if (P_ > 0) code->sub(_rsp, P_);
 #ifdef XBYAK64_WIN
 		for (int i = 0; i < (std::min)(saveNum_, 4); i++) {
-			code->mov(ptr [rsp + P_ + (i + 1) * 8], Reg64(tbl[i]));
+			code->mov(_ptr [_rsp + P_ + (i + 1) * 8], Reg64(tbl[i]));
 		}
 		for (int i = 4; i < saveNum_; i++) {
-			code->mov(ptr [rsp + P_ - 8 * (saveNum_ - i)], Reg64(tbl[i]));
+			code->mov(_ptr [_rsp + P_ - 8 * (saveNum_ - i)], Reg64(tbl[i]));
 		}
 #else
 		for (int i = 0; i < saveNum_; i++) {
-			code->mov(ptr [rsp + P_ - 8 * (saveNum_ - i)], Reg64(tbl[i]));
+			code->mov(_ptr [_rsp + P_ - 8 * (saveNum_ - i)], Reg64(tbl[i]));
 		}
 #endif
 		int pos = 0;
@@ -462,22 +548,22 @@ public:
 	void close(bool callRet = true)
 	{
 		using namespace Xbyak;
-		const Reg64& rsp = code_->rsp;
-		const AddressFrame& ptr = code_->ptr;
+		const Reg64& _rsp = code_->rsp;
+		const AddressFrame& _ptr = code_->ptr;
 		const int *tbl = getOrderTbl() + noSaveNum;
 #ifdef XBYAK64_WIN
 		for (int i = 0; i < (std::min)(saveNum_, 4); i++) {
-			code_->mov(Reg64(tbl[i]), ptr [rsp + P_ + (i + 1) * 8]);
+			code_->mov(Reg64(tbl[i]), _ptr [_rsp + P_ + (i + 1) * 8]);
 		}
 		for (int i = 4; i < saveNum_; i++) {
-			code_->mov(Reg64(tbl[i]), ptr [rsp + P_ - 8 * (saveNum_ - i)]);
+			code_->mov(Reg64(tbl[i]), _ptr [_rsp + P_ - 8 * (saveNum_ - i)]);
 		}
 #else
 		for (int i = 0; i < saveNum_; i++) {
-			code_->mov(Reg64(tbl[i]), ptr [rsp + P_ - 8 * (saveNum_ - i)]);
+			code_->mov(Reg64(tbl[i]), _ptr [_rsp + P_ - 8 * (saveNum_ - i)]);
 		}
 #endif
-		if (P_ > 0) code_->add(rsp, P_);
+		if (P_ > 0) code_->add(_rsp, P_);
 
 		if (callRet) code_->ret();
 	}
@@ -486,8 +572,8 @@ public:
 		if (!makeEpilog_) return;
 		try {
 			close();
-		} catch (Xbyak::Error e) {
-			printf("ERR:StackFrame %s\n", ConvertErrorToString(e));
+		} catch (std::exception& e) {
+			printf("ERR:StackFrame %s\n", e.what());
 			exit(1);
 		} catch (...) {
 			printf("ERR:StackFrame otherwise\n");
