@@ -19,29 +19,27 @@ extern std::string s_strIniPath;
 
 void SaveConfig()
 {
-	const std::string iniFile = s_strIniPath + "/Padnull.ini";
+    const std::string iniFile = s_strIniPath + "/Padnull.ini";
 
-	PluginConf ini;
-	if (!ini.Open(iniFile, READ_FILE))
-	{
-		printf("failed to open %s\n", iniFile.c_str());
-		SaveConfig();//save and return
-		return;
-	}
-	conf.Log = ini.ReadInt("logging", 0);
-	ini.Close();
+    PluginConf ini;
+    if (!ini.Open(iniFile, READ_FILE)) {
+        printf("failed to open %s\n", iniFile.c_str());
+        SaveConfig();  //save and return
+        return;
+    }
+    conf.Log = ini.ReadInt("logging", 0);
+    ini.Close();
 }
 
 void LoadConfig()
 {
-	const std::string iniFile(s_strIniPath + "/Padnull.ini");
+    const std::string iniFile(s_strIniPath + "/Padnull.ini");
 
-	PluginConf ini;
-	if (!ini.Open(iniFile, WRITE_FILE))
-	{
-		printf("failed to open %s\n", iniFile.c_str());
-		return;
-	}
-	ini.WriteInt("logging", conf.Log);
-	ini.Close();
+    PluginConf ini;
+    if (!ini.Open(iniFile, WRITE_FILE)) {
+        printf("failed to open %s\n", iniFile.c_str());
+        return;
+    }
+    ini.WriteInt("logging", conf.Log);
+    ini.Close();
 }
